// ComTest.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include "Serial/Serial0.h"
#include "PIC/PICProgram.h"
#include "PIC/DeviceList.h"
#include "PIC/PIC18F/PIC18FXXXProgram.h"

#ifndef uint
#define uint unsigned int
#endif

CIOPort m_Port;
const PICDevice *device;

#define BAUD_RATE CBR_115200
// #define BAUD_RATE 38400

void
GetVersionTest()
{
	uint result;
	int i, j;

	for (i=0;i<32;i++)
	{
		j = PICProgram::TxCmd(&m_Port, PICProgram::EVA_VERSION_CMD, 0, &result);
		if (j >= 0)
			printf("V: %02x:%02x\n", ((result & 0xff00) >> 8), (result & 0xff));
		else
			printf("Failed\n");
	}
}

// Send packets of various sizes to the programmer and look at their contents after
// they are echoed back.
void
EchoTest1()
{
	uint resLen;
	int i, j;
	uint i1, q;
	byte outBuf[128], inBuf[128];
	memset(outBuf, 0, sizeof(outBuf)/sizeof(outBuf[0]));
	int ctr = 0;
	int failCount = 0;

	for (i1=4;i1<=64;i1+=4)
	{
		ctr = 0;
		// for (i=0;i<16;i++)
		for (i=0;i<1;i++)
		{
			for (q=0;q<i1;q++)
				outBuf[q] = ctr++;
			j = PICProgram::SendPacket(&m_Port, PICProgram::EVA_ECHO_CMD, i1, outBuf, 128, &resLen, inBuf);
			if (j >= 0)
			{
				printf("E: ");
				for (q=0;q<resLen;q++)
					printf("%02x ", inBuf[q]);
				printf("\n");
			}
			else
			{
				failCount++;
				printf("Failed\n");
				m_Port.Purge();
			}
		}
	}
}

// Send packets to the programmer and have it echo them back.  Good test of throughput
// on the serial port.  Overhead ranges from 100% for 4 byte packets to about 10% for
// the 64 byte maximum size packets.
void
EchoTest2()
{
	uint resLen;
	int i, i1, j, q;
	byte outBuf[128], inBuf[128];
	memset(outBuf, 0, sizeof(outBuf)/sizeof(outBuf[0]));
	int ctr = 0;
	int failCount = 0;
	int charCount;

	for (i1=4;i1<=64;i1*=2)
	{
		DWORD startTime = timeGetTime();
		// i1 = # of entries in the packet.  MUST be 64 or less or will crash the programmer.
		charCount = 0;
		for (i=0;i<256;i++)
		{
			int ctr0 = ctr;
			for (q=0;q<i1;q++)
				outBuf[q] = ctr++;
			j = PICProgram::SendPacket(&m_Port, PICProgram::EVA_ECHO_CMD, i1, outBuf, 128, &resLen, inBuf);
			charCount += (i1 + 5); // length of data + 2 STX + Data len + Checksum + ETX
			if (j >= 0 && resLen == i1+3)
				charCount += (i1 + 5);
			else
			{
				failCount++;
			}
			// Double check that the programmer sent back exactly what we sent to it.
			for (int t0=0;t0<i1;t0++)
				if (inBuf[t0+2] != outBuf[t0])
				{
					printf("*");
					break;
				}
		}
		DWORD endTime = timeGetTime();
		printf("Failures: %d\n", failCount);
		double elapsedTime = (endTime - startTime) / 1000.0;
		printf("Size %d, %5.2f sec, %d total bytes, (approx %d Baud, %d w/o overhead)\n",
			i1, elapsedTime, charCount,
			(int)(10.0 * charCount / elapsedTime),
			(int)(10.0 * i1 * 2 * 256 / elapsedTime));
	}
}

void
ReadCodeMemory()
{
	byte outBuf[4], inBuf[128];
	memset(outBuf, 0, sizeof(outBuf)/sizeof(outBuf[0]));

	const PIC18FXXXProgram *driver = (PIC18FXXXProgram *)PIC18FXXXProgram::GetDriver();

	PICProgram::PICOpen(&m_Port, device->devVppMask);
//	::Sleep(150);

	if (!driver->retryOpen(device, &m_Port))
	{
		// Issue a warning that we aren't getting anything from the chip
		printf("Failed to open chip\n");
		return;
	}


	// Read code, a block of words at a time
	uint resLen, retryCount = 0;
	uint val = 0;
	byte ctr = 0;
	uint addr, block, wordVal, j;
	uint numWords = 2048; // 1024;
	uint blockSize = 64;
	for (addr=0,block=0;block<numWords;block+=blockSize/2)
	{
		uint byteAddr = addr * 2;
		outBuf[0] = byteAddr & 0xff;
		outBuf[1] = (byteAddr & 0xff00) >> 8;
		outBuf[2] = (byteAddr & 0xff0000) >> 16;
		outBuf[3] = blockSize;
		j = PICProgram::SendPacket(&m_Port, PICProgram::EVA_PIC16RB_CMD,  4, outBuf, 128, &resLen, inBuf);
		if (j < 0)
			retryCount++;
		else
		{
			// Print out the info...
			for (uint i0=0;i0<blockSize;i0+=2)
			{
				if (!(addr % 8))
					printf("\n%04x: ", addr);
				// wordVal = ((inBuf[i0+7] & 0x7f) << 7) | (inBuf[i0+6] >> 1);
				wordVal = (inBuf[i0+7] << 8) | (inBuf[i0+6]);
				printf("%04x ", wordVal);
				addr++;
			}
		}

		// Check for any errors
		// if (i) { PICProgram::PICClose(port); return addr+1; }

		// Save the contents of the address.  Note that the response contains the
		// start and stop bit from the PIC.  To get the final 14 bit contents
		// we need to shift and mask the extra bits.
	}

	PICProgram::PICClose(&m_Port);
}

// Read code memory from a 14 bit PIC, one a block at a time
void
ReadCodeMemory2()
{
	byte outBuf[4], inBuf[128];
	memset(outBuf, 0, sizeof(outBuf)/sizeof(outBuf[0]));

	PICProgram::PICOpen(&m_Port);
//	::Sleep(150);

	// Read code, a block of words at a time
	uint resLen, retryCount = 0;
	uint val = 0;
	byte ctr = 0;
	uint addr, block, wordVal, j;
	uint numWords = 2048; // 1024;
	uint blockSize = 64;
	for (addr=0,block=0;block<numWords;block+=blockSize/2)
	{
		outBuf[0] = addr & 0xff;
		outBuf[1] = (addr & 0xff00) >> 8;
		outBuf[2] = (addr & 0xff0000) >> 8;
		outBuf[3] = blockSize;
		j = PICProgram::SendPacket(&m_Port, PICProgram::EVA_PIC14RB_CMD,  4, outBuf, 128, &resLen, inBuf);
		if (j < 0)
			retryCount++;
		else
		{
			// Print out the info...
			for (uint i0=0;i0<blockSize;i0+=2)
			{
				if (!(addr % 8))
					printf("\n%04x: ", addr);
				// wordVal = ((inBuf[i0+7] & 0x7f) << 7) | (inBuf[i0+6] >> 1);
				wordVal = (inBuf[i0+7] << 8) | (inBuf[i0+6]);
				printf("%04x ", wordVal);
				addr++;
			}
		}

		// Check for any errors
		// if (i) { PICProgram::PICClose(port); return addr+1; }

		// Save the contents of the address.  Note that the response contains the
		// start and stop bit from the PIC.  To get the final 14 bit contents
		// we need to shift and mask the extra bits.
	}

	PICProgram::PICClose(&m_Port);
}

int _tmain(int argc, _TCHAR* argv[])
{
	CString portName(_T("COM1:"));
	if (argc > 1)
	{
		portName = argv[1];
	}
	if (!SUCCEEDED(m_Port.Open((LPCTSTR)portName, BAUD_RATE)))
	{
		printf("Failed to open port '%s'\n", (LPCTSTR)portName);
		return 1;
	}

	int devIndex = DeviceList::DeviceIndex("18f2455");
	if (devIndex < 0)
	{
		printf("Failed to identify device\n");
		return 1;
	}
	device = DeviceList::Devices[devIndex];

	CIOPort *port = &m_Port;
	int retryCount = 0;

#if 0
	GetVersionTest();
#endif

#if 1
#if 1
	// Read memory from a 16 bit PIC
	ReadCodeMemory();
#else
	// Read memory from a 14 bit PIC
	ReadCodeMemory2();
#endif
#endif

#if 1
	EchoTest1();
#endif

#if 2
	EchoTest2();
#endif

#if 1
	uint result;
	int res = PICProgram::TxCmd(&m_Port, PICProgram::EVA_VERSION_CMD, 0, &result);
	if (res >= 0)
	{
		int versionMinor = (result & 0xff);
		int versionMajor = ((result & 0xff00) >> 8);
		printf("Firmware Version: %02x:%02x\n", versionMajor, versionMinor);
		res++;
	}
	else
	{
		printf("Failed to get firmware version\n");
		res--;
	}
#endif

#if 0
	PICProgram::PICOpen(port);

	// Read code, one word at a time
	uint val = 0;
	byte ctr = 0;
	uint addr, wordVal, j;
	uint numWords = 1024;
	for (addr=0;addr<numWords;addr++)
	{
		if (!(addr % 8))
			printf("\n%04x: ", addr+1);
		val = (ctr++ << 8); val |= ctr++;
		retryCount += PICProgram::TxCmd(port, PICProgram::EVA_PIC14R_CMD, PICProgram::PIC14_READDATA | 0xffff, &wordVal);
		val = (ctr++ << 8); val |= ctr++;
		retryCount += PICProgram::TxCmd(port, PICProgram::EVA_PIC14_CMD, PICProgram::PIC14_INCADDR | 0xffff, &j);

		// Check for any errors
		// if (i) { PICProgram::PICClose(port); return addr+1; }

		// Save the contents of the address.  Note that the response contains the
		// start and stop bit from the PIC.  To get the final 14 bit contents
		// we need to shift and mask the extra bits.
		int word = ((wordVal >> 1) & 0x3fff);
		printf("%04x ", word);
	}

	PICProgram::PICClose(port);
#endif

	m_Port.Close();

	printf("\nRetries: %d\n", retryCount);

	return 0;
}

