// FileOpenDlg1.cpp : implementation file
//

#include "stdafx.h"
#include "DateTimeFormat.h"
#include "FileOpenDlg.h"
#include "mainfrm.h"

#include <list>

int CALLBACK SortNameAsc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);
int CALLBACK SortNameDes(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);

int CALLBACK SortSizeAsc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);
int CALLBACK SortSizeDes(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);

int CALLBACK SortDateAsc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);
int CALLBACK SortDateDes(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);

/////////////////////////////////////////////////////////////////////////////
// CFileOpenDlg dialog

void CFileOpenDlg::initVars(
		BOOL bOpenFileDialog,
		LPCTSTR lpszDefExt,
		LPCTSTR lpszFileName,
		DWORD dwFlags,
		LPCTSTR lpszFilter)
{
	m_openFileFlag = bOpenFileDialog;
	m_bOk = FALSE;
	m_bTracking = FALSE;
	m_ySplit = 152;
	m_nSort = 1;

	TCHAR		szFmt[65];

	//{{AFX_DATA_INIT(CFileOpenDlg)
	//}}AFX_DATA_INIT

	// TBD: Set caption/title for this dialog
	// SetTitle(_T("File Open"));

	if (lpszDefExt == NULL)
		m_strFilter = _T("*");

	m_nState	= vsStandard;

	// Starting file name
	m_FileName = CString(lpszFileName != NULL ? lpszFileName : _T(""));

	// Default extension
	m_FileExt = CString(lpszDefExt != NULL ? lpszDefExt : _T(""));

	// Parse filter string into names/exts
	if (lpszFilter != NULL)
	{
		WCHAR *filterString = new WCHAR[::wcslen(lpszFilter)+1];
		wcscpy(filterString, lpszFilter);

		// Collect the pieces into a single list
		std::list<CString> filterEntries;
		WCHAR *token = wcstok(filterString, _T("|"));
		while (token != NULL)
		{
			filterEntries.push_back(CString(token));
			token = wcstok(NULL, _T("|"));
		}

		m_nFilterCount = filterEntries.size() / 2;
		m_FilterTitles = new CString[m_nFilterCount];
		m_FilterStrings = new CString[m_nFilterCount];
		std::list<CString>::iterator lItr = filterEntries.begin();
		int fIndex = 0;
		while (lItr != filterEntries.end())
		{
			m_FilterTitles[fIndex] = *lItr++;
			m_FilterStrings[fIndex] = *lItr++;

			fIndex++;
		}

		delete[] filterString;
	}
	else
	{
		m_nFilterCount = 1;
		m_FilterTitles = new CString[1];
		m_FilterTitles[0] = _T("All Files");
		m_FilterStrings = new CString[1];
		m_FilterStrings[0] = _T("*.*");
	}

	m_nFilterIndex = 0;

	m_dwFlags = dwFlags;

	m_strPath = getPreviousDir();

	//
	// Get the locale date format
	//
	GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SSHORTDATE, szFmt, 64);
	m_strDateFormat = szFmt;
}

CFileOpenDlg::CFileOpenDlg(
		BOOL bOpenFileDialog,
		LPCTSTR lpszDefExt,
		LPCTSTR lpszFileName,
		DWORD dwFlags,
		LPCTSTR lpszFilter,
		CWindow* pParent)
{
	initVars(bOpenFileDialog, lpszDefExt, lpszFileName, dwFlags, lpszFilter);
}


CFileOpenDlg::~CFileOpenDlg()
{

}

LRESULT CFileOpenDlg::OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    // m_CmdBar is of type CCommandBarCtrl, defined in AtlCtrlw.h
    HWND hWndCmdBar = m_pCmdBar.Create(m_hWnd, IDR_FILEOPEN);

	return 0;
}

//---------------------------------------------------------------------------
//
//	CFileOpenDlg virtual functions
//
//---------------------------------------------------------------------------


// CFileOpenDlg::DoPaint
//
//		Paints the splitter if it is displayed
//
void CFileOpenDlg::DoPaint(CPaintDC& dc) 
{
	if(m_nState == vsStandard)
	{
		RECT	rcClient;
		int		y;

		GetClientRect(&rcClient);

		y = m_ySplit;

		dc.MoveTo(0, y);
		dc.LineTo(rcClient.right, y);

		y += 6;		// This is the splitter height

		dc.MoveTo(0, y);
		dc.LineTo(rcClient.right, y);
	}
}


//---------------------------------------------------------------------------
//
//	CFileOpenDlg utility functions
//
//---------------------------------------------------------------------------


// CFileOpenDlg::DoSortList
//
//		Sorts the list according to the specified criterion
//
void CFileOpenDlg::DoSortList()
{
	switch(m_nSort)
	{
	case 1:
		m_listFile.SortItems(SortNameAsc, (DWORD)&m_cont);
		break;

	case -1:
		m_listFile.SortItems(SortNameDes, (DWORD)&m_cont);
		break;

	case 2:
		m_listFile.SortItems(SortSizeAsc, (DWORD)&m_cont);
		break;

	case -2:
		m_listFile.SortItems(SortSizeDes, (DWORD)&m_cont);
		break;

	case 3:
		m_listFile.SortItems(SortDateAsc, (DWORD)&m_cont);
		break;

	case -3:
		m_listFile.SortItems(SortDateDes, (DWORD)&m_cont);
		break;
	}
}


// CFileOpenDlg::SetColumnIcons
//
//		Correctly sets the columns sort icons
//
void CFileOpenDlg::SetColumnIcons()
{
	CHeaderCtrl pHeader = m_listFile.GetHeader();

	if(pHeader)
	{
		HDITEM hdi;
		int i;
		int nItems	= pHeader.GetItemCount();

		for(i = 0; i < nItems; ++i)
		{
			hdi.mask = HDI_IMAGE | HDI_FORMAT;
			pHeader.GetItem(i, &hdi);

			if(i+1 == m_nSort)
			{
				hdi.iImage = 1;
				hdi.fmt |= HDF_IMAGE | HDF_BITMAP_ON_RIGHT;
			}
			else if(i+1 == -m_nSort)
			{
				hdi.iImage = 2;
				hdi.fmt |= HDF_IMAGE | HDF_BITMAP_ON_RIGHT;
			}
			else
			{
				hdi.iImage = 0;
				hdi.fmt &= ~(HDF_IMAGE | HDF_BITMAP_ON_RIGHT);
			}
			pHeader.SetItem(i, &hdi);
		}
	}
}


// CFileOpenDlg::EnumDirs
//
//		Enumerates all subdirectories. Recurses depth-first 
//
int CFileOpenDlg::EnumDirs(LPCTSTR pszPath, LPCTSTR pszFilter, HTREEITEM hItemParent)
{
	WIN32_FIND_DATA	fd;
	HANDLE			hFind;
	BOOL			bFind;
	HTREEITEM		hItem = TVI_LAST;
	CString		strSearch(pszPath),
					strBase(pszPath);
	int				nCount	= 0;

	strSearch += pszFilter;

	hFind = FindFirstFile(strSearch, &fd);
	bFind = (hFind != INVALID_HANDLE_VALUE);

	while(bFind)
	{
		if((fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) &&
		   !(fd.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN))
		{
			TVINSERTSTRUCT tvi;
			CString strSub(strBase);
			int iIcon;

			strSub += fd.cFileName;
			iIcon = GetIconIndex(strSub);

			tvi.hParent				= hItemParent;
			tvi.hInsertAfter		= TVI_LAST;
			tvi.item.mask			= TVIF_CHILDREN | TVIF_IMAGE | TVIF_SELECTEDIMAGE |
									  TVIF_TEXT | TVIF_PARAM;
			tvi.item.pszText		= fd.cFileName;
			tvi.item.iImage			= iIcon;
			tvi.item.iSelectedImage	= iIcon;
			tvi.item.cChildren		= I_CHILDRENCALLBACK;
			tvi.item.lParam			= -1;

			hItem = m_treeFolder.InsertItem(&tvi);

			++nCount;
/*
			strSub += _T("\\");
			if(hItem)
			{
				m_treeFolder.SetItemData(hItem, 1);

				EnumDirs(strSub, pszFilter, hItem);
			}*/
		}

		bFind = FindNextFile(hFind, &fd);
	}
	return nCount;
}


// CFileOpenDlg::EnumFiles
//
//		Enumerates all files in a folder
//
void CFileOpenDlg::EnumFiles(LPCTSTR pszPath, LPCTSTR pszFilter)
{
	WIN32_FIND_DATA	fd;
	HANDLE			hFind;
	BOOL			bFind;
	CString		strSearch(pszPath),
					strFile;
	int				i = 0,
					iItem;
	FILETIME		odt;
	CDateTimeFormat	dtf;

	//
	// Flush the list
	//
	m_listFile.DeleteAllItems();
	m_cont.clear();

	//
	// Fill in the list
	//
	strSearch += pszFilter;

	hFind = FindFirstFile(strSearch, &fd);
	bFind = (hFind != INVALID_HANDLE_VALUE);

	//
	// Disable painting the list
	//
	m_listFile.SetRedraw(FALSE);

	while(bFind)
	{
		if(!(fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) && 
		   !(fd.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN))
		{
			strFile = pszPath;
			strFile += fd.cFileName;
			iItem = m_listFile.InsertItem(i, fd.cFileName, GetIconIndex(strFile));
			if(iItem != -1)
			{
				//
				// Format size according to locale
				//
				m_listFile.SetItemText(iItem, 1, FormatSize(fd.nFileSizeLow));

				//
				// Format date according to locale
				//
				odt = fd.ftLastWriteTime;
				dtf.SetDateTime(odt);
				dtf.SetFormat(m_strDateFormat);

				m_listFile.SetItemText(iItem, 2, dtf.GetString());
				m_listFile.SetItemData(iItem, i);

				m_cont.push_back(fd);
			}

			i++;
		}

		bFind = FindNextFile(hFind, &fd);
	}
	DoSortList();

	//
	// Enable painting the list
	//
	m_listFile.SetRedraw(TRUE);

}


// CFileOpenDlg::GetIconIndex
//
//		Retrieves the file's icon index
//		Code contributed by Amit Dey.
//
int CFileOpenDlg::GetIconIndex(LPCTSTR pszFileName)
{
	SHFILEINFO	ssfi;

	SHGetFileInfo(pszFileName, 0, &ssfi, sizeof(SHFILEINFO),
		SHGFI_SYSICONINDEX | SHGFI_SMALLICON);

	return ssfi.iIcon;
}


// CFileOpenDlg::GetFullPath
//
//		Returns the full path of a node
//
CString CFileOpenDlg::GetFullPath(HTREEITEM hItem)
{
	CString	strFullPath, strFolder, strSep(_T("\\"));
	
	if(hItem)
	{
		while(hItem)
		{
			TCHAR buf[257];
			if (SUCCEEDED(m_treeFolder.GetItemText(hItem, buf, 256)))
			{
				strFolder = buf;
				if(strFolder != strSep)
					strFullPath = strFolder + strSep + strFullPath;

				hItem = m_treeFolder.GetParentItem(hItem);
			}
		}
		strFullPath = strSep + strFullPath;
	}

	return strFullPath;
}


// CFileOpenDlg::FormatSize
//
//		Returns a string with the appropriate formatting
//
CString CFileOpenDlg::FormatSize(DWORD dwSize)
{
	CString		strFmt,
					strLet;

	if(dwSize > (1024 * 1024))
	{
		dwSize /= 1024 * 1024;
		strLet += _T("M");
	}
	else if(dwSize > 1024)
	{
		dwSize /= 1024;
		strLet += _T("K");
	}
	TCHAR buf[32];
	_stprintf(buf, _T("%d"), (int)dwSize);
	strFmt = buf + strLet;

	return strFmt;
}


// CFileOpenDlg::GetSystemName
//
//		Returns the system name
//
CString CFileOpenDlg::GetSystemName()
{
	HKEY		hKey;
	LONG		nRval;
	DWORD		dwSize = 64,
				dwType;
	CString	strSysName;
	TCHAR		szName[65];

	//
	// Get the system name
	//
	nRval = RegOpenKeyEx(HKEY_LOCAL_MACHINE, _T("\\Ident"), 0, 0, &hKey);
	if(ERROR_SUCCESS == nRval)
	{
		nRval = RegQueryValueEx(hKey, _T("Name"), NULL, &dwType, 
			(LPBYTE) szName, &dwSize);

		if(ERROR_SUCCESS == nRval)
			strSysName = szName;

		RegCloseKey(hKey);
	}
	
	//
	// On any error, use the default name
	//
	if(ERROR_SUCCESS != nRval)
		strSysName = _T("My Device");

	return strSysName;
}


// CFileOpenDlg::UpdateControls
//
//		Updates the control states
//
void CFileOpenDlg::UpdateControls()
{
	ASSERT(m_pCmdBar);

#if 1
	m_pCmdBar.CheckDlgButton(ID_VIEW_STANDARD,	m_nState == vsStandard);
	m_pCmdBar.CheckDlgButton(ID_VIEW_TREE,		m_nState == vsTreeOnly);
	m_pCmdBar.CheckDlgButton(ID_VIEW_LIST,		m_nState == vsListOnly);
	// m_pCmdBar.EnableButton(ID_BAR_OK, m_bOk);
#else
	CToolBarCtrl &tb = m_pCmdBar.GetToolBar();
	tb.CheckButton(ID_VIEW_STANDARD,	m_nState == vsStandard);
	tb.CheckButton(ID_VIEW_TREE,		m_nState == vsTreeOnly);
	tb.CheckButton(ID_VIEW_LIST,		m_nState == vsListOnly);

	tb.EnableButton(ID_BAR_OK, m_bOk);
#endif
}


//---------------------------------------------------------------------------
//
//	CFileOpenDlg message handlers
//
//---------------------------------------------------------------------------


void CFileOpenDlg::resizeDialog(UINT nType, int cx, int cy)
{
	if (nType == SIZE_MINIMIZED)
	{
		// Don't do anything if the dialog is being hidden
		return;
	}
	int availWidth = cx;

	bool mHighResDisplay = (cx > 320 || cy > 320);	// If high res, then use high res font

	// Determine the size of the group box.  We need it to be big enough that
	// the text of the groupbox is unobscured and the entire combobox fits
	// inside it.
	RECT rect;
	m_filetypeCombo.GetWindowRect(&rect);
	int comboHeight = rect.bottom - rect.top;
	int comboWidth  = rect.right - rect.left;

#if 1
	// TBD: Figure out why GetLogFont() is returning a success value, but
	// garbage data.
	int charHeight = 16;
#else
	// Get the size of the font in the group box
	CFont grpFnt = m_filetypeGroupBox.GetFont();
	LOGFONT lf;
	int charHeight = 16;
	if (grpFnt.GetLogFont(&lf) != 0)
	{
		charHeight = 4 + -lf.lfHeight;
	}
#endif

	int innerPadding = 6;
	int filetypePad = 3;
	// Make the group box big enough for it's own text and the inner combobox
	int groupBoxHeight = charHeight + comboHeight + 2 * filetypePad + 6;
	m_filetypeGroupBox.MoveWindow(0, 0, cx, groupBoxHeight);

	int midPoint = cx / 2;

	if (m_openFileFlag)
	{
		// Put the file extension combo box centered in the group box
		// rect.top = GetHeaderHeight() + 1;
		int offset = midPoint - (comboWidth / 2);
		rect.top = charHeight + filetypePad;
		rect.left = offset;
		rect.bottom = rect.top + comboHeight;
		rect.right  = rect.left + comboWidth;
		m_filetypeCombo.MoveWindow(&rect);
	}
	else
	{
		// Put the file extension combo box on the left side of the group box
		// and the file name edit box on the right
		rect.top = charHeight + filetypePad;
		rect.left = innerPadding;
		rect.bottom = rect.top + comboHeight;
		rect.right  = rect.left + comboWidth;
		m_filetypeCombo.MoveWindow(&rect);

		rect.top = charHeight + filetypePad;
		rect.left = midPoint + filetypePad;
		rect.bottom = rect.top + comboHeight;
		rect.right  = rect.left + comboWidth;
		m_editFilename.MoveWindow(&rect);
	}

	int availHeight = cy - (groupBoxHeight);

	// Default size of tree is 2/3's of available space (after addign combo box)
	int treeHeight = 2 * availHeight / 3;

	// Put the splitter right below the tree
	m_ySplit = groupBoxHeight + treeHeight;

	// Remaining height available for the list box
	int listHeight = availHeight - (m_ySplit + nTrackHeight);

	// Position the tree.
	m_rcTree.left = 0;
	m_rcTree.right = availWidth;
	m_rcTree.top   = groupBoxHeight;
	m_rcTree.bottom = m_ySplit;
	m_treeFolder.MoveWindow(&m_rcTree);

	// Position the list
	m_rcList.left = 0;
	m_rcList.right = availWidth;
	m_rcList.top = m_ySplit + nTrackHeight;
	m_rcList.bottom = cy;
	m_listFile.MoveWindow(&m_rcList);
}

// CFileOpenDlg::OnInitDialog
//
//		Initialize the dialog
//
LRESULT CFileOpenDlg::OnInitDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	// Size it to fit the screen
	int cx = ::GetSystemMetrics(SM_CXSCREEN);
	int cy = ::GetSystemMetrics(SM_CYSCREEN);
	// SetWindowPos(NULL, 0, 0, cx, cy, SWP_NOZORDER);

    HWND hWndCmdBar = m_pCmdBar.Create(m_hWnd, IDR_FILEOPEN);

	// Connect dialog items to member variables
	m_editFilename.Attach(GetDlgItem(IDC_EDIT_FILENAME));
	m_filetypeGroupBox.Attach(GetDlgItem(IDC_FILETYPE_GROUPBOX));
	m_filetypeCombo.Attach(GetDlgItem(IDC_FILETYPE_COMBO));
	m_treeFolder.Attach(GetDlgItem(IDC_FOLDER_TREE));
	m_listFile.Attach(GetDlgItem(IDC_FILE_LIST));

	AddToolButtons();

	HTREEITEM		hItemRoot;
	HIMAGELIST		hImlSys;
	SHFILEINFO		ssfi;
	TCHAR			szRoot[]	= _T("\\");

	// If this is an open file dialog, then don't show the edit box
	// for specifying an output file.
	if (m_openFileFlag)
	{
		m_editFilename.ShowWindow(SW_HIDE);
	}

	// Size the elements of the dialog
	RECT rect;
	GetClientRect(&rect);
	resizeDialog(SIZE_RESTORED, rect.right-rect.left, rect.bottom-rect.top);

	//
	// Hide the OK button
	//
	ModifyStyle(0, WS_NONAVDONEBUTTON, SWP_NOSIZE); 
	SHDoneButton(m_hWnd, SHDB_HIDE);

	//
	// Retrieve the small system image list
	//
	hImlSys = (HIMAGELIST)SHGetFileInfo(_T("dummy.txt"), 0, &ssfi,
				sizeof(SHFILEINFO), 
				SHGFI_USEFILEATTRIBUTES | SHGFI_SYSICONINDEX | SHGFI_SMALLICON);
	
	m_imlSystem.Attach(hImlSys);
	m_listFile.SetImageList(m_imlSystem, LVSIL_SMALL);
	m_treeFolder.SetImageList(m_imlSystem, TVSIL_NORMAL);
	m_imlSystem.Detach();

	// Initialize the filter selection combo box
	for (int fInd=0;fInd<m_nFilterCount;fInd++)
	{
		int cind = m_filetypeCombo.AddString(m_FilterTitles[fInd]);
	}

	// TBD: select filter index based on default string passed to constructor
	m_nFilterIndex = 0;
	m_filetypeCombo.SetCurSel(m_nFilterIndex);
	m_strFilter = m_FilterStrings[0];

	m_editFilename.SetParent(m_filetypeGroupBox);
	m_filetypeCombo.SetParent(m_filetypeGroupBox);

	//
	// Initialize the tree
	//
	hItemRoot = TVI_ROOT;//m_treeFolder.InsertItem(GetSystemName(), 0, 1);
	if(hItemRoot)
	{
		TVINSERTSTRUCT	tvi;
		int				iIcon		= GetIconIndex(szRoot);

		tvi.hParent				= TVI_ROOT;
		tvi.hInsertAfter		= TVI_LAST;
		tvi.item.mask			= TVIF_CHILDREN | TVIF_IMAGE | TVIF_SELECTEDIMAGE |
								  TVIF_TEXT | TVIF_PARAM;
		tvi.item.pszText		= szRoot;
		tvi.item.iImage			= iIcon;
		tvi.item.iSelectedImage	= iIcon;
		tvi.item.cChildren		= I_CHILDRENCALLBACK;
		tvi.item.lParam			= -1;

		m_treeFolder.InsertItem(&tvi);

		//m_treeFolder.SetItemData(hItemRoot, 0);

		//EnumDirs(_T("\\"), _T("*"), hItemRoot);
		//m_treeFolder.Expand(hItemRoot, TVE_EXPAND);
	}

	//
	// Initialize the list (root level)
	//
	if (GetSystemMetrics(SM_CXSCREEN) > 300 && GetSystemMetrics(SM_CYSCREEN) > 300)
	{
		m_listFile.InsertColumn(0, _T("Name"), LVCFMT_LEFT,  200, -1);
		m_listFile.InsertColumn(1, _T("Size"), LVCFMT_RIGHT, 120,  1);
		m_listFile.InsertColumn(2, _T("Date"), LVCFMT_LEFT,  140,  2);
	}
	else
	{
		m_listFile.InsertColumn(0, _T("Name"), LVCFMT_LEFT,  100, -1);
		m_listFile.InsertColumn(1, _T("Size"), LVCFMT_RIGHT, 60,  1);
		m_listFile.InsertColumn(2, _T("Date"), LVCFMT_LEFT,  60,  2);
	}
	EnumFiles(szRoot, m_strFilter);

	// If the starting file isn't the same as the root then walk down the tree
	// until we get to it.
	selectFile(m_strPath);

	//
	// Initialize the header imagelist
	//
	if(m_imlHeader.Create(IDB_SORT_ARROWS, 16, 8, RGB(255,0,255)))
	{
		CHeaderCtrl pHeader = m_listFile.GetHeader();

		if(pHeader)
			pHeader.SetImageList(m_imlHeader);
	}

	SetColumnIcons();

	//
	// Update toolbar buttons
	//
	UpdateControls();
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


// CFileOpenDlg::OnSelchangedFolderTree
//
//		The tree has changed its selection
//
LRESULT CFileOpenDlg::OnSelchangedFolderTree(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/) 
{
	NM_TREEVIEW*	pNMTreeView = (NM_TREEVIEW*)pnmh;
	HTREEITEM		hItem;
	
	hItem = pNMTreeView->itemNew.hItem;
	if(hItem)
	{
		m_strPath = GetFullPath(hItem);

		//
		// If displaying only the tree, do not enumerate files
		//
		if(m_nState != vsTreeOnly)
			EnumFiles(m_strPath, m_strFilter);
	}

	return 0;
}


// CFileOpenDlg::OnClickFileList
//
//		The user clicked the file list
//
LRESULT CFileOpenDlg::OnClickFileList(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/)
{
	int iItem;
	TCHAR strBuf[128];

	iItem = m_listFile.GetNextItem(-1, LVNI_SELECTED);
	if(iItem != -1)
	{
		m_listFile.GetItemText(iItem, 0, strBuf, 128);
		m_strFile = strBuf;

		// If we are choosing a file name to save-as then
		// put the name of the file into the edit box
		if (!m_openFileFlag)
		{
			m_editFilename.SetWindowText(m_strFile);
		}
	}

	//
	// Enable or disable the OK button
	//
	m_bOk = iItem != -1;
	UpdateControls();
	
	return 0;
}

LRESULT CFileOpenDlg::OnDblclkFileList(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/)
{
	int iItem;
	TCHAR strBuf[128];

	iItem = m_listFile.GetNextItem(-1, LVNI_SELECTED);
	if(iItem != -1)
	{
		m_listFile.GetItemText(iItem, 0, strBuf, 128);
		m_strFile = strBuf;

		setPreviousDir(m_strPath + m_strFile);

		EndDialog(IDOK);
	}

	//
	// Enable or disable the OK button
	//
	m_bOk = iItem != -1;
	UpdateControls();
		
	return 0;
}

// CFileOpenDlg::OnLButtonDown
//
//		Processes the button down message.

LRESULT CFileOpenDlg::OnLButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	int x = LOWORD(lParam);
	int y = HIWORD(lParam);

	if(m_nState == vsStandard)
	{
		SetCapture();

		m_bTracking = TRUE;
		m_yTrackDel = y - m_ySplit;
		m_yTrack	= m_ySplit;
	}

	return 0;
}


// CFileOpenDlg::OnLButtonUp
//
//		The user stopped dragging
//
LRESULT CFileOpenDlg::OnLButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	if(m_nState == vsStandard)
	{
		ReleaseCapture();

		m_bTracking = FALSE;

		InvalidateRect(NULL);
		UpdateWindow();
	}

	return 0;
}


// CFileOpenDlg::OnMouseMove
//
//		The user is dragging the stylus.
//
LRESULT CFileOpenDlg::OnMouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	int x = LOWORD(lParam);
	int y = HIWORD(lParam);

	if(m_nState == vsStandard && m_bTracking)
	{
		// Get the client rectangle that holds the view controls.
		RECT dlgRect;
		GetClientRect(&dlgRect);

		// Don't go above the bottom of the group box at top
		RECT grpRect;
		m_filetypeGroupBox.GetWindowRect(&grpRect);
		if (y < (grpRect.bottom-grpRect.top) + 6)
		{
			return 0;
		}

		// Don't go below the top of the toolbar at the bottom
		RECT cmdRect;
		m_pCmdBar.GetWindowRect(&cmdRect);
		if (y > dlgRect.bottom - 6)
		{
			return 0;
		}

		m_yTrack = y - m_yTrack;
		m_ySplit = y - m_yTrackDel;

		//
		// Position the controls
		//
		m_rcTree.bottom = m_ySplit;
		m_treeFolder.MoveWindow(&m_rcTree);

		m_rcList.top	= m_ySplit + nTrackHeight + 1;
		m_listFile.MoveWindow(&m_rcList);

		InvalidateRect(NULL);
		UpdateWindow();
	}

	return 0;
}


// CFileOpenDlg::OnHeaderClickFileList
//
//		The header was clicked: sort the file list
//
LRESULT CFileOpenDlg::OnHeaderClickFileList(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/)
{
	HD_NOTIFY*	phdn = (HD_NOTIFY *) pnmh;
	int			iItem;

	iItem = phdn->iItem + 1;

	if(m_nSort == iItem)
		m_nSort = -iItem;
	else
		m_nSort = iItem;

	SetColumnIcons();
	DoSortList();
	
	return 0;
}


// CFileOpenDlg::OnCustomDrawList
//
//		Draws the list with stripes
//
LRESULT CFileOpenDlg::OnCustomDrawList(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/)
{
	LRESULT res = 0;

	NMLVCUSTOMDRAW*	pLVCD = (NMLVCUSTOMDRAW*) pnmh;
	DWORD			dwDrawStage;

	res = CDRF_DODEFAULT;

	dwDrawStage = pLVCD->nmcd.dwDrawStage;

	if(dwDrawStage == CDDS_PREPAINT)
	{
		res = CDRF_NOTIFYITEMDRAW;
	}
	else if(dwDrawStage == CDDS_ITEMPREPAINT)
	{
		res = CDRF_NOTIFYSUBITEMDRAW;
	}
	else if(dwDrawStage == (CDDS_ITEMPREPAINT | CDDS_SUBITEM))
	{
		COLORREF	colBack;
		int			/*iItem,*/
					iSubItem;
		
		//
		// Get the list column pointer
		//
//		iItem		= pLVCD->nmcd.dwItemSpec;
		iSubItem	= pLVCD->iSubItem;

		if(iSubItem + 1 == abs(m_nSort))
			colBack = RGB(224, 224, 255);	// Light blue?
		else
			colBack = RGB(255, 255, 255);	// White

			pLVCD->clrTextBk = colBack;

		res = CDRF_DODEFAULT;
	}

	return res;
}


// CFileOpenDlg::OnItemExpandingFolderTree
//
//		The user is expanding a tree node
//
LRESULT CFileOpenDlg::OnItemExpandingFolderTree(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/)
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pnmh;

	if(pNMTreeView->action & TVE_EXPAND)
	{
		TVITEM	item;

		item.mask	= TVIF_HANDLE | TVIF_PARAM;
		item.hItem	= pNMTreeView->itemNew.hItem;

		if(pNMTreeView->itemNew.lParam == -1)
		{
			CString	strFullPath = GetFullPath(item.hItem);
			RECT	rc;

//			if(strFullPath != _T("\\"))
//				strFullPath += _T("\\");

			if(EnumDirs(strFullPath, _T("*"), item.hItem))
				item.lParam	   = 1;
			else
				item.lParam	   = 0;
			m_treeFolder.SetItem(&item);

			m_treeFolder.GetItemRect(item.hItem, &rc, FALSE);
			m_treeFolder.InvalidateRect(&rc);
			m_treeFolder.UpdateWindow();
		}
	}

	return 0;
}


// CFileOpenDlg::OnGetDispInfoFolderTree
//
//		
LRESULT CFileOpenDlg::OnGetDispInfoFolderTree(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/)
{
	TV_DISPINFO* pTVDispInfo = (TV_DISPINFO*)pnmh;

	if(pTVDispInfo->item.mask & TVIF_CHILDREN)
	{
		if(pTVDispInfo->item.lParam)
			pTVDispInfo->item.cChildren	= 1;
		else
			pTVDispInfo->item.cChildren	= 0;
	}
	return 0;
}


//---------------------------------------------------------------------------
//
//	CFileOpenDlg command handlers
//
//---------------------------------------------------------------------------


// CFileOpenDlg::OnViewStandard
//
//		Enables the standard view
//
LRESULT CFileOpenDlg::OnViewStandard(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	m_treeFolder.ShowWindow(SW_SHOW);
	m_treeFolder.MoveWindow(&m_rcTree);

	m_listFile.ShowWindow(SW_SHOW);
	m_listFile.MoveWindow(&m_rcList);

	if(m_nState == vsTreeOnly)
		EnumFiles(m_strPath, m_strFilter);
	m_nState = vsStandard;

	UpdateControls();

	return 0;
}


// CFileOpenDlg::OnViewTree
//
//		Enables the tree view only
//
LRESULT CFileOpenDlg::OnViewTree(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	m_treeFolder.ShowWindow(SW_SHOW);
	m_treeFolder.MoveWindow(m_rcTree.left,
							m_rcTree.top,
							m_rcList.right - m_rcList.left,
							m_rcList.bottom - m_rcTree.top);

	m_listFile.ShowWindow(SW_HIDE);

	m_nState = vsTreeOnly;

	UpdateControls();

	return 0;
}


// CFileOpenDlg::OnViewList
//
//		Enables the list view only
//
LRESULT CFileOpenDlg::OnViewList(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	m_treeFolder.ShowWindow(SW_HIDE);

	m_listFile.ShowWindow(SW_SHOW);
	m_listFile.MoveWindow(m_rcTree.left,
							m_rcTree.top,
							m_rcList.right - m_rcList.left,
							m_rcList.bottom - m_rcTree.top);

	if(m_nState == vsTreeOnly)
		EnumFiles(m_strPath, m_strFilter);
	m_nState = vsListOnly;

	UpdateControls();

	return 0;
}


// CFileOpenDlg::OnBarOk
//
//		Confirms the selection
//
LRESULT CFileOpenDlg::OnBarOk(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	// If we are saving a file, there may be a conflict between the
	// filename that was selected and what is in the edit box.  If
	// so, the edit box wins.
	if (!m_openFileFlag)
	{
		TCHAR fText[128];
		m_editFilename.GetWindowText(fText, 127);
		if (fText[0] != '\0')
		{
			m_strFile = fText;
		}
	}

	// Save the last directory selected so we can start there the
	// next time this dialog is used.
	setPreviousDir(m_strPath + m_strFile);

	EndDialog(IDOK);

	return 0;
}


// CFileOpenDlg::OnBarCancel
//
//		Cancels the operation
//
LRESULT CFileOpenDlg::OnBarCancel(WORD /*wNotifyCode*/, WORD wID, HWND hWndCtl, BOOL& /*bHandled*/)
{
	EndDialog(IDCANCEL);

	return 0;
}


//---------------------------------------------------------------------------
//
//	Sorting functions
//
//---------------------------------------------------------------------------
static int SortNameAsc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	CFileInfoCont*	pCont = (CFileInfoCont*)lParamSort;

	return _wcsicmp((*pCont)[lParam1].cFileName, (*pCont)[lParam2].cFileName);
}

static int SortNameDes(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	CFileInfoCont*	pCont = (CFileInfoCont*)lParamSort;

	return _wcsicmp((*pCont)[lParam2].cFileName, (*pCont)[lParam1].cFileName);
}

static int SortSizeAsc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	CFileInfoCont*	pCont = (CFileInfoCont*)lParamSort;

	return (int)(*pCont)[lParam1].nFileSizeLow - 
		   (int)(*pCont)[lParam2].nFileSizeLow;
}

static int SortSizeDes(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	CFileInfoCont*	pCont = (CFileInfoCont*)lParamSort;

	return (int)(*pCont)[lParam2].nFileSizeLow - 
		   (int)(*pCont)[lParam1].nFileSizeLow;
}

static int SortDateAsc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	CFileInfoCont*	pCont = (CFileInfoCont*)lParamSort;

	return CompareFileTime(&(*pCont)[lParam1].ftLastWriteTime,  
						   &(*pCont)[lParam2].ftLastWriteTime);
}

static int SortDateDes(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	CFileInfoCont*	pCont = (CFileInfoCont*)lParamSort;

	return CompareFileTime(&(*pCont)[lParam2].ftLastWriteTime,  
						   &(*pCont)[lParam1].ftLastWriteTime);
}

LRESULT CFileOpenDlg::OnPaint(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	CPaintDC dc(m_hWnd); // device context for painting
	
	DoPaint(dc);

	return 0;
}

LRESULT CFileOpenDlg::OnSelchangeFiletypeCombo(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	// TODO: Add your control notification handler code here
	int selIndex = m_filetypeCombo.GetCurSel();
	if (selIndex != LB_ERR)
	{
		m_strFilter = m_FilterStrings[selIndex];
	}
	else
	{
		m_strFilter = _T("*.*");
	}

	if(m_nState != vsTreeOnly)
		EnumFiles(m_strPath, m_strFilter);

	return 0;
}

CString CFileOpenDlg::getPreviousDir()
{
	CString lastDir(_T("\\My Documents"));
	HKEY key;
	unsigned long disposition;
	int res = RegCreateKeyEx(HKEY_CURRENT_USER , _T("Software\\Xander\\spAsm"), 0,
		_T("string"), 0, 0, NULL,
		&key, &disposition);
	if (res == ERROR_SUCCESS)
	{

		if (disposition == REG_CREATED_NEW_KEY)
		{
			// Haven't used this dialog before, or reg has been cleaned.  Set last directory
			// to standard "My Documents".
			CString defDir(_T("\\My Documents"));
			int defDirLen = (defDir.GetLength()+1)*sizeof(WCHAR);
			RegSetValueEx(key, _T("LastDir"), 0, REG_SZ, (BYTE *)(LPCTSTR)defDir, defDirLen);
		}
		else
		{
			unsigned long keyType;
			TCHAR *charBuf = new TCHAR[1024];
			unsigned long dataLen = 2046;

			res = RegQueryValueEx(key, _T("LastDir"), NULL, &keyType, (BYTE *)charBuf, &dataLen);
			if (res == ERROR_SUCCESS && keyType == REG_SZ)
			{
				lastDir = charBuf;
			}

			delete[] charBuf;
		}

		RegCloseKey(key);
	}

	return lastDir;
}

void CFileOpenDlg::setPreviousDir(CString &dirName)
{
	CString lastDir(_T("\\My Documents"));
	HKEY key;
	unsigned long disposition;
	int res = RegCreateKeyEx(HKEY_CURRENT_USER , _T("Software\\Xander\\spAsm"), 0,
		_T("string"), 0, 0, NULL,
		&key, &disposition);
	if (res == ERROR_SUCCESS)
	{
		int numBytes = (dirName.GetLength() + 1) * sizeof(WCHAR);
		res = RegSetValueEx(key, _T("LastDir"), 0, REG_SZ, (BYTE *)(LPCTSTR)dirName, numBytes);

		RegCloseKey(key);
	}
}

int CFileOpenDlg::selectFile(CString &fileName)
{
	int nameLen = fileName.GetLength() + 1;
	TCHAR *fileBuf = new WCHAR[nameLen];
	_tcscpy(fileBuf, fileName);
	TCHAR *token = _tcstok(fileBuf, _T("\\/"));
	HTREEITEM currentItem = m_treeFolder.GetRootItem();

	// Ensure we have the directories at the top level
	CString	strFullPath = GetFullPath(currentItem);
	EnumDirs(strFullPath, _T("*"), currentItem);

	int itemCount = m_treeFolder.GetCount();
	while (token != NULL)
	{
		if (token[0] == '\0')
		{
			// Root directory
			continue;
		}
		else
		{
			if (m_treeFolder.ItemHasChildren(currentItem))
			{
				// Has children - look for a match with the current directory name
				HTREEITEM hChildItem = m_treeFolder.GetChildItem(currentItem);

				if (!hChildItem)
				{
					delete[] fileBuf;
					return -1;
				}

				strFullPath = GetFullPath(hChildItem);

				while (hChildItem != NULL)
				{
					TCHAR childName[128];
					m_treeFolder.GetItemText(hChildItem, childName, 127);
					if (_tcsicmp(childName, token) == 0)
					{
						// Found it.
						currentItem = hChildItem;

						// Select it and enumerate below.
						CString	strFullPath = GetFullPath(hChildItem);

#if 1
						TVITEM tvI;
						tvI.mask = TVIF_HANDLE | TVIF_PARAM;
						tvI.hItem = hChildItem;
						m_treeFolder.GetItem(&tvI);
						if (tvI.lParam == -1)
						{
							// Haven't expanded this directory before
							if (EnumDirs(strFullPath, _T("*"), hChildItem))
							{
								tvI.lParam = 1;
							}
							else
							{
								tvI.lParam = 0;
							}
							m_treeFolder.SetItem(&tvI);
						}
#else
						EnumDirs(strFullPath, _T("*"), hChildItem);
#endif

						m_treeFolder.Expand(currentItem, TVE_EXPAND);
						m_treeFolder.SelectItem(hChildItem);

						currentItem = hChildItem;
						break;
					}
					else
					{
						// Not a match - look at the next directory
						hChildItem = m_treeFolder.GetNextItem(hChildItem, TVGN_NEXT);
					}
				}

				if (hChildItem == NULL)
				{
					// Failed to find a matching name
					delete[] fileBuf;
					return -1;
				}
			}
			else
			{
				// Had a directory but no children.  Abort processing
				delete[] fileBuf;
				return -1;
			}
		}

		token = wcstok(NULL, _T("\\/"));
	}

	delete[] fileBuf;

	return 0;
}

LRESULT CFileOpenDlg::OnSize(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	int nType = wParam;
	int cx = LOWORD(lParam);
	int cy = HIWORD(lParam);

	resizeDialog(nType, cx, cy);

	return 0;
}

LRESULT CFileOpenDlg::OnUpdateEditFilename(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	// See if the new filename matches anything in the list.  If so, then highlight
	// that file.  If not, then deselect everything in the list view.
	int iItem = -1;

	if (!m_openFileFlag)
	{
		TCHAR fText[128];
		m_editFilename.GetWindowText(fText, 127);
		if (!fText[0])
		{
			int itemIndex;
			int nItems = m_listFile.GetItemCount();
			m_strFile = fText;

			// Clear selection state for everything
			m_listFile.SetItemState(-1, LVIS_SELECTED, 0);


			// Look for a match in the list of files
			for (itemIndex=0;itemIndex<nItems;itemIndex++)
			{
				TCHAR fileName[128];
				m_listFile.GetItemText(itemIndex, 0, fileName, 127);
				if (_tcsicmp(fText, fileName) == 0)
				{
					// Have a match
					int stateFlags = LVIS_SELECTED|LVM_ENSUREVISIBLE;
					iItem = itemIndex;
					m_listFile.SetCheckState(iItem, TRUE);
					m_listFile.SetItemState(iItem, stateFlags, stateFlags);
					m_listFile.Update(iItem);

					break;
				}
			}

			// No matter what the filename, we can save, so turn on Ok button.
			m_bOk = 1;
		}
		else
		{
			// No text, therefore no filename
			m_bOk = 0;
		}
	}
	else
	{
		// Only allow "ok" button for opening files if there is actually
		// an existing file that is selected in the list.
		m_bOk = iItem != -1;
	}

	UpdateControls();

	return 0;
}

TBBUTTON btns[] = {
	// { 0, IDOK, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, -1},
	{ 1, IDCANCEL, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, -1},
	{ -1, 0, TBSTATE_ENABLED, TBSTYLE_SEP, 0, -1},
	{ 2, ID_VIEW_STANDARD, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, -1},
	{ 3, ID_VIEW_TREE, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, -1},
	{ 4, ID_VIEW_LIST, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, -1}
};

void CFileOpenDlg::AddToolButtons()
{
#if defined(_WIN32_WCE)
	HINSTANCE hInst = _Module.GetResourceInstance();

	SHMENUBARINFO mbi = { 0 };
	mbi.cbSize = sizeof(mbi);
	mbi.hwndParent = m_hWnd;
	mbi.dwFlags    = SHCMBF_HMENU;
	mbi.nToolBarId = IDR_MENUFILEOPEN;
	mbi.hInstRes   = _Module.GetResourceInstance();

	BOOL bRet = ::SHCreateMenuBar(&mbi);

	m_hWndCECommandBar = mbi.hwndMB;

	if(bRet == FALSE)
	{
		m_hWndCECommandBar = NULL;
		return;
	}

	CToolBarCtrl mTool = m_hWndCECommandBar;

	mTool.AddBitmap(5, IDB_BITMAP_FILEOPEN);
	mTool.AddButtons(5, btns);
	UIAddToolBar(m_hWndCECommandBar);
#else
	// TBD: Build the toolbar for standard Win32 targets
#endif
}
