// MainFrm.cpp : implmentation of the CMainFrame class
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"
#if WINVER >= 0x0500
#pragma message ("Version > 0x500")
#elif WINVER >= 0x0420
#pragma message ("Version > 0x420")
#elif WINVER >= 0x0410
#pragma message ("Version > 0x410")
#elif WINVER >= 0x0400
#pragma message ("Version > 0x400")
#else
#pragma message ("Version < 0x400")
#endif
#if defined(_WIN32_WCE)
#else
#include <mmsystem.h>
#include <process.h>
extern "C"
{
#include <Dbt.h>
}
#endif
#include <assert.h>

#include "aboutdlg.h"
#include "OptionsDlg.h"
#include "RegistersDlg.h"
#include "TesterDlg.h"
#include "PIC/DeviceList.h"
#include "PIC/PICHexFile.h"
#include "PIC/HexFileIO.h"
#include "MainFrm.h"
#include "PIC/ProgramSelector.h"
#if defined(_WIN32_WCE)
#include "FileOpenDlg.h"
#endif

#define BAUD_RATE CBR_115200
// #define BAUD_RATE CBR_38400

#define NUM_STATUS_PANES 6
#define BASE_STATUS_PANE 0
#define PORT_STATUS_PANE 1
#define PROCESSOR_STATUS_PANE 2
#define VPP_STATUS_PANE 3
#define EXT_PWR_STATUS_PANE 4
#define EXTRA_STATUS_PANE 5

BOOL CMainFrame::PreTranslateMessage(MSG* pMsg)
{
	if(CFrameWindowImpl<CMainFrame>::PreTranslateMessage(pMsg))
		return TRUE;

	return m_view.PreTranslateMessage(pMsg);
}



void CMainFrame::readProgrammerVoltage(void)
{
	static DWORD lastClockVal = ::timeGetTime();
	if (m_ProgrammerActive)
	{
		// Don't bother the programmer if something useful (read/write/erase)
		// is going on.
	}

	DWORD currentClockVal = ::timeGetTime();
	DWORD deltaTime = currentClockVal - lastClockVal;
	if (deltaTime < 500)
		return;

	// It's been a second since the last update
	lastClockVal = currentClockVal;
	// More than a quarter second - read the analog val
	if (!openCOMPort(BAUD_RATE))
		// Failed to open the port
		return;
	if (!SUCCEEDED(PICProgram::ResetProgrammer(&m_Port)))
	{
		// Failed to get the programmer into a known good state.
		m_Port.Close();
		return;
	}

	// Ask the programmer for the current voltage and PWM duty cycle
	uint i, analogVal, PWMDutyCycle;
	i = PICProgram::TxCmd(&m_Port, PICProgram::EVA_READ_ANALOG_CMD, 0, &analogVal);
	if (i != 0)
	{
		PICProgram::PICClose(&m_Port);
		m_Port.Close();
		printf("Voltage command failed\n");
		return;
	}
	i = PICProgram::TxCmd(&m_Port, PICProgram::EVA_READ_PWM_CMD, 0, &PWMDutyCycle);
	if (i)
	{
		PICProgram::PICClose(&m_Port);
		m_Port.Close();
		printf("Voltage command failed\n");
		return;
	}

	// We are reading voltage from a divider that uses a 47K and 10K resistor.
	// For a total voltage V, we measure a voltage X, where X / 10 = V / (47 + 10).
	// To report the total voltage, we just need to multiply, V = 57 X / 10.
	// Note: older design used full 5V power, whereas newer one get around
	// 4.7V - the scaling of the result depends on this value.
	TCHAR vStatus[128];
	float boostAnalog = 57.0f * (4.7f * (float)(analogVal & 0xff) / 255.0f) / 10.0f;
	float extAnalog = 57.0f * (4.7f * (float)((analogVal >> 8) & 0xff) / 255.0f) / 10.0f;
	// float analog = 57.0f * (5.0f * (float)analogVal / 255.0f) / 10.0f;
	_stprintf(vStatus, _T("Vpp: %5.2fV, PWM: 0x%x"), boostAnalog, PWMDutyCycle);

	setStatusPane(VPP_STATUS_PANE, vStatus);

	_stprintf(vStatus, _T("Pwr: %5.2fV"), extAnalog);
	setStatusPane(EXT_PWR_STATUS_PANE, vStatus);

	PICProgram::PICClose(&m_Port);
	m_Port.Close();
}

/* Backgound thread that reads the current voltage being generated in the programmer
   and reports it in a status pane. */
#if defined(_WIN32_WCE)
DWORD
#else
unsigned __stdcall
#endif
statusTimerThreadProc(void *arg)
{
	CMainFrame *mainWnd = (CMainFrame *)arg;
	HANDLE hMutex = CreateMutex(NULL, FALSE, _T("COMPortMutex"));

	if (hMutex == NULL)
		// No mutex to keep this thread from trampling on the main thread
		return -1;

	for (;;)
	{
		::Sleep(100);
		if (mainWnd->m_Terminating)
			break;
		DWORD dwWaitResult; 

	    // Request ownership of mutex.
		dwWaitResult = WaitForSingleObject(hMutex, 250L);
	    switch (dwWaitResult) 
		{
			// The thread got mutex ownership.
			case WAIT_OBJECT_0: 
				if (mainWnd->m_PollStatusFlag)
	 				mainWnd->readProgrammerVoltage();
               // Release ownership of the mutex object.
                if (!ReleaseMutex(hMutex)) 
                {
                    // Deal with error.
                }
                break; 
			// Cannot get mutex ownership due to time-out.
			case WAIT_TIMEOUT: 
				break; 
			// Got ownership of the abandoned mutex object.
			case WAIT_ABANDONED: 
				break; 
		}
	}

	// Let the main application know we are done and gone.
	mainWnd->m_StatusThreadHandle = NULL;

	// Ensure cleanup.
#if defined(_WIN32_WCE)
#else
	::_endthread();
#endif

	return 0;
}

void statusTimerCallback(UINT uTimerID, UINT uMsg, DWORD_PTR dwUser, DWORD_PTR dw1, DWORD_PTR dw2)
{
	CMainFrame *mainWnd = (CMainFrame *)dwUser;
	mainWnd->readProgrammerVoltage();
}


BOOL CMainFrame::OnIdle()
{
	UIUpdateToolBar();
	return FALSE;
}

void CMainFrame::buildStatusBar(void)
{
	CStatusBarCtrl statusBar;
	statusBar.Attach(m_hWndStatusBar);
	statusBar.SetSimple(FALSE);
	RECT rect;
	statusBar.GetClientRect(&rect);
	int paneWidths[NUM_STATUS_PANES] = {rect.right - 360, rect.right - 310, rect.right - 230, rect.right - 96, rect.right - 20, -1};
	statusBar.SetParts(NUM_STATUS_PANES, paneWidths);

	for (int i=0;i<NUM_STATUS_PANES;i++)
		setStatusPane(i, _T(""));
}

static bool cmpTSTRS(LPTSTR elem1, LPTSTR elem2)
{
	return (_tcscmp(elem1, elem2) < 0);
}

static bool equalTSTRS(LPTSTR elem1, LPTSTR elem2)
{
	return (_tcscmp(elem1, elem2) == 0);
}

void CMainFrame::buildPortMenu(void)
{
	std::list<LPTSTR> *COMPorts = CSerial::GetCOMPortList();

	m_ComPortMenu.CreatePopupMenu();
	if (COMPorts != NULL)
	{
		// Put the com ports into order
		COMPorts->sort(cmpTSTRS);
		// Remove any duplicates
		COMPorts->unique(equalTSTRS);
#if !defined(_WIN32_WCE)
		// If not on a PocketPC, then add USB as serial port
		LPTSTR usbStr = new TCHAR[4];
		_tcscpy(usbStr, _T("USB")); // Make local copy so we can free later...
		COMPorts->insert(COMPorts->begin(), usbStr);
#endif
		m_ComPortMenuCount = (int)COMPorts->size();
		std::list<LPTSTR>::iterator itr;
		int index = 1;
		for (itr=COMPorts->begin();itr!=COMPorts->end();itr++)
		{
			LPTSTR str = (LPTSTR)*itr;
			size_t strLen = _tcslen(str);
			// Make sure we aren't dealing with an empty string
			if (strLen < 1)
				continue;
			CString tmpComName = str;
			if (str[strLen-1] != _T(':'))
				tmpComName += _T(':');
			m_ComPortMenu.AppendMenu(MF_STRING, index++, tmpComName);
		}
	}
	else
	{
#if !defined(_WIN32_WCE)
		m_ComPortMenu.AppendMenu(MF_STRING, 1, _T("USB:"));
#endif
		m_ComPortMenu.AppendMenu(MF_STRING, 1, _T("COM1:"));
		m_ComPortMenu.AppendMenu(MF_STRING, 2, _T("COM2:"));
		m_ComPortMenu.AppendMenu(MF_STRING, 3, _T("COM3:"));
		m_ComPortMenu.AppendMenu(MF_STRING, 4, _T("COM4:"));
		m_ComPortMenu.AppendMenu(MF_STRING, 5, _T("COM5:"));
		m_ComPortMenu.AppendMenu(MF_STRING, 6, _T("COM6:"));
#if defined(_WIN32_WCE)
		m_ComPortMenu.AppendMenu(MF_STRING, 7, _T("COM7:"));
		m_ComPortMenu.AppendMenu(MF_STRING, 8, _T("COM8:"));
		m_ComPortMenu.AppendMenu(MF_STRING, 9, _T("COM9:"));
		m_ComPortMenuCount = 10;
#else
		m_ComPortMenuCount = 5;
#endif
	}
	bool foundPort = false;
	MENUITEMINFO mii = { 0 };
	mii.cbSize = sizeof(mii);
#if defined(_WIN32_WCE)
	uint itemCount = m_ComPortMenuCount;
#else
	uint itemCount = m_ComPortMenu.GetMenuItemCount();
#endif
	for (uint mInd=0;mInd<itemCount;mInd++)
	{
		TCHAR buf[16];
#if defined(_WIN32_WCE)
		mii.fMask = MIIM_TYPE;
		mii.fType = MFT_STRING;
		mii.cch = sizeof(buf)-1;
		mii.dwTypeData = buf;
		int res = m_ComPortMenu.GetMenuItemInfo(mInd, MF_BYPOSITION, &mii);
#else
		int res = m_ComPortMenu.GetMenuString(mInd, buf, sizeof(buf)-1, MF_BYPOSITION);
#endif
		if (SUCCEEDED(res))
		{
			if (_tcsicmp(m_Defaults.portName, buf) == 0)
			{
				// Found the default port in the list of available ports
				m_ComPortMenu.CheckMenuItem(mInd, MF_BYPOSITION | MF_CHECKED);
				m_PortName = m_Defaults.portName;
				foundPort = true;
				break;
			}
		}
	}
	if (!foundPort)
	{
		m_PortName = _T("COM1:");
		m_ComPortMenu.CheckMenuItem(0, MF_BYPOSITION | MF_CHECKED);
	}

	// Free memory associated with the list of COM port names
	if (COMPorts != NULL)
	{
		std::list<LPTSTR>::iterator itr;
		for (itr=COMPorts->begin();itr!=COMPorts->end();itr++)
		{
			LPTSTR str = (LPTSTR)*itr;
			if (str != NULL)
				delete[] str;
		}
		delete COMPorts;
	}

	setStatusPane(PORT_STATUS_PANE, m_PortName);
}

// Build a menu with available processors.  Check the menu item that
// holds the default processor.
void CMainFrame::buildDeviceMenu(int deviceIndex)
{
	m_DeviceListMenu.CreatePopupMenu();
	P12Menu.CreatePopupMenu();
	P16Menu.CreatePopupMenu();
	P18Menu.CreatePopupMenu();
	P30Menu.CreatePopupMenu();
	m_DeviceListMenu.AppendMenu(MF_POPUP, (UINT_PTR)P12Menu.m_hMenu, _T("PIC12F"));
	m_DeviceListMenu.AppendMenu(MF_POPUP, (UINT_PTR)P16Menu.m_hMenu, _T("PIC16F"));
	m_DeviceListMenu.AppendMenu(MF_POPUP, (UINT_PTR)P18Menu.m_hMenu, _T("PIC18F"));
	m_DeviceListMenu.AppendMenu(MF_POPUP, (UINT_PTR)P30Menu.m_hMenu, _T("dsPIC30F"));
	int devCount = DeviceList::deviceCount();
	for (int devIndex=0;devIndex<devCount;devIndex++)
	{
		const PICDevice *dev = DeviceList::Devices[devIndex];
		if (dev != NULL)
		{
			// Use devIndex+1 because 0 is an error/no response during tracking
			// of a popup menu.  (Which will occur in response to clicking the
			// processor selection button on the toolbar.)
			if ((dev->name[0][1] == _T('3')) && (dev->name[0][2] == _T('0')))
				P30Menu.AppendMenu(MF_STRING, devIndex+1, dev->name[2]);
			else if (dev->name[0][2] == _T('2'))
				P12Menu.AppendMenu(MF_STRING, devIndex+1, dev->name[2]);
			else if (dev->name[0][2] == _T('6'))
				P16Menu.AppendMenu(MF_STRING, devIndex+1, dev->name[2]);
			else if (dev->name[0][2] == _T('8'))
				P18Menu.AppendMenu(MF_STRING, devIndex+1, dev->name[2]);
			else
				// Unknown part # - DeviceList must be invalid
				assert(false);
		}
	}

	// Check the last used device
	const PICDevice *lastDev = DeviceList::Devices[deviceIndex];
	if ((lastDev->name[0][1] == _T('3')) && (lastDev->name[0][2] == _T('0')))
		P30Menu.CheckMenuItem(deviceIndex+1, MF_CHECKED);
	else if (lastDev->name[0][2] == _T('2'))
		P12Menu.CheckMenuItem(deviceIndex+1, MF_CHECKED);
	else if (lastDev->name[0][2] == _T('6'))
		P16Menu.CheckMenuItem(deviceIndex+1, MF_CHECKED);
	else if (lastDev->name[0][2] == _T('8'))
		P18Menu.CheckMenuItem(deviceIndex+1, MF_CHECKED);
}

void CMainFrame::AddToolButtons()
{
#if defined(_WIN32_WCE)
	HINSTANCE hInst = _Module.GetResourceInstance();
	CreateSimpleCEMenuBar(IDR_CEMAINFRAME, SHCMBF_HMENU);

	CToolBarCtrl mTool = m_hWndCECommandBar;

	mTool.AddBitmap(5, IDB_BITMAP_TOOLS);

	TBBUTTON tbi = { 0 };
	tbi.fsStyle = TBSTYLE_BUTTON;
	tbi.iBitmap = 0;
	tbi.fsState = TBSTATE_ENABLED;
	tbi.idCommand = ID_CHIP_READ;
	tbi.iString = 0;
	mTool.InsertButton(1, &tbi);

	tbi.iBitmap = 1;
	tbi.idCommand = ID_CHIP_PROGRAM;
	mTool.InsertButton(2, &tbi);

	tbi.iBitmap = 2;
	tbi.idCommand = ID_CHIP_ERASE;
	mTool.InsertButton(3, &tbi);

	tbi.fsStyle = TBSTYLE_SEP;
	mTool.InsertButton(4, &tbi);

	tbi.fsStyle = TBSTYLE_DROPDOWN;
	tbi.fsState = TBSTATE_ENABLED;
	tbi.iBitmap = 3;
	tbi.idCommand = ID_PROCESSOR_COMBO;
	mTool.InsertButton(5, &tbi);

	tbi.fsStyle = TBSTYLE_SEP;
	mTool.InsertButton(6, &tbi);

	tbi.fsStyle = TBSTYLE_DROPDOWN;
	tbi.fsState = TBSTATE_ENABLED;
	tbi.iBitmap = 4;
	tbi.idCommand = ID_BUTTON_COM_PORT;
	mTool.InsertButton(7, &tbi);

	UIAddToolBar(m_hWndCECommandBar);
#else
	// create command bar window
	HWND hWndCmdBar = m_CmdBar.Create(m_hWnd, rcDefault, NULL, ATL_SIMPLE_CMDBAR_PANE_STYLE);
	// attach menu
	m_CmdBar.AttachMenu(GetMenu());
	// load command bar images
	m_CmdBar.LoadImages(IDR_MAINFRAME);
	// remove old menu
	SetMenu(NULL);

	HWND hWndToolBar = CreateSimpleToolBarCtrl(m_hWnd, IDR_MAINFRAME, FALSE, ATL_SIMPLE_TOOLBAR_PANE_STYLE);
	CreateSimpleReBar(ATL_SIMPLE_REBAR_NOBORDER_STYLE);
	AddSimpleReBarBand(hWndCmdBar);
	AddSimpleReBarBand(hWndToolBar, NULL, TRUE);

	CToolBarCtrl toolBar;
	toolBar.Attach(hWndToolBar);
	toolBar.SetExtendedStyle(TBSTYLE_LIST | TBSTYLE_EX_DRAWDDARROWS);

	TBBUTTONINFO tbi;
	tbi.cbSize = sizeof(tbi);
	tbi.dwMask = TBIF_STYLE;
	tbi.fsStyle = TBSTYLE_DROPDOWN;
	toolBar.SetButtonInfo(ID_PROCESSOR_COMBO, &tbi);
	toolBar.SetButtonInfo(ID_BUTTON_COM_PORT, &tbi);

	UIAddToolBar(hWndToolBar);
#endif
}

LRESULT CMainFrame::OnDeviceChange(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
#if !defined(_WIN32_WCE)
	DEV_BROADCAST_HDR *devHeader = NULL;
	DEV_BROADCAST_DEVICEINTERFACE *devIF = NULL;
	DEV_BROADCAST_PORT *port = NULL;
	DEV_BROADCAST_VOLUME *vol = NULL;
	char *portName = NULL;
	int devType = -1;

	if (wParam == DBT_DEVICEARRIVAL)
	{
		devHeader = (DEV_BROADCAST_HDR *)lParam;
		devType = devHeader->dbch_devicetype;
		switch (devType)
		{
		case DBT_DEVTYP_DEVICEINTERFACE:
			devIF = (DEV_BROADCAST_DEVICEINTERFACE *)lParam;
			// TBD: See if it is the device we are interested in
			// For now, set DeviceDetected to false on any
			// interface change.  Slight performance penalty,
			// but will ensure we don't try talking to a device
			// that isn't there.
			// Ensure the device is available.
			// DeviceDetected = false;
			if (m_Port.IsOpen())
				m_Port.Close();
			break;
		case DBT_DEVTYP_PORT:
			port = (DEV_BROADCAST_PORT *)lParam;
			portName = port->dbcp_name;
			break;
		case DBT_DEVTYP_VOLUME:
			vol = (DEV_BROADCAST_VOLUME *)lParam;
			break;
		default:
			break;
		}
	}
	else if (wParam == DBT_DEVICEREMOVECOMPLETE)
	{
		devHeader = (DEV_BROADCAST_HDR *)lParam;
		devType = devHeader->dbch_devicetype;
		switch (devType)
		{
		case DBT_DEVTYP_DEVICEINTERFACE:
			devIF = (DEV_BROADCAST_DEVICEINTERFACE *)lParam;
			if (m_Port.IsOpen())
				m_Port.Close();
			break;
		case DBT_DEVTYP_PORT:
			port = (DEV_BROADCAST_PORT *)lParam;
			portName = port->dbcp_name;
			break;
		case DBT_DEVTYP_VOLUME:
			vol = (DEV_BROADCAST_VOLUME *)lParam;
			break;
		default:
			break;
		}
	}
#endif
	return 0;
}

LRESULT CMainFrame::OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	m_hWndClient = m_view.Create(m_hWnd);

	m_PortName = NULL;
	m_procMenuItem = NULL;
	m_BaudRate = CBR_115200;
	m_AsmView = 0;

	m_Device = NULL;
	m_ProgrammerActive = false;
	m_PollStatusFlag = false;

	// Load default/last used values
	m_Defaults.Load();

	AddToolButtons();

	CreateSimpleStatusBar();

	// Lay out the panes in the status bar.
	buildStatusBar();

	UISetCheck(ID_VIEW_TOOLBAR, 1);
	UISetCheck(ID_VIEW_STATUS_BAR, 1);

	// register object for message filtering and idle updates
	CMessageLoop* pLoop = _Module.GetMessageLoop();
	ATLASSERT(pLoop != NULL);
	pLoop->AddMessageFilter(this);
	pLoop->AddIdleHandler(this);

	// Build a menu with available processors
	int deviceIndex = DeviceList::DeviceIndex(m_Defaults.processorName);
	if (deviceIndex == -1)
	{
		// Got a bad/unsupported processor name from the registry.  Use a specific
		// default name.
		deviceIndex = DeviceList::DeviceIndex(_T("16f684"));
		if (deviceIndex == -1)
			// The name we chose failed too.  Try setting the default index to 0
			// and see what happens.
			deviceIndex = 0;
	}
	buildDeviceMenu(deviceIndex);

	// Build a menu with available COM ports
	buildPortMenu();

	// Set defaults
	m_BaudRate = m_Defaults.baudRate;
	m_AsmView = m_Defaults.asmView;

	// Set the processor to a default value and create a blank hex file to
	// go along with it.  TBD: load defaults from config/registry
	m_Device = DeviceList::Devices[deviceIndex];
	setStatusPane(PROCESSOR_STATUS_PANE, m_Device->name[2]);
	m_Driver = ProgramSelector::GetProgrammer(m_Device);
	m_HexFile1 = new PICHexFile(m_Device);
	displayHexFile(m_view.m_hexPage1, m_HexFile1);
	m_HexFile2 = new PICHexFile(m_Device);
	displayHexFile(m_view.m_hexPage2, m_HexFile2);

#if defined(_WIN32_WCE)
#else
	CMenuHandle formMenu = m_CmdBar.GetMenu();
	CMenuHandle settingsSubMenu = formMenu.GetSubMenu(1);
#endif
	// settingsSubMenu.InsertMenu(0, MF_POPUP, (UINT_PTR)m_DeviceListMenu.m_hMenu, _T("Device"));

	// m_ProgressDlg.Create(m_hWnd);

	// The following trashes the program - must be callign system routines in the
	// timer callback that makes Windows unhappy.
	// timeSetEvent(500, 100, (LPTIMECALLBACK)statusTimerCallback, (DWORD_PTR)this, TIME_PERIODIC);
	m_ProgrammerMutex = CreateMutex(NULL, FALSE, _T("COMPortMutex"));

	// We are starting up
	m_Terminating = false;

#if defined(_WIN32_WCE)
	DWORD threadID;
	m_StatusThreadHandle = CreateThread(NULL, NULL, statusTimerThreadProc, this, 0, &threadID);
#else
    m_StatusThreadHandle = (HANDLE)_beginthreadex(NULL, 0, statusTimerThreadProc, this, 0, NULL);
#endif

	return 0;
}

HRESULT CMainFrame::ActivatePreviousInstance(HINSTANCE hInstance)
{
	CFrameWndClassInfo& classInfo = CMainFrame::GetWndClassInfo();

	int nRet = ::LoadString(hInstance, IDR_CEMAINFRAME, classInfo.m_szAutoName, sizeof(classInfo.m_szAutoName)/sizeof(classInfo.m_szAutoName[0]));
	ATLASSERT(0 != nRet);

	classInfo.m_wc.lpszClassName = classInfo.m_szAutoName;

	const TCHAR* pszClass = classInfo.m_wc.lpszClassName;

	if(NULL == pszClass || '\0' == *pszClass)
	{
		return E_FAIL;
	}

	// Orginally 500ms in SmartPhone 2003 App Wizard generated code
	// A lower value will result in a more responsive start-up of 
	// the existing instance or termination of this instance.
	const DWORD dRetryInterval = 100; 

	// Orginally 5 in SmartPhone 2003 App Wizard generated code
	// Multiplied by 5, since wait time was divided by 5.
	const int iMaxRetries = 25;

	for(int i = 0; i < iMaxRetries; ++i)
	{
		// Don't need ownership of the mutex
		HANDLE hMutex = CreateMutex(NULL, FALSE, pszClass);

		DWORD dw = GetLastError();

		if(NULL == hMutex)
		{
			HRESULT hr;

			switch(dw)
			{
			case ERROR_INVALID_HANDLE:
				// A non-mutext object with this name already exists.
				hr = E_INVALIDARG;
				break;
			default:
				// This should never happen...
				hr = E_FAIL;
			}

			return hr;
		}

		// If the mutex already exists, then there should be another instance running
		if(ERROR_ALREADY_EXISTS == dw)
		{
			// Just needed the error result, in this case, so close the handle.
			CloseHandle(hMutex);

			// Try to find the other instance, don't need to close HWND's.
			// Don't check title in case it is changed by app after init.
			HWND hwnd = FindWindow(pszClass, NULL);

			if(NULL == hwnd)
			{
				// It's possible that the other istance is in the process of starting up or shutting down.
				// So wait a bit and try again.
				Sleep(dRetryInterval);
				continue;
			}
			else
			{
				// Set the previous instance as the foreground window

				// The "| 0x1" in the code below activates the correct owned window 
				// of the previous instance's main window according to the SmartPhone 2003
				// wizard generated code.
				if(0 != SetForegroundWindow(reinterpret_cast<HWND>(reinterpret_cast<ULONG>(hwnd) | 0x1)))
				{
					// S_FALSE indicates that another instance was activated, so this instance should terminate.
					return S_FALSE;
				}
			}
		}
		else
		{
			// This is the first istance, so return S_OK.
			// Don't close the mutext handle here.
			// Do it on app shutdown instead.
			return S_OK;
		}
	}

	// The mutex was created by another instance, but it's window couldn't be brought
	// to the foreground, so ssume  it's not a invalid instance (not this app, hung, etc.)
	// and let this one start.
	return S_OK;
}


LRESULT CMainFrame::OnClose(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
{
	// Let all threads know we are halting.
	m_Terminating = true;

	// Wait a second to give threads a chance to clean up.
	::Sleep(500);

	if (m_StatusThreadHandle != 0)
	{
		// Didn't cleanup - terminate the thread now.
		TerminateThread((HANDLE)m_StatusThreadHandle, -3);
		m_StatusThreadHandle = 0;
	}

	if (m_Port.IsOpen())
	{
		m_Port.Close();
	}

	bHandled = FALSE;

	return 1;
}


CString getDevNameFromID(uint devID, uint wordSize)
{
	int numDevs = DeviceList::deviceCount();
	uint revNum;
	uint baseDevID;

	if ((wordSize == 14) || (wordSize == 16))
	{
		baseDevID = devID & 0xffe0;
		revNum = devID & 0x001f;
	}
	else if (wordSize == 24)
	{
		baseDevID = devID & 0xffff;
		revNum = ((devID >> 21) & 0x3f);
	}
	else
	{
		baseDevID = 0;
		revNum = 0;
	}

	for (int j=0;j<numDevs;j++)
	{
		const PICDevice *device = DeviceList::Devices[j];
		if (device == NULL)
			return _T("Unknown Device");
		uint tdevID;
		if (wordSize == 14 || wordSize == 16)
			tdevID = (device->deviceIDMask & 0xffe0); // Mask off revision bits
		else
			tdevID = (device->deviceIDMask & 0xffff); // Rev bits are in high word
		if ((device->wordSize == wordSize) && (tdevID == baseDevID))
		{
			// Found our device
			return device->name[2];
		}
	}

	return _T("Unknown Device");
}

// Get the index of the device having the ID "devID" from the list of
// known devices.  Returns -1 if the device isn't found.
int indexOfDeviceID(uint devID, uint wordSize)
{
	// First check for known bad values
	if (devID == 0xFFFF  || devID == 0x0000)
		return -1;

	// Now check against the list of processors
	int numDevs = DeviceList::deviceCount();
	uint revNum;
	uint baseDevID;

	if ((wordSize == 14) || (wordSize == 16))
	{
		baseDevID = devID & 0xffe0;
		revNum = devID & 0x001f;
	}
	else if (wordSize == 24)
	{
		baseDevID = devID & 0xffff;
		revNum = ((devID >> 21) & 0x3f);
	}
	else
	{
		baseDevID = 0;
		revNum = 0;
	}

	for (int j=0;j<numDevs;j++)
	{
		const PICDevice *device = DeviceList::Devices[j];
		if (device == NULL)
			return -1;
		uint tdevID;
		if (wordSize == 14 || wordSize == 16)
			tdevID = (device->deviceIDMask & 0xffe0); // Mask off revision bits
		else
			tdevID = (device->deviceIDMask & 0xffff); // Rev bits are in high word
		if ((device->wordSize == wordSize) && (tdevID == baseDevID))
		{
			// Found our device
			return j;
		}
	}

	return -1;
}

bool readDeviceID(CIOPort *port, uint *devID, uint *wordSize, uint *deviceIndex)
{
	int devIndex = -1;
	bool res;
	ushort configBuf[32];

	// First try treating it as a 40 pin chip

	// See if it is a 14 bit device (PIC12F/PIC16F)
	PICProgram::PICOpen(port, 0x02);
	res = PICProgram::readPIC14Config(port, &configBuf[0]);
	PICProgram::PICClose(port);
	if (res)
	{
		ushort tdevID = configBuf[6];
		devIndex = indexOfDeviceID(tdevID, 14);
		if (devIndex >= 0)
		{
			*devID = tdevID;
			*wordSize = 14;
			*deviceIndex = devIndex;
			return true;
		}
	}

	// See if it is a 24 bit device (PIC18F)
	PICProgram::PICOpen(port, 0x06);
	res = PICProgram::readPIC24DevID(port, configBuf);
	PICProgram::PICClose(port);
	if (res)
	{
		ushort tdevID = (configBuf[0] | (configBuf[1] << 16));
		devIndex = indexOfDeviceID(tdevID, 24);
		if (devIndex >= 0)
		{
			*devID = tdevID;
			*wordSize = 24;
			*deviceIndex = devIndex;
			return true;
		}
	}

	// See if it is a 16 bit device (PIC18F)
	PICProgram::PICOpen(port, 0x02);
	res = PICProgram::readPIC16DevID(port, configBuf);
	PICProgram::PICClose(port);
	if (res)
	{
		ushort tdevID = configBuf[0];
		devIndex = indexOfDeviceID(tdevID, 16);
		if (devIndex >= 0)
		{
			*devID = tdevID;
			*wordSize = 16;
			*deviceIndex = devIndex;
			return true;
		}
	}


	// Now look at 8-28 pin devices (14 and 16 bit only - dsPICs should
	// all respond to the single query above).

	// See if it is a 14 bit device (PIC12F/PIC16F)
	PICProgram::PICOpen(port, 0x03); // Turn on both Vpp lines
	res = PICProgram::readPIC14Config(port, configBuf);
	PICProgram::PICClose(port);
	if (res)
	{
		ushort tdevID = configBuf[6];
		devIndex = indexOfDeviceID(tdevID, 14);
		if (devIndex >= 0)
		{
			*devID = tdevID;
			*wordSize = 14;
			*deviceIndex = devIndex;
			return true;
		}
	}

	// See if it is a 24 bit device (PIC18F)
	PICProgram::PICOpen(port, 0x06);
	res = PICProgram::readPIC24DevID(port, configBuf);
	PICProgram::PICClose(port);
	if (res)
	{
		ushort tdevID = (configBuf[0] | (configBuf[1] << 16));
		devIndex = indexOfDeviceID(tdevID, 24);
		if (devIndex >= 0)
		{
			*devID = tdevID;
			*wordSize = 24;
			*deviceIndex = devIndex;
			return true;
		}
	}

	// See if it is a 16 bit device (PIC12F/PIC16F)
	PICProgram::PICOpen(port, 0x03);
	res = PICProgram::readPIC16DevID(port, configBuf);
	PICProgram::PICClose(port);
	if (res)
	{
		ushort tdevID = configBuf[0];
		devIndex = indexOfDeviceID(tdevID, 16);
		if (devIndex >= 0)
		{
			*devID = tdevID;
			*wordSize = 16;
			*deviceIndex = devIndex;
			return true;
		}
	}

	return false;
}

LRESULT CMainFrame::OnProcessorDropDown(UINT uMsg, LPNMHDR lParam, BOOL& /*bHandled*/)
{
	NMTOOLBAR *tb = (NMTOOLBAR *)lParam;
	DWORD pos = GetMessagePos();
	POINT pt;
	pt.x = LOWORD(pos);
	pt.y = HIWORD(pos);

	if (tb->iItem == ID_PROCESSOR_COMBO)
	{
		int item = m_DeviceListMenu.TrackPopupMenu(TPM_LEFTALIGN | TPM_RETURNCMD, pt.x, pt.y, m_hWnd);
		if (item > 0)
		{
			replaceDevice(DeviceList::Devices[item-1]);
			return 0;
		}
		else
		{
			int err = GetLastError();
			return 0;
		}
	}
	else if (tb->iItem == ID_BUTTON_COM_PORT)
	{
		int item = m_ComPortMenu.TrackPopupMenu(TPM_LEFTALIGN | TPM_RETURNCMD, pt.x, pt.y, m_hWnd);
		if (item > 0)
		{
			// Uncheck all ports
#if defined(_WIN32_WCE)
			uint portCount = m_ComPortMenuCount;
#else
			uint portCount = m_ComPortMenu.GetMenuItemCount();
#endif
			for (uint pIndx=0;pIndx<portCount;pIndx++)
				m_ComPortMenu.CheckMenuItem(pIndx, MF_BYPOSITION | MF_UNCHECKED);
			// Check the one that was just selected
			m_ComPortMenu.CheckMenuItem(item-1, MF_BYPOSITION | MF_CHECKED);

#if defined(_WIN32_WCE)
			LPTSTR lpstr = new TCHAR[64];
			MENUITEMINFO mii = { 0 };
			mii.cbSize = sizeof(mii);
			mii.fMask = MIIM_TYPE;
			mii.fType = MFT_STRING;
			mii.cch = 63;
			mii.dwTypeData = lpstr;
			int nRet = m_ComPortMenu.GetMenuItemInfo(item-1, MF_BYPOSITION, &mii);
#else
			int itemLen = m_ComPortMenu.GetMenuStringLen(item-1, MF_BYPOSITION) + 1; // Add room for '\0'
			LPTSTR lpstr = new TCHAR[itemLen];
			int nRet = m_ComPortMenu.GetMenuString(item-1, lpstr, itemLen, MF_BYPOSITION);
#endif

			m_Defaults.setCOMPort(lpstr);
			m_PortName = lpstr;
			// delete []lpstr;

			setStatusPane(PORT_STATUS_PANE, m_PortName);

			return 0;
		}
		else
		{
			int err = GetLastError();
			return 0;
		}
	}

	return 0;
}


LRESULT CMainFrame::OnProcessorCombo(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	// See if we can autodetect what chip is in the programmer.

	// Try reading the device ID
	bool res = false;
	uint devID = 0, wordSize = 0, deviceIndex = -1;
	if (openCOMPort(BAUD_RATE))
	{
		res = readDeviceID(&m_Port, &devID, &wordSize, &deviceIndex);
		m_Port.Close();
	}
	if (res)
	{
		replaceDevice(DeviceList::Devices[deviceIndex]);
	}
	else
	{
		MessageBox(_T("Failed to determine processor ID"));
	}

	return 0;
}

LRESULT CMainFrame::OnComPortButton(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	return 0;
}

LRESULT CMainFrame::OnFileExit(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	PostMessage(WM_CLOSE);
	return 0;
}

LRESULT CMainFrame::OnFileNew(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	// Which tab is active?
	int activePage = getActiveTabPage();

	// Device and programming code stay the same.  Just change the hex file.
	if (activePage == 0)
	{
		delete m_HexFile1;
		m_HexFile1 = new PICHexFile(m_Device);
		displayHexFile(m_view.m_hexPage1, m_HexFile1);
	}
	else if (activePage == 1)
	{
		delete m_HexFile2;
		m_HexFile2 = new PICHexFile(m_Device);
		displayHexFile(m_view.m_hexPage2, m_HexFile2);
	}

	return 0;
}

LRESULT CMainFrame::OnFileOpen(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
#if defined(_WIN32_WCE)
	CFileOpenDlg openDlg(TRUE, _T("hex"), NULL,
		OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST,
		_T("Hex Files|*.hex||All Files|*.*||\0\0"),
		this);
#else
	CFileDialog openDlg(TRUE, _T("hex"), NULL, 0, _T("Hex Files\0*.hex\0All Files\0*.*\0\0"));
#endif

	HRESULT hRes = (HRESULT)openDlg.DoModal();
	if (hRes == IDOK)
	{
#if defined(_WIN32_WCE)
		CString pathName = openDlg.GetPathName();
#else
		CString pathName = openDlg.m_szFileName;
#endif

		if (::_tcslen(pathName) > 0)
		{
			PICHexFile *newHexFile = new PICHexFile(m_Device);
			int res = newHexFile->LoadFromFile(pathName);
			if (res >= 0)
			{
				int activePage = getActiveTabPage();
				if (activePage == 0)
				{
					delete m_HexFile1;
					m_HexFile1 = newHexFile;
					displayHexFile(m_view.m_hexPage1, m_HexFile1);
				}
				else if (activePage == 1)
				{
					delete m_HexFile2;
					m_HexFile2 = newHexFile;
					displayHexFile(m_view.m_hexPage2, m_HexFile2);
				}
			}
			else
			{
				delete newHexFile;
				MessageBox(_T("Failed to load hex file"));
			}
		}
	}

	return 0;
}

LRESULT CMainFrame::OnFileSave(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	// Get a pointer to the hex file in the current tab page
	PICHexFile *hexFile = NULL;
	int activePage = getActiveTabPage();
	if (activePage == 0)
		hexFile = m_HexFile1;
	else if (activePage = 1)
		hexFile = m_HexFile2;
	if (hexFile == NULL)
	{
		MessageBox(_T("No hex file to write"));
		return 0;
	}

	// Modified (tree-view) file save dialog
#if defined(_WIN32_WCE)
	CFileOpenDlg saveDlg(FALSE, _T("hex"), NULL,
		OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
		_T("Hex Files|*.hex|All Files|*.*||\0\0"), NULL);
#else
	CFileDialog saveDlg(FALSE, _T("hex"), NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
		_T("Hex Files\0*.hex\0All Files\0*.*\0\0"));
#endif

	HRESULT hRes = (HRESULT)saveDlg.DoModal();
	if (SUCCEEDED(hRes))
	{
#if defined(_WIN32_WCE)
		CString pathName = saveDlg.GetPathName();
#else
		CString pathName = saveDlg.m_szFileName;
#endif
		if (::_tcslen(pathName) > 0)
		{
			HexFileIO hexIO;
			hRes = hexIO.Save(pathName, hexFile);
			if (hRes != 0)
			{
				MessageBox(_T("Failed to save hex file"));
			}
		}
	}

	return 0;
}

LRESULT CMainFrame::OnFileSaveAs(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	// Get a pointer to the hex file in the current tab page
	PICHexFile *hexFile = NULL;
	int activePage = getActiveTabPage();
	if (activePage == 0)
		hexFile = m_HexFile1;
	else if (activePage = 1)
		hexFile = m_HexFile2;
	if (hexFile == NULL)
	{
		MessageBox(_T("No hex file to write"));
		return 0;
	}

	// Modified (tree-view) file save dialog
#if defined(_WIN32_WCE)
	CFileOpenDlg saveDlg(FALSE, _T("hex"), NULL,
		OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
		_T("Hex Files|*.hex|All Files|*.*||\0\0"), NULL);
#else
	CFileDialog saveDlg(FALSE, _T("hex"), NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
		_T("Hex Files\0*.hex\0All Files\0*.*\0\0"));
#endif

	HRESULT hRes = (HRESULT)saveDlg.DoModal();
	if (SUCCEEDED(hRes))
	{
#if defined(_WIN32_WCE)
		CString pathName = saveDlg.GetPathName();
#else
		CString pathName = saveDlg.m_szFileName;
#endif
		if (::_tcslen(pathName) > 0)
		{
			HexFileIO hexIO;
			hRes = hexIO.Save(pathName, hexFile);
			if (hRes != 0)
			{
				MessageBox(_T("Failed to save hex file"));
			}
		}
	}

	return 0;
}

LRESULT CMainFrame::OnViewToolBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	static BOOL bVisible = TRUE;	// initially visible
	bVisible = !bVisible;
	CReBarCtrl rebar = m_hWndToolBar;
	int nBandIndex = rebar.IdToIndex(ATL_IDW_BAND_FIRST + 1);	// toolbar is 2nd added band
	rebar.ShowBand(nBandIndex, bVisible);
	UISetCheck(ID_VIEW_TOOLBAR, bVisible);
	UpdateLayout();
	return 0;
}

LRESULT CMainFrame::OnViewStatusBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	BOOL bVisible = !::IsWindowVisible(m_hWndStatusBar);
	::ShowWindow(m_hWndStatusBar, bVisible ? SW_SHOWNOACTIVATE : SW_HIDE);
	UISetCheck(ID_VIEW_STATUS_BAR, bVisible);
	UpdateLayout();
	return 0;
}


LRESULT CMainFrame::OnViewVppstatus(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	m_PollStatusFlag = !m_PollStatusFlag;

	CMenuHandle formMenu;
#if defined(_WIN32_WCE)
#else
	formMenu = m_CmdBar.GetMenu();
#endif
	formMenu.CheckMenuItem(ID_VIEW_VPPSTATUS, (m_PollStatusFlag ? MF_CHECKED : MF_UNCHECKED));

	if (!m_PollStatusFlag)
	{
		// If we aren't tracking Vpp then clear the status pane (otherwise we will
		// continue to see the last value read).
		setStatusPane(VPP_STATUS_PANE, _T(""));
	}
	return 0;
}

LRESULT CMainFrame::OnAppAbout(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	CAboutDlg dlg;
	dlg.DoModal();
	return 0;
}

#define MIN_HEIGHT 512
#define MIN_WIDTH 472

#if defined(_WIN32_WCE)
#else
LRESULT CMainFrame::OnSizing(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	LPRECT pRect = (LPRECT)lParam;
	int width = pRect->right - pRect->left;
	int height = pRect->bottom - pRect->top;

	//if (width >= MIN_WIDTH && height >= MIN_HEIGHT)
	//	return 0;

	if (width < MIN_WIDTH || width > MIN_WIDTH)
	{
		switch (wParam)
		{
		case WMSZ_LEFT:
		case WMSZ_BOTTOMLEFT:
		case WMSZ_TOPLEFT:
			pRect->left = pRect->right - MIN_WIDTH;
			break;
		case WMSZ_RIGHT:
		case WMSZ_BOTTOMRIGHT:
		case WMSZ_TOPRIGHT:
			pRect->right = pRect->left + MIN_WIDTH;
			break;
		}
	}

	if (height < MIN_HEIGHT || height > MIN_HEIGHT)
	{
		switch (wParam)
		{
		case WMSZ_TOP:
		case WMSZ_TOPLEFT:
		case WMSZ_TOPRIGHT:
			pRect->top = pRect->bottom - MIN_HEIGHT;
			break;
		case WMSZ_BOTTOM:
		case WMSZ_BOTTOMLEFT:
		case WMSZ_BOTTOMRIGHT:
			pRect->bottom = pRect->top + MIN_HEIGHT;
			break;
		}
	}

	// bHandled = TRUE;
	return 1;
}
#endif

// Change the current device to a new one.  If the new device isn't the
// same as the old one, clear the contents of the hex file.
void CMainFrame::replaceDevice(const PICDevice *device)
{
	if (device != m_Device)
	{
		int oldIndex = DeviceList::DeviceIndex(m_Device->name[2]);
		// m_DeviceListMenu.CheckMenuItem(oldIndex, MF_BYPOSITION | MF_UNCHECKED);
		// Uncheck the old device
		if ((m_Device->name[0][1] == _T('3')) && (m_Device->name[0][2] == _T('0')))
			P30Menu.CheckMenuItem(oldIndex+1, MF_UNCHECKED);
		else if (m_Device->name[0][2] == _T('2'))
			P12Menu.CheckMenuItem(oldIndex+1, MF_UNCHECKED);
		else if (m_Device->name[0][2] == _T('6'))
			P16Menu.CheckMenuItem(oldIndex+1, MF_UNCHECKED);
		else if (m_Device->name[0][2] == _T('8'))
			P18Menu.CheckMenuItem(oldIndex+1, MF_UNCHECKED);

		m_Device = device;

		m_Defaults.setProcessorName(m_Device->name[2]);

		setStatusPane(PROCESSOR_STATUS_PANE, m_Device->name[2]);

		int newIndex = DeviceList::DeviceIndex(m_Device->name[2]);
		// m_DeviceListMenu.CheckMenuItem(newIndex, MF_BYPOSITION | MF_CHECKED);
		if ((m_Device->name[0][1] == _T('3')) && (m_Device->name[0][2] == _T('0')))
			P30Menu.CheckMenuItem(oldIndex+1, MF_CHECKED);
		else if (m_Device->name[0][2] == _T('2'))
			P12Menu.CheckMenuItem(newIndex+1, MF_CHECKED);
		else if (m_Device->name[0][2] == _T('6'))
			P16Menu.CheckMenuItem(newIndex+1, MF_CHECKED);
		else if (m_Device->name[0][2] == _T('8'))
			P18Menu.CheckMenuItem(newIndex+1, MF_CHECKED);

		m_Driver = ProgramSelector::GetProgrammer(device);

		// Clear out all hex tab pages
		delete m_HexFile1;
		m_HexFile1 = new PICHexFile(device);
		m_view.m_hexPage1->m_hexPanel.mCodeDisplay = false;
		displayHexFile(m_view.m_hexPage1, m_HexFile1);
		delete m_HexFile2;
		m_HexFile2 = new PICHexFile(device);
		m_view.m_hexPage2->m_hexPanel.mCodeDisplay = false;
		displayHexFile(m_view.m_hexPage2, m_HexFile2);
	}
}

void CMainFrame::displayHexFile(ChexPageDlg *page, PICHexFile *hexFile)
{
	updateCodeFromFile(page, hexFile);
	updateDataFromFile(page, hexFile);
	updateConfigFromFile(page, hexFile);
}

void CMainFrame::updateCodeFromFile(ChexPageDlg *hexPage, PICHexFile *hexFile)
{
	CEdit &editCtrl =hexPage->m_hexPanel;

	// Remove old contents.
	editCtrl.Clear();
#if 1
	if (hexPage->m_hexPanel.mCodeDisplay)
	{
		// Show the hex file as assembler
		CTextList *asmCode = hexFile->Disassemble(true);

		// If disassembly failed, then asmCode will be NULL
		if (asmCode != NULL)
		{
			// Step through each line of the disassembled code, building
			// a single string for the output.
			CTextList::iterator asmItr;
#if 0
			CString asmText;
			for (asmItr=asmCode->begin();asmItr!=asmCode->end();asmItr++)
			{
				asmText += *asmItr;
				asmText += _T("\r\n");
			}
			// Don't need the disassembler output any more.
			delete asmCode;

			// Put the output buffer into the text control
			editCtrl.SetWindowText(asmText);
#else
			TCHAR *asmText = NULL;
			size_t asmTextLen = 0;
			for (asmItr=asmCode->begin();asmItr!=asmCode->end();asmItr++)
			{
				// Append the line to the output buffer
				TCHAR *lineBuf = *asmItr;
				if (lineBuf != NULL)
				{
					size_t lineBufLen = ::_tcslen(lineBuf);
					if (asmText == NULL)
					{
						asmText = new TCHAR[lineBufLen + 3]; // Extra char for terminatin '\0'
						_tcscpy(asmText, lineBuf);
						_tcscpy(asmText + lineBufLen, _T("\r\n"));
						asmTextLen = lineBufLen + 2;
					}
					else
					{
						TCHAR *oldBuf = asmText;
						asmText = new TCHAR[asmTextLen + lineBufLen + 3];
						_tcscpy(asmText, oldBuf);
						_tcscpy(asmText+asmTextLen, lineBuf);
						_tcscpy(asmText+asmTextLen+lineBufLen, _T("\r\n"));
						delete[] oldBuf;
						asmTextLen += lineBufLen + 2;
					}
				}
			}
			// Don't need the disassembler output any more.
			delete asmCode;

			// Put the output buffer into the text control
			editCtrl.SetWindowText(asmText);

			// Free memory associated with output buffer
			delete[] asmText;

#endif
			return;
		}

		// Failed to disassemble - revert to hex display.
		hexPage->m_hexPanel.mCodeDisplay = false;
	}
	else
	{
		// Dump contents into edit 
	}
#endif
	uint wordSize = hexFile->Processor->wordSize;

	// Fill the program view with values (8 per line)
#if defined(_WIN32_WCE)
	uint wordsPerLine = 4;
#else
	uint wordsPerLine = 8;
#endif
	// TBD: This is very slow - build the string in a smarter way.
	uint numLines = (uint)(hexFile->Processor->programWordsSize / wordsPerLine);
	uint lineLength = 6 + wordsPerLine * 5 + 2;
	int pAddr = 0;
	int outLen = numLines * lineLength + 1;
	TCHAR *outBuf = new TCHAR[outLen];
	TCHAR hexStr[16];
	int offset = 0;
	// char c0 = 'F', c1 = 'F', c2 = 'F', c3 = 'F';
	for (uint pRow=0;pRow<numLines;pRow++)
	{
		uint ctr;
		bool hasContent = false;
		for (ctr=0;ctr<wordsPerLine&&!hasContent;ctr++)
			if (hexFile->programWordsUsed[pAddr+ctr])
				hasContent = true;

		_stprintf(hexStr, _T("%04X"), pAddr);
		// String line = String.Format("{0:X4}: ", pAddr);
		outBuf[offset++] = hexStr[0]; outBuf[offset++] = hexStr[1]; outBuf[offset++] = hexStr[2];
		outBuf[offset++] = hexStr[3]; outBuf[offset++] = ':'; outBuf[offset++] = ' ';

		if (hasContent)
		{
			for (ctr=0;ctr<wordsPerLine;ctr++)
			{
				_stprintf(hexStr, _T("%04X"), hexFile->programWords[pAddr]);
				outBuf[offset++] = hexStr[0];
				outBuf[offset++] = hexStr[1];
				outBuf[offset++] = hexStr[2];
				outBuf[offset++] = hexStr[3];
				if (ctr < (wordsPerLine-1))
					outBuf[offset++] = ' ';
				// line += String.Format("{0:X4} ", hexFile.programWords[pAddr]);
				pAddr++;
			}
		}
		else
		{
			for (ctr=0;ctr<wordsPerLine;ctr++)
			{
				if (wordSize == 14)
					outBuf[offset++] = _T('3');
				else
					outBuf[offset++] = _T('F');
				outBuf[offset++] = _T('F');
				outBuf[offset++] = _T('F');
				outBuf[offset++] = _T('F');
				if (ctr < (wordsPerLine-1))
					outBuf[offset++] = ' ';
				// line += String.Format("{0:X4} ", hexFile.programWords[pAddr]);
				pAddr++;
			}
		}

		outBuf[offset++] = _T('\r');
		outBuf[offset++] = _T('\n');
	}
	outBuf[offset++] = 0;

	editCtrl.SetWindowText(outBuf);

	delete[] outBuf;
}

void CMainFrame::updateDataFromFile(ChexPageDlg *hexPage, PICHexFile *hexFile)
{
	CEdit &editCtrl = hexPage->m_dataPanel;

	// Remove old contents.
	editCtrl.Clear();

	uint wordSize = hexFile->Processor->wordSize;

	// Fill the program view with values (8 per line)
#if defined(_WIN32_WCE)
	uint wordsPerLine = 4;
#else
	uint wordsPerLine = ((wordSize <= 16) ? 8 : 4);
#endif
	// TBD: This is very slow - build the string in a smarter way.
	uint numLines = (uint)(hexFile->Processor->EEPROMDataSize / wordsPerLine);
	uint lineLength = 6 + wordsPerLine * 3 + 4 + wordsPerLine * (wordSize / 2) + 2;
	int pAddr = 0;
	int outLen = numLines * lineLength + 1;
	TCHAR *outBuf = new TCHAR[outLen];
	TCHAR hexStr[16];
	int offset = 0;
	// char c0 = 'F', c1 = 'F', c2 = 'F', c3 = 'F';
	for (uint pRow=0;pRow<numLines;pRow++)
	{
		uint ctr;

		_stprintf(hexStr, _T("%04X"), pAddr);
		// String line = String.Format("{0:X4}: ", pAddr);
		outBuf[offset++] = hexStr[0]; outBuf[offset++] = hexStr[1]; outBuf[offset++] = hexStr[2];
		outBuf[offset++] = hexStr[3]; outBuf[offset++] = ':'; outBuf[offset++] = ' ';

		for (ctr=0;ctr<wordsPerLine;ctr++)
		{
			if (wordSize <= 16)
				// Smaller PICs are one byte per entry in the EEPROM
				_stprintf(hexStr, _T("%02X"), (byte)(hexFile->EEPROMData[pAddr+ctr] & 0xff));
			else
				// dsPIC uses a 16 bit word in the EEPROM
				_stprintf(hexStr, _T("%04X"), hexFile->EEPROMData[pAddr+ctr]);
			outBuf[offset++] = hexStr[0];
			outBuf[offset++] = hexStr[1];
			if (wordSize > 16)
			{
				outBuf[offset++] = hexStr[2];
				outBuf[offset++] = hexStr[3];
			}
			if (ctr < (wordsPerLine-1))
				outBuf[offset++] = ' ';
			// line += String.Format("{0:X4} ", hexFile.programWords[pAddr]);
		}

		outBuf[offset++] = _T(' ');
		outBuf[offset++] = _T(' ');
		outBuf[offset++] = _T(' ');
		outBuf[offset++] = _T(' ');

		for (ctr=0;ctr<wordsPerLine;ctr++)
		{
			ushort val = hexFile->EEPROMData[pAddr+ctr];
			if (val < _T(' ') || val > _T('~'))
				outBuf[offset++] = '.';
			else
			{
				_stprintf(hexStr, _T("%c"), val);
				outBuf[offset++] = hexStr[0];
			}
		}

		outBuf[offset++] = _T('\r');
		outBuf[offset++] = _T('\n');

		pAddr += wordsPerLine;
	}
	outBuf[offset++] = 0;

	editCtrl.SetWindowText(outBuf);
}

void CMainFrame::updateConfigFromFile(ChexPageDlg *hexPage, PICHexFile *hexFile)
{
	uint i;

	TCHAR hexStr[16];
	uint numConfigEditElems = sizeof(hexPage->m_configWords)/sizeof(hexPage->m_configWords[0]);
	for (i=0;i<hexFile->Processor->configWordsSize&&i<numConfigEditElems;i++)
	{
		CEdit &editCtrl = hexPage->m_configWords[i];
		_stprintf(hexStr, _T("%04X"), hexFile->configWords[i]);
		editCtrl.SetWindowText(hexStr);
		editCtrl.EnableWindow(TRUE);
	}
	// Clear and disable remaining config entries
	for (;i<numConfigEditElems;i++)
	{
		CEdit &editCtrl = hexPage->m_configWords[i];
		editCtrl.SetWindowText(_T("0000"));
		editCtrl.EnableWindow(FALSE);
	}
	// Update user ID entries
	uint numUserIDEditElems = sizeof(hexPage->m_userID)/sizeof(hexPage->m_userID[0]);
	for (i=0;i<hexFile->Processor->userIDWordsSize&&i<numUserIDEditElems;i++)
	{
		CEdit &editCtrl = hexPage->m_userID[i];
		_stprintf(hexStr, _T("%04X"), hexFile->userIDWords[i]);
		editCtrl.SetWindowText(hexStr);
		editCtrl.EnableWindow(TRUE);
	}
	// Clear and disable remaining config entries
	for (;i<numUserIDEditElems;i++)
	{
		CEdit &editCtrl = hexPage->m_userID[i];
		editCtrl.SetWindowText(_T("0000"));
		editCtrl.EnableWindow(FALSE);
	}

	CEdit &editDevID0Ctrl = hexPage->m_deviceID[0];
	_stprintf(hexStr, _T("%04X"), hexFile->deviceID[0]);
	editDevID0Ctrl.SetWindowText(hexStr);
	editDevID0Ctrl.EnableWindow(TRUE);
	CEdit &editDevID1Ctrl = hexPage->m_deviceID[1];
	_stprintf(hexStr, _T("%04X"), hexFile->deviceID[1]);
	editDevID1Ctrl.SetWindowText(hexStr);
	editDevID1Ctrl.EnableWindow(TRUE);

}

void CMainFrame::updateFileFromConfig(ChexPageDlg *hexPage, PICHexFile *hexFile)
{
	uint i;

	uint numConfigEditElems = sizeof(hexPage->m_configWords)/sizeof(hexPage->m_configWords[0]);
	for (i=0;i<hexFile->Processor->configWordsSize&&i<numConfigEditElems;i++)
	{
		hexFile->configWords[i] = hexPage->m_configWords[i].HexValue();
	}
	// Update user ID entries
	uint numUserIDEditElems = sizeof(hexPage->m_userID)/sizeof(hexPage->m_userID[0]);
	for (i=0;i<hexFile->Processor->userIDWordsSize&&i<numUserIDEditElems;i++)
	{
		hexFile->userIDWords[i] = hexPage->m_userID[i].HexValue();
	}
}

#define READ_CHIP_REQUEST  0
#define WRITE_CHIP_REQUEST 1
#define ERASE_CHIP_REQUEST 2
typedef struct _stWorkerThreadInfo {
	CMainFrame *mainWnd;
	PICHexFile *hexFile;
	int activeTabPage;
	int programChip;    // 0=read, 1=program, 2=erase
	bool bootFlag;      // True if operation should include boot block
	bool codeFlag;      // True if operation should include code memory
	bool dataFlag;      // True if operation should include data memory
	bool configFlag;    // True if operation should include config words
} _WorkerThreadInfo;

// Singleton variable to hold programming request.
_WorkerThreadInfo workerInfo;


#if defined(_WIN32_WCE)
DWORD
#else
unsigned __stdcall
#endif
programChipThreadProc(void *arg)
{
	_WorkerThreadInfo *workerInfo = (_WorkerThreadInfo *)arg;
	CMainFrame *mainWnd = workerInfo->mainWnd;
	PICHexFile *hexFile = workerInfo->hexFile;
	int activeTabPage = workerInfo->activeTabPage;
	int programFlag = workerInfo->programChip;

	HANDLE hMutex = CreateMutex(NULL, FALSE, _T("COMPortMutex"));
	if (hMutex == NULL)
		// No mutex to keep this thread from trampling on the main thread
		return -1;

	// Request ownership of mutex.
	DWORD dwWaitResult = WaitForSingleObject(hMutex, 5000L);
	switch (dwWaitResult) 
	{
		// The thread got mutex ownership.
		case WAIT_OBJECT_0: 
            break; 
		// Cannot get mutex ownership due to time-out.
		case WAIT_TIMEOUT: 
			// TBD: Put up an error box to say that we got blocked from reading
			// so user doesn't think there's a problem with the programmer.
			if (mainWnd->m_ProgressDlg != NULL)
				// The progress dialog may already have been cancelled out from
				// under us.  If not, then we explicitly cancel it.
				mainWnd->m_ProgressDlg.EndDialog(IDCANCEL);
			mainWnd->MessageBox(_T("Failed to connect to programmer"));
			return -1;
		// Got ownership of the abandoned mutex object.
		case WAIT_ABANDONED: 
			break; 
	}
	if (!mainWnd->openCOMPort(BAUD_RATE))
	{
		ReleaseMutex(hMutex);
		mainWnd->m_ProgressDlg.EndDialog(IDCANCEL);
		return -1;
	}

	// uint r0;
	// int r1 = PICProgram::TxCmd(&mainWnd->m_Port, PICProgram::EVA_VERSION_CMD, 0, &r0);

	int tRes;
	if (programFlag == WRITE_CHIP_REQUEST)
	{
		if (!workerInfo->codeFlag || !workerInfo->dataFlag || !workerInfo->configFlag)
		{
			tRes = mainWnd->m_Driver->PartialProgram(&mainWnd->m_Port, hexFile,
				workerInfo->codeFlag, workerInfo->dataFlag, workerInfo->configFlag,
				&mainWnd->m_ProgressDlg);
		}
		else
		{
			tRes = mainWnd->m_Driver->Program(&mainWnd->m_Port, hexFile, &mainWnd->m_ProgressDlg);
		}
	}
	else if (programFlag == READ_CHIP_REQUEST)
	{
		if (!workerInfo->codeFlag || !workerInfo->dataFlag || !workerInfo->configFlag)
		{
			tRes = mainWnd->m_Driver->PartialRead(&mainWnd->m_Port, hexFile,
				workerInfo->codeFlag, workerInfo->dataFlag, workerInfo->configFlag,
				&mainWnd->m_ProgressDlg);
		}
		else
		{
			tRes = mainWnd->m_Driver->Read(&mainWnd->m_Port, hexFile, &mainWnd->m_ProgressDlg);
		}
	}
	else if (programFlag == ERASE_CHIP_REQUEST)
	{
		// Erase the chip
		if (workerInfo->codeFlag && workerInfo->dataFlag && workerInfo->configFlag)
			tRes = mainWnd->m_Driver->Erase(&mainWnd->m_Port, hexFile, &mainWnd->m_ProgressDlg);
		else
			tRes = mainWnd->m_Driver->PartialErase(&mainWnd->m_Port, hexFile, workerInfo->bootFlag, workerInfo->codeFlag, workerInfo->dataFlag, workerInfo->configFlag, &mainWnd->m_ProgressDlg);
	}
	else
	{
		// Unknown command
	}

	if (tRes != 0)
	{
		if (mainWnd->m_ProgressDlg.IsWindow())
		{
			mainWnd->m_ProgressDlg.EndDialog(IDCANCEL);
			// Only cancel if the dialog is still active.
			mainWnd->m_ProgressDlg.errVal = tRes;
			// Read failed - say something about it?
			TCHAR errBuf[128];
			_stprintf(errBuf, _T("%s failed at addr: 0x%04X"),
				(programFlag == 1 ? _T("Write") : (programFlag == 0 ? _T("Read") : _T("Erase"))), tRes-1);
		}
	}
	else
	{
		mainWnd->m_ProgressDlg.EndDialog(IDOK);
	}

	mainWnd->m_Port.Close();

	ReleaseMutex(hMutex);

	return 0;
}

int CMainFrame::ChipCommand(int programFlag, bool bootFlag, bool codeFlag, bool dataFlag, bool configFlag)
{
	// Don't do a read if the programmer is in use.
	if (m_ProgrammerActive)
		return 0;

	m_ProgrammerActive = true;

	PICHexFile *hexFile = getActiveHexFile();
	int activePage = getActiveTabPage();
	if (programFlag && configFlag)
	{
		ChexPageDlg *hexPage = (activePage == 0 ? m_view.m_hexPage1 : m_view.m_hexPage2);
		updateFileFromConfig(hexPage, hexFile);
	}

	// Package info needed by worker thread
	workerInfo.hexFile = hexFile;
	workerInfo.activeTabPage = activePage;
	workerInfo.mainWnd = this;
	workerInfo.programChip = programFlag; // Is this a read, program, or erase command?
	workerInfo.bootFlag = bootFlag;
	workerInfo.codeFlag = codeFlag;
	workerInfo.dataFlag = dataFlag;
	workerInfo.configFlag = configFlag;

	// Read the chip
#if defined(_WIN32_WCE)
	// No support for _beginthread() in WinCE - need to use the older CreateThread()
	// API.  TBD: ensure everything gets cleaned up.
	DWORD threadID;
	HANDLE threadHandle = CreateThread(NULL, NULL, programChipThreadProc, &workerInfo, 0, &threadID);
#else
	HANDLE threadHandle = (HANDLE)_beginthreadex(NULL, 0, programChipThreadProc, &workerInfo, 0, NULL);
#endif

	HRESULT hRes = (HRESULT)m_ProgressDlg.DoModal(m_hWnd);
	if ((programFlag == READ_CHIP_REQUEST) && (true || (hRes == IDOK)))
	{
		// TBD: For now load what we read, even if the operation was aborted.
		// Mark entire address space as used
		for (uint addr=0;addr<m_Device->programWordsSize;addr++)
		{
			hexFile->programWordsUsed[addr] = true;
		}
		hexFile->programWordsBase = 0;
		hexFile->programWordsCeil = m_Device->programWordsSize-1;

		// Update the hex view
		if (activePage == 0)
			displayHexFile(m_view.m_hexPage1, hexFile);
		else
			displayHexFile(m_view.m_hexPage2, hexFile);
	}
	
	if (hRes != IDOK)
	{
		int errVal = m_ProgressDlg.errVal;
		TCHAR errBuf[128];
		_stprintf(errBuf, _T("%s failed at addr: 0x%04X"),
			(programFlag == 1 ? _T("Write") : (programFlag == 0 ? _T("Read") : _T("Erase"))), errVal-1);
		MessageBox(errBuf);
	}

	m_ProgrammerActive = false;

	return 0;
}

LRESULT CMainFrame::OnReadCode(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	ChipCommand(0, true, true, false, false);

	return 0;
}

LRESULT CMainFrame::OnReadConfig(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	ChipCommand(0, false, false, false, true);

	return 0;
}

LRESULT CMainFrame::OnReadData(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	ChipCommand(0, false, false, true, false);

	return 0;
}

LRESULT CMainFrame::OnWriteCode(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	ChipCommand(1, true, true, false, false);

	return 0;
}

LRESULT CMainFrame::OnWriteConfig(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	ChipCommand(1, false, false, false, true);

	return 0;
}

LRESULT CMainFrame::OnWriteData(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	ChipCommand(1, false, false, true, false);

	return 0;
}

LRESULT CMainFrame::OnChipErase(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	ChipCommand(2, true, true, true, true);

	return 0;
}

LRESULT CMainFrame::OnChipProgram(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	ChipCommand(1, true, true, true, true);

	return 0;
}

LRESULT CMainFrame::OnChipRead(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	ChipCommand(0, true, true, true, true);

	return 0;
}


LRESULT CMainFrame::OnEraseBoot(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	ChipCommand(2, true, false, false, false);

	return 0;
}

LRESULT CMainFrame::OnEraseNonBoot(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	ChipCommand(2, false, true, false, false);

	return 0;
}

LRESULT CMainFrame::OnEraseCode(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	ChipCommand(2, true, true, false, false);

	return 0;
}

LRESULT CMainFrame::OnEraseData(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	ChipCommand(2, false, false, true, false);

	return 0;
}

int CMainFrame::getActiveTabPage(void)
{
	CTabCtrl &m_tabCtrl	= m_view.m_tabCtrl;
	int activePage = m_tabCtrl.GetCurSel();

	return activePage;
}

PICHexFile *CMainFrame::getActiveHexFile(void)
{
	// CTabCtrl &m_tabCtrl	= m_view.m_tabCtrl;
	// int activePage = m_tabCtrl.GetCurSel();
	int activePage = m_view.m_tabCtrl.GetCurSel();
	if (activePage == 0)
		return m_HexFile1;
	else if (activePage == 1)
		return m_HexFile2;
	else
		return NULL;
}

bool CMainFrame::openCOMPort(int baudRate)
{
	// If the com port is currently open (e.g., for testing purposes) then
	// close it.
	if (m_Port.IsOpen())
	{
		if (!SUCCEEDED(m_Port.Close()))
		{
			// Failed to close the port
			return false;
		}
	}

	// Open the com port
	if (::_tcslen(m_PortName) <= 0)
	{
		// Bad COM port name
		return false;
	}

	return m_Port.Open(m_PortName, baudRate);
}

void CMainFrame::setStatusPane(int pane, LPCTSTR paneText)
{
	CStatusBarCtrl statusBar;
	statusBar.Attach(m_hWndStatusBar);
	statusBar.SetText(pane, paneText, 0);
}

LRESULT CMainFrame::OnSize(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
{
	bHandled = FALSE;

	return 0;
}

LRESULT CMainFrame::OnGetMinMaxInfo(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	// WM_GETMINMAXINFO isn't processed on the PocketPC so we need to only
	// enable the code for standard Windows.
#if !defined(_WIN32_WCE)
	MINMAXINFO *mmi = (MINMAXINFO *)lParam;

	mmi->ptMaxSize.x = 472;
	mmi->ptMaxSize.y = 512;
	bHandled = FALSE;
#endif

	return 0;
}

// Debugging routine - this sends a command to the firmware to echo back
// whatever we send to it.  This is useful for checking that packets
// are making it back and forth uncorrupted.
LRESULT CMainFrame::OnEvaEcho(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	if (!openCOMPort(BAUD_RATE))
	{
		return -1;
	}
	static int ctr = 0;
	byte outBuf[128];
	byte inBuf[128];
	for (int i=1;i<=64;i++)
		outBuf[i] = i;
	outBuf[0] = 0x0f;
    outBuf[1] = 0x0b;
	outBuf[2] = ctr++;
	outBuf[3] = 0x00;

	uint resLen = 0;
	for (int q=0;q<4;q++)
	{
		int res = PICProgram::SendPacket(&m_Port, PICProgram::EVA_ECHO_CMD, 64, outBuf,
			64, &resLen, inBuf, 500);
		if (res >= 0)
		{
			int versionMinor = inBuf[2];
			int versionMajor = inBuf[3];
			res++;
		}
		else
		{
			res--;
		}
	}

    m_Port.Close();

	return 0;
}

// Ask the firmware for the current state of the registers.  Note that
// the order of values in the array we get back has to be the same
// as the order they are expected when building the dialog box.
LRESULT CMainFrame::OnEvaRegisters(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	if (!openCOMPort(BAUD_RATE))
	{
		return -1;
	}
	// ::Sleep(4);
	byte outBuf[128];
	byte inBuf[128];
	for (int i=1;i<=70;i++)
		outBuf[i] = i;
	outBuf[0] = 0x00;
    outBuf[1] = 0x00;
	outBuf[2] = 0x00;
	outBuf[3] = 0x00;

	uint resLen = 0;
	int res = PICProgram::SendPacket(&m_Port, PICProgram::EVA_READ_REGS_CMD, 4, outBuf,
		67, &resLen, inBuf, 4);
	if (res >= 0)
	{
		PICProgram::TxCmd(&m_Port, PICProgram::EVA_ENDPROG_CMD, 0, &resLen);

		// TBD: Populate a dialog to display the values
		CRegistersDlg dlg;
		dlg.SetValues(&inBuf[6]);
		dlg.DoModal();
	}
	else
	{
		MessageBox(_T("Failed to retrieve programmer registers"));
	}

    m_Port.Close();

	return 0;
}

// Ask the firmware what version it is.
LRESULT CMainFrame::OnEvaVersion(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	if (!openCOMPort(BAUD_RATE))
	{
		return -1;
	}
	// Decimal values of version packet: <85><85><1><255><4>
	uint result;
	int res = PICProgram::TxCmd(&m_Port, PICProgram::EVA_VERSION_CMD, 0, &result);
	if (res >= 0)
	{
		int versionMinor = (result & 0xff);
		int versionMajor = ((result & 0xff00) >> 8);
		CString verLow, verHigh;
		verLow.Format(_T("%d"), versionMinor);
		verHigh.Format(_T("%d"), versionMajor);
		CString versStr = _T("Firmware version: ") + verHigh + _T(".") + verLow;
		MessageBox(versStr);
		res++;

		PICProgram::TxCmd(&m_Port, PICProgram::EVA_ENDPROG_CMD, 0, &result);
	}
	else
	{
		MessageBox(_T("Failed to retrieve firmware version"));
		res--;
	}

	m_Port.Close();

	return 0;
}

// Lowest byte address at which firware can be automatically updated.  If
// anything below this has to be changed (e.g., the USB code), then the
// chip will need to be externally programmed.
// TBD: Determine available space for the firmware directly from target -
// should be able to query processor and firmware version without having
// to call routines in higher addresses.
#define MIN_UPDATE_ADDRESS 0x4000

// Highest byte address available for firmware updates.  TBD: update if
// the target chip changes from the 18f2455 (which has 24K of program memory).
#define MAX_UPDATE_ADDRESS 0x6000

// Size of the block of bytes to write to the target.  Note that this value
// must be a power of 2 or the update code will not work correctly.
#define UPDATE_BLOCKSIZE 0x40

LRESULT CMainFrame::OnEvaUpdate(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	int res;
	PICHexFile *hexFile = getActiveHexFile();
	uint hexStartAddress = 2 * hexFile->programWordsBase; // Byte address of first instruction
	uint hexEndAddress   = 2 * (hexFile->programWordsCeil + 1); // Byte address of last instruction

	// Start by verifying that the code in the current buffer is located
	// within the memory window used for updatable firmware.
	if (hexStartAddress < MIN_UPDATE_ADDRESS)
	{
		res = MessageBox(_T("Hex file starts below base update address.\r\n")
			_T("Continue with partial update?"), _T("Misaligned update file"),
			MB_OKCANCEL);
		if (res != 1)
			return -1;
	}

	if (hexEndAddress >= MAX_UPDATE_ADDRESS)
	{
		res = MessageBox(_T("Hex file too large for target address space.\r\n")
			_T("Continue with partial update?"), _T("Misaligned update file"),
			MB_OKCANCEL);
		if (res != 1)
			return -1;
	}

	if (!openCOMPort(BAUD_RATE))
	{
		MessageBox(_T("Failed to open target for update"), MB_OK);
		return -1;
	}

	// Clamp the address range to what is allowed.  TBD: Consider allowing
	// the user to do something stupid, like overwriting parts of the firmware
	// that aren't in the updatable range?
	uint startAddress = (hexStartAddress < MIN_UPDATE_ADDRESS) ? MIN_UPDATE_ADDRESS : hexStartAddress;
	uint endAddress   = (hexEndAddress > MAX_UPDATE_ADDRESS) ? MAX_UPDATE_ADDRESS : hexEndAddress;

	byte outBuf[128];
	byte inBuf[128];

	// Blocks of 64 bytes
	unsigned int blockSize = UPDATE_BLOCKSIZE;

	// Ensure writes align on 64 byte boundaries
	unsigned int addr = (startAddress & ~(blockSize - 1));
	uint resLen = 0;
	while (addr < endAddress)
	{
		// Set the write address
		byte UAddr = (addr & 0xFF0000) >> 16;
		byte HAddr = (addr & 0x00FF00) >> 8;
		byte LAddr = (addr & 0x0000FF);
		outBuf[0] = LAddr;
		outBuf[1] = HAddr;
		outBuf[2] = UAddr;
		outBuf[3] = blockSize;

		// Copy a block from the hex file to the write buffer
		unsigned int wordAddr = addr / 2;
		byte *outPtr = &outBuf[4];
		unsigned int blockSize2 = blockSize / 2;
		for (unsigned int ctr=0;ctr<blockSize2;ctr++,addr+=2)
		{
			short updateWord = hexFile->programWords[wordAddr++];
			*outPtr++ = (updateWord & 0xFF);
			*outPtr++ = ((updateWord & 0xFF00) >> 8);
		}

		// Send the write command to the target
		int res = PICProgram::SendPacket(&m_Port, PICProgram::EVA_IWRITE_CMD, blockSize+4,
			outBuf, 2*blockSize, &resLen, inBuf, 16);
		if (res < 0)
		{
			CString addrStr;
			addrStr.Format(_T("%x"), addr);
			CString versStr = _T("Write failure during firmware update at: 0x") + addrStr;
			MessageBox((LPCTSTR)versStr);
			break;
		}
	}

	m_Port.Close();

	return 0;
}

// Clean out all code above 0x4000 in the programmer.  This should only be executed immediately
// prior to updating the firmware.  This routine is not necessary in most cases
// because the Update function performs an erase prior to writing.  It can be
// used to ensure 
LRESULT CMainFrame::OnEvaErase(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	if (!openCOMPort(BAUD_RATE))
	{
		return -1;
	}

	byte outBuf[128];
	byte inBuf[128];

	int blockSize = 0x40; // Blocks of 64 bytes
	unsigned int addr = MIN_UPDATE_ADDRESS;
	uint resLen = 0;
	for (;addr<MAX_UPDATE_ADDRESS;addr+=blockSize)
	{
		byte UAddr = (addr & 0xFF0000) >> 16;
		byte HAddr = (addr & 0x00FF00) >> 8;
		byte LAddr = (addr & 0x0000FF);
		outBuf[0] = LAddr;
		outBuf[1] = HAddr;
		outBuf[2] = UAddr;
		outBuf[3] = blockSize;
		int res = PICProgram::SendPacket(&m_Port, PICProgram::EVA_IERASE_CMD, 4,
			outBuf, 64, &resLen, inBuf, 500);
		if (res >= 0)
		{
			uint val = inBuf[0];
		}
	}

	m_Port.Close();

	return 0;
}

// Read code from the updatable region of firmware into a buffer and display it.
LRESULT CMainFrame::OnEvaRead(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	if (!openCOMPort(BAUD_RATE))
	{
		return -1;
	}

	// Create a blank device file to populate with the target's memory
	int targetPICIndex = DeviceList::DeviceIndex(_T("p18f2455"));
	const PICDevice *targetPICDevice = DeviceList::Devices[targetPICIndex];
	PICHexFile *newHexFile = new PICHexFile(targetPICDevice);

	byte outBuf[128];
	byte inBuf[128];

	int res = 0;
	unsigned int blockSize = UPDATE_BLOCKSIZE; // Blocks of 64 bytes
	uint startAddress = MIN_UPDATE_ADDRESS;
	uint endAddress   = MAX_UPDATE_ADDRESS;
	uint resLen = 0;
	for (unsigned int addr=startAddress;addr<endAddress;addr+=blockSize)
	{
		byte UAddr = (addr & 0xFF0000) >> 16;
		byte HAddr = (addr & 0x00FF00) >> 8;
		byte LAddr = (addr & 0x0000FF);
		outBuf[0] = LAddr;
		outBuf[1] = HAddr;
		outBuf[2] = UAddr;
		outBuf[3] = blockSize;
		res = PICProgram::SendPacket(&m_Port, PICProgram::EVA_IREAD_CMD, 4,
			outBuf, 128, &resLen, inBuf, 700);
		if (res < 0)
			break;

		// Move the response buffer into the hex file
		unsigned int wordAddress = addr / 2;
		unsigned int blockSize2 = blockSize / 2;
		byte *byteBuf = &inBuf[6];
		for (unsigned int index=0;index<blockSize2;index++,wordAddress++)
		{
			byte lowByte = *byteBuf++;
			byte highByte = *byteBuf++;
			unsigned short memWord = (highByte << 8) | lowByte;
			newHexFile->programWords[wordAddress] = memWord;
			newHexFile->programWordsUsed[wordAddress] = true;
		}
	}

	if (res >= 0)
	{
		int activePage = getActiveTabPage();
		if (activePage == 0)
		{
			delete m_HexFile1;
			m_HexFile1 = newHexFile;
			displayHexFile(m_view.m_hexPage1, m_HexFile1);
		}
		else if (activePage == 1)
		{
			delete m_HexFile2;
			m_HexFile2 = newHexFile;
			displayHexFile(m_view.m_hexPage2, m_HexFile2);
		}
	}
	else
	{
		delete newHexFile;
		MessageBox(_T("Failed to load program from target"));
	}

	m_Port.Close();

	return 0;
}

// Turn on/off debug print statements in the firmware.  Note: the DEBUG_PRINT flag
// has to have been turned on when building the firmware for this command
// to have any effect.
LRESULT CMainFrame::OnEvaDebug(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
#if !defined(_WIN32_WCE)
	CMenuHandle formMenu;
	formMenu = m_CmdBar.GetMenu();
	int menuFlags = formMenu.GetMenuState(ID_EVA_DEBUG, MF_BYCOMMAND);
	int isChecked = !(menuFlags & MF_CHECKED); // Toggle debug state
	formMenu.CheckMenuItem(ID_EVA_DEBUG, (isChecked ? MF_CHECKED : MF_UNCHECKED));

	if (!openCOMPort(BAUD_RATE))
	{
		return -1;
	}

	// Decimal values of version packet: <85><85><68><0><0><flag><4>
	uint result;
	int res = PICProgram::TxCmd(&m_Port, PICProgram::EVA_IDEBUG_PRINT_CMD, isChecked, &result);
	if (res < 0)
	{
		MessageBox(_T("Failed to toggle debug printing"));
	}

	m_Port.Close();
#endif

	return 0;
}

// Bring up a dialog that presents checkboxes for manually turning signal lines
// on/off.  This gives user the ability to test the functionality and voltage
// levels for: Vdd, Vpp, PGC, PGD, Select.
LRESULT CMainFrame::OnEvaSignaltest(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	byte outBuf[128];
	byte inBuf[128];
	memset(outBuf, 0, sizeof(outBuf));

	if (!openCOMPort(BAUD_RATE))
	{
		return -1;
	}

	uint resLen = 0;
	int res = PICProgram::SendPacket(&m_Port, PICProgram::EVA_READ_LINES_CMD, 4, outBuf,
		67, &resLen, inBuf, 4);
	if (true || res >= 0)
	{
		// res = PICProgram::TxCmd(&m_Port, PICProgram::EVA_BEGINPROG_CMD, 0, &resLen);

		CTesterDlg dlg(&m_Port);
		dlg.SetValues(&inBuf[2]);
		dlg.DoModal();

		// Ensure programming lines are reset (Vpp1, Vpp2, Vdd pulled to ground).
		PICProgram::TxCmd(&m_Port, PICProgram::EVA_ENDPROG_CMD, 0, &resLen);
	}
	else
	{
		MessageBox(_T("Failed to retrieve signal line values"));
	}

    m_Port.Close();

	return 0;
}

// Toggle between hex and assembly views of the code in the current buffer
LRESULT CMainFrame::OnViewAsm(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	// Switch between hex and disassembly view
	int activePage = getActiveTabPage();
	PICHexFile *hexFile = NULL;
	ChexPageDlg *tabPage = NULL;

	// Device and programming code stay the same.  Just change the hex file.
	if (activePage == 0)
	{
		hexFile = m_HexFile1;
		tabPage = m_view.m_hexPage1;
	}
	else if (activePage == 1)
	{
		hexFile = m_HexFile2;
		tabPage = m_view.m_hexPage2;
	}
	tabPage->m_hexPanel.mCodeDisplay = true;

	displayHexFile(tabPage, hexFile);

	return 0;
}

// Toggle between assembly and hex view of the code in the current buffer
LRESULT CMainFrame::OnViewHex(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	// Switch between hex and disassembly view
	int activePage = getActiveTabPage();
	PICHexFile *hexFile = NULL;
	ChexPageDlg *tabPage = NULL;

	// Device and programming code stay the same.  Just change the hex file.
	if (activePage == 0)
	{
		hexFile = m_HexFile1;
		tabPage = m_view.m_hexPage1;
	}
	else if (activePage == 1)
	{
		hexFile = m_HexFile2;
		tabPage = m_view.m_hexPage2;
	}
	tabPage->m_hexPanel.mCodeDisplay = false;

	displayHexFile(tabPage, hexFile);

	return 0;
}

LRESULT CMainFrame::OnSettingsOptions(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	COptionsDlg dlg(&m_Defaults);

	if (dlg.DoModal() == MB_OK)
	{
		DWORD br = dlg.mBaudRate;
		DWORD aview = dlg.mAsmView;
	}

	return 0;
}

