// MainFrm.h : interface of the CMainFrame class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_MAINFRM_H__2E66682E_48D7_4D08_B63D_551FDC88F521__INCLUDED_)
#define AFX_MAINFRM_H__2E66682E_48D7_4D08_B63D_551FDC88F521__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "EvaView.h"
#include "PIC/PICProgram.h"
#include "ProgressDlg.h"
#include "Registry/DefaultsManager.h"
#include "Serial/IOPort.h"

#if !defined(_WIN32_WCE)
extern "C"
{
#include <hidsdi.h>
#include <setupapi.h>
}
#endif

class CMainFrame : public CFrameWindowImpl<CMainFrame>, public CUpdateUI<CMainFrame>,
		public CMessageFilter, public CIdleHandler
{
public:
	DECLARE_FRAME_WND_CLASS(NULL, IDR_MAINFRAME)

	CEvaView m_view;
#if !defined(_WIN32_WCE)
	CCommandBarCtrl m_CmdBar;
#endif

	LPTSTR m_PortName;
	CIOPort m_Port;
	DWORD m_BaudRate;
	DWORD m_AsmView;

	CMenu m_DeviceListMenu, P12Menu, P16Menu, P18Menu, P30Menu;
	CMenu m_ComPortMenu;
	int m_ComPortMenuCount;

	// This member is set when we are trying to close down the application.  Should be
	// used by any processing threads to see if they need to end early.
	bool m_Terminating;
	// Flag that determines if we poll programmer for its status
	bool m_PollStatusFlag;
	// Handle to background thread that polls the programmer for its status.
	HANDLE m_StatusThreadHandle;
	// Flag to signal that the COM port/Programmer is currently in use
	bool m_ProgrammerActive;
	// MUTEX used to allow only one thread at a time access to the COM port/programmer
	HANDLE m_ProgrammerMutex;

	int getActiveTabPage(void);
	PICHexFile *getActiveHexFile(void);

	void replaceDevice(const PICDevice *device);
	void displayHexFile(ChexPageDlg *page, PICHexFile *hexFile);
	void updateCodeFromFile(ChexPageDlg *hexPage, PICHexFile *hexFile);
	void updateDataFromFile(ChexPageDlg *hexPage, PICHexFile *hexFile);
	void updateConfigFromFile(ChexPageDlg *hexPage, PICHexFile *hexFile);
	void updateFileFromConfig(ChexPageDlg *hexPage, PICHexFile *hexFile);
	bool openCOMPort(int baudRate = CBR_38400);
	void readProgrammerVoltage(void);
	void buildPortMenu(void);
	void buildDeviceMenu(int deviceIndex);
	void buildStatusBar(void);
	void setStatusPane(int pane, LPCTSTR paneText);
	void AddToolButtons();
	// ChipCommand() is the single entry point for sending programming commands
	// to the chip.  It packages up the request and starts a worker thread to
	// perform the programming (read, write, or erase).  This makes cancelling
	// easier because the main thread can take a user input and abort the operation.
	int ChipCommand(int writeFlag, bool bootFlag, bool codeFlag, bool dataFlag, bool configFlag);

	const PICProgram *m_Driver;
	const PICDevice *m_Device;
	PICHexFile *m_HexFile1, *m_HexFile2;
	LPTSTR m_procMenuItem;
	int m_MenuSel1, m_MenuSel2;

	CProgressDlg m_ProgressDlg;

	CDefaultsManager m_Defaults;

	virtual BOOL PreTranslateMessage(MSG* pMsg);
	virtual BOOL OnIdle();

	BEGIN_UPDATE_UI_MAP(CMainFrame)
		UPDATE_ELEMENT(ID_VIEW_TOOLBAR, UPDUI_MENUPOPUP)
		UPDATE_ELEMENT(ID_VIEW_STATUS_BAR, UPDUI_MENUPOPUP)
	END_UPDATE_UI_MAP()


	BEGIN_MSG_MAP(CMainFrame)
		MESSAGE_HANDLER(WM_CREATE, OnCreate)
		MESSAGE_HANDLER(WM_CLOSE, OnClose)
		MESSAGE_HANDLER(WM_SIZE, OnSize)
#if !defined(_WIN32_WCE)
		MESSAGE_HANDLER(WM_DEVICECHANGE, OnDeviceChange)
		MESSAGE_HANDLER(WM_SIZING, OnSizing)
		MESSAGE_HANDLER(WM_GETMINMAXINFO, OnGetMinMaxInfo)
#endif
		COMMAND_ID_HANDLER(ID_APP_EXIT, OnFileExit)
		COMMAND_ID_HANDLER(ID_FILE_NEW, OnFileNew)
		COMMAND_ID_HANDLER(ID_VIEW_TOOLBAR, OnViewToolBar)
		COMMAND_ID_HANDLER(ID_VIEW_STATUS_BAR, OnViewStatusBar)
		COMMAND_ID_HANDLER(ID_APP_ABOUT, OnAppAbout)
		COMMAND_ID_HANDLER(ID_FILE_OPEN, OnFileOpen)
		COMMAND_ID_HANDLER(ID_CHIP_ERASE, OnChipErase)
		COMMAND_ID_HANDLER(ID_CHIP_PROGRAM, OnChipProgram)
		COMMAND_ID_HANDLER(ID_CHIP_READ, OnChipRead)
		COMMAND_ID_HANDLER(ID_PROCESSOR_COMBO, OnProcessorCombo)
		COMMAND_ID_HANDLER(ID_BUTTON_COM_PORT, OnComPortButton)
		COMMAND_ID_HANDLER(ID_VIEW_VPPSTATUS, OnViewVppstatus)
		COMMAND_ID_HANDLER(ID_READ_CODE, OnReadCode)
		COMMAND_ID_HANDLER(ID_READ_CONFIG, OnReadConfig)
		COMMAND_ID_HANDLER(ID_READ_DATA, OnReadData)
		COMMAND_ID_HANDLER(ID_WRITE_CODE, OnWriteCode)
		COMMAND_ID_HANDLER(ID_WRITE_CONFIG, OnWriteConfig)
		COMMAND_ID_HANDLER(ID_WRITE_DATA, OnWriteData)
		COMMAND_ID_HANDLER(ID_ERASE_BOOT, OnEraseBoot)
		COMMAND_ID_HANDLER(ID_ERASE_DATA, OnEraseData)
		COMMAND_ID_HANDLER(ID_ERASE_CODE, OnEraseCode)
		COMMAND_ID_HANDLER(ID_ERASE_NONBOOT, OnEraseNonBoot)
		COMMAND_ID_HANDLER(ID_EVA_ECHO, OnEvaEcho)
		COMMAND_ID_HANDLER(ID_EVA_VERSION, OnEvaVersion)
		COMMAND_ID_HANDLER(ID_FILE_SAVE, OnFileSave)
		COMMAND_ID_HANDLER(ID_FILE_SAVE_AS, OnFileSaveAs)
		COMMAND_ID_HANDLER(ID_VIEW_ASM, OnViewAsm)
		COMMAND_ID_HANDLER(ID_VIEW_HEX, OnViewHex)
		COMMAND_ID_HANDLER(ID_SETTINGS_OPTIONS, OnSettingsOptions)
		COMMAND_ID_HANDLER(ID_EVA_UPDATE, OnEvaUpdate)
		COMMAND_ID_HANDLER(ID_EVA_ERASE, OnEvaErase)
		COMMAND_ID_HANDLER(ID_EVA_READ, OnEvaRead)
		COMMAND_ID_HANDLER(ID_EVA_REGISTERS, OnEvaRegisters)
		COMMAND_ID_HANDLER(ID_EVA_DEBUG, OnEvaDebug)
		COMMAND_ID_HANDLER(ID_EVA_SIGNALTEST, OnEvaSignaltest)
		NOTIFY_CODE_HANDLER(TBN_DROPDOWN, OnProcessorDropDown)
		CHAIN_MSG_MAP(CUpdateUI<CMainFrame>)
		CHAIN_MSG_MAP(CFrameWindowImpl<CMainFrame>)
	END_MSG_MAP()

// Handler prototypes (uncomment arguments if needed):
//	LRESULT MessageHandler(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
//	LRESULT CommandHandler(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
//	LRESULT NotifyHandler(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)

	LRESULT OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnDeviceChange(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnSizing(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnClose(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnSize(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnGetMinMaxInfo(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);

	LRESULT OnFileExit(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnFileNew(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnViewToolBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnViewStatusBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnAppAbout(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnFileOpen(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnChipErase(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnChipProgram(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnChipRead(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnProcessorDropDown(UINT /*uMsg*/, LPNMHDR /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnProcessorCombo(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnComPortButton(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnViewVppstatus(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnReadCode(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnReadConfig(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnReadData(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnWriteCode(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnWriteConfig(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnWriteData(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnEraseBoot(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnEraseNonBoot(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnEraseData(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnEraseCode(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnEvaEcho(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnEvaVersion(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnFileSave(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnFileSaveAs(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnViewAsm(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnViewHex(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnSettingsOptions(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnEvaUpdate(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnEvaErase(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnEvaRead(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnEvaRegisters(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnEvaDebug(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnEvaSignaltest(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);

	static HRESULT ActivatePreviousInstance(HINSTANCE hInstance);
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft eMbedded Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MAINFRM_H__2E66682E_48D7_4D08_B63D_551FDC88F521__INCLUDED_)
