#include "StdAfx.h"
#include "TesterDlg.h"
#include "PIC/PICProgram.h"

CTesterDlg::CTesterDlg(CIOPort *ioPort)
{
	mPort = ioPort;
}

CTesterDlg::~CTesterDlg(void)
{
}

void CTesterDlg::SetValues(unsigned char *values)
{
	memcpy(mFlags, values, sizeof(mFlags));
}

#if defined(_WIN32_WCE)
int initWinCEDialog(HWND hwnd)
{
	// Extra code to ensure the dialog is shown full screen on the PocketPC
    BOOL rb;
    int rc;
    SHINITDLGINFO sid;

    sid.dwMask  = SHIDIM_FLAGS;  // This is the only allowed value.
    sid.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_FULLSCREENNOMENUBAR;  // Make the DB full screen.
    sid.hDlg = hwnd;  // Handle to the dialog box.

    rb = SHInitDialog(&sid);
    if (rb == FALSE)  // SHInitDialog failed.
    {
        rc = MessageBox(NULL, _T("Could not create dialog box."),
                        _T("Error"), MB_OK);
        if (rc == 0)  // Not enough memory to create MessageBox.
            return E_OUTOFMEMORY;
        return E_FAIL;  // Replace with specific error handling.
    }
}
#endif

LRESULT CTesterDlg::OnInitDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
#if 0 && defined(_WIN32_WCE)
	if (!SUCCEEDED(initWinCEDialog((HWND)wParam)))
		return FALSE;
#endif
	// Assume signal lines already read and mFlags are correctly set.

	// Set values of all check boxes based on values in mFlags
	CheckDlgButton(IDC_CHECK_VDD,   ((mFlags[0] & 0x01) ? BST_CHECKED : BST_UNCHECKED));
	CheckDlgButton(IDC_CHECK_VPP1,  ((mFlags[0] & 0x02) ? BST_CHECKED : BST_UNCHECKED));
	CheckDlgButton(IDC_CHECK_VPP2,  ((mFlags[0] & 0x04) ? BST_CHECKED : BST_UNCHECKED));
	CheckDlgButton(IDC_CHECK_PGC,   ((mFlags[0] & 0x08) ? BST_CHECKED : BST_UNCHECKED));
	CheckDlgButton(IDC_CHECK_PGD,   ((mFlags[0] & 0x10) ? BST_CHECKED : BST_UNCHECKED));
	CheckDlgButton(IDC_CHECK_PRGSEL,((mFlags[0] & 0x20) ? BST_CHECKED : BST_UNCHECKED));
	CheckDlgButton(IDC_CHECK_LED1,  ((mFlags[1] & 0x01) ? BST_CHECKED : BST_UNCHECKED));
	CheckDlgButton(IDC_CHECK_LED2,  ((mFlags[1] & 0x02) ? BST_CHECKED : BST_UNCHECKED));
	CheckDlgButton(IDC_CHECK_LED3,  ((mFlags[1] & 0x04) ? BST_CHECKED : BST_UNCHECKED));

	// Attach dialog items to convenience classes
	mSignalBtns[0].Attach(GetDlgItem(IDC_CHECK_VDD));
	mSignalBtns[1].Attach(GetDlgItem(IDC_CHECK_VPP1));
	mSignalBtns[2].Attach(GetDlgItem(IDC_CHECK_VPP2));
	mSignalBtns[3].Attach(GetDlgItem(IDC_CHECK_PGC));
	mSignalBtns[4].Attach(GetDlgItem(IDC_CHECK_PGD));
	mSignalBtns[5].Attach(GetDlgItem(IDC_CHECK_PRGSEL));

	mLEDBtns[0].Attach(GetDlgItem(IDC_CHECK_LED1));
	mLEDBtns[1].Attach(GetDlgItem(IDC_CHECK_LED2));
	mLEDBtns[2].Attach(GetDlgItem(IDC_CHECK_LED3));

	return TRUE;
}

void CTesterDlg::setSignalLines()
{
	unsigned int resVal;
    int flags = (mFlags[0] << 16) | mFlags[1];

	if (mPort == NULL)
		return;

	int res = PICProgram::TxCmd(mPort, PICProgram::EVA_SET_LINES_CMD, flags, &resVal);
}

LRESULT CTesterDlg::OnClose(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	// TODO: Turn signal lines off before closing the window (set to default non-programming
	// state).  This will prevent stray voltages being present if a chip is connected.

	// Close the dialog
	EndDialog(IDOK);

	return 0;
}

LRESULT CTesterDlg::OnBnClickedCheckVdd(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	if (mSignalBtns[0].GetCheck() == BST_CHECKED)
		mFlags[0] |= 0x01;
	else
		mFlags[0] &= 0xfe;
	setSignalLines();

	return 0;
}

LRESULT CTesterDlg::OnBnClickedCheckVpp1(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	if (mSignalBtns[1].GetCheck() == BST_CHECKED)
		mFlags[0] |= 0x02;
	else
		mFlags[0] &= 0xfd;
	setSignalLines();

	return 0;
}

LRESULT CTesterDlg::OnBnClickedCheckVpp2(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	if (mSignalBtns[2].GetCheck() == BST_CHECKED)
		mFlags[0] |= 0x04;
	else
		mFlags[0] &= 0xfb;
	setSignalLines();

	return 0;
}

LRESULT CTesterDlg::OnBnClickedCheckPgc(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	if (mSignalBtns[3].GetCheck() == BST_CHECKED)
		mFlags[0] |= 0x08;
	else
		mFlags[0] &= 0xf7;
	setSignalLines();

	return 0;
}

LRESULT CTesterDlg::OnBnClickedCheckPgd(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	if (mSignalBtns[4].GetCheck() == BST_CHECKED)
		mFlags[0] |= 0x10;
	else
		mFlags[0] &= 0xef;
	setSignalLines();

	return 0;
}


LRESULT CTesterDlg::OnBnClickedCheckPrgsel(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	if (mSignalBtns[5].GetCheck() == BST_CHECKED)
		mFlags[0] |= 0x20;
	else
		mFlags[0] &= 0xdf;
	setSignalLines();

	return 0;
}

LRESULT CTesterDlg::OnBnClickedCheckLed1(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	if (mLEDBtns[0].GetCheck() == BST_CHECKED)
		mFlags[1] |= 0x01;
	else
		mFlags[1] &= 0xfe;
	setSignalLines();

	return 0;
}

LRESULT CTesterDlg::OnBnClickedCheckLed2(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	if (mLEDBtns[1].GetCheck() == BST_CHECKED)
		mFlags[1] |= 0x02;
	else
		mFlags[1] &= 0xfd;
	setSignalLines();

	return 0;
}

LRESULT CTesterDlg::OnBnClickedCheckLed3(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	if (mLEDBtns[2].GetCheck() == BST_CHECKED)
		mFlags[1] |= 0x04;
	else
		mFlags[1] &= 0xfb;
	setSignalLines();

	return 0;
}

