// hexPageDlg.cpp : Implementation of CtabPageDlg

#include "stdafx.h"
#include "resource.h"

#include "hexPageDlg.h"

// CtabPageDlg

LRESULT ChexPageDlg::OnInitDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	m_showAsm = false; // Default to hex display.  true = show disassembly
	m_inHighRes = false; // Assume standard sizing

	m_codeBox.Attach(GetDlgItem(IDC_CODE_GROUPBOX));
	m_dataBox.Attach(GetDlgItem(IDC_DATA_GROUPBOX));
	m_configBox.Attach(GetDlgItem(IDC_CONFIG_GROUPBOX));

	// Attach controls to member variables
	m_hexPanel.SubclassWindow(GetDlgItem(IDC_EDIT_CODE_VIEW));
	m_hexPanel.SetReadOnly(TRUE);
	m_dataPanel.Attach(GetDlgItem(IDC_EDIT_DATA_VIEW));
	m_dataPanel.SetReadOnly(TRUE);

	m_hexPanel.SetParent(m_codeBox);
	m_dataPanel.SetParent(m_dataBox);

	// Subclass edit controls so they can manage windows messages themselves
	m_configWords[0].SubclassWindow(GetDlgItem(IDC_EDIT_CONFIG1));
	m_configWords[1].SubclassWindow(GetDlgItem(IDC_EDIT_CONFIG2));
	m_configWords[2].SubclassWindow(GetDlgItem(IDC_EDIT_CONFIG3));
	m_configWords[3].SubclassWindow(GetDlgItem(IDC_EDIT_CONFIG4));
	m_configWords[4].SubclassWindow(GetDlgItem(IDC_EDIT_CONFIG5));
	m_configWords[5].SubclassWindow(GetDlgItem(IDC_EDIT_CONFIG6));
	m_configWords[6].SubclassWindow(GetDlgItem(IDC_EDIT_CONFIG7));
	m_configWords[7].SubclassWindow(GetDlgItem(IDC_EDIT_CONFIG8));
	m_userID[0].SubclassWindow(GetDlgItem(IDC_EDIT_USER_ID1));
	m_userID[1].SubclassWindow(GetDlgItem(IDC_EDIT_USER_ID2));
	m_userID[2].SubclassWindow(GetDlgItem(IDC_EDIT_USER_ID3));
	m_userID[3].SubclassWindow(GetDlgItem(IDC_EDIT_USER_ID4));
	m_deviceID[0].SubclassWindow(GetDlgItem(IDC_EDIT_DEVICE_ID1));
	m_deviceID[0].SetReadOnly(TRUE);
	m_deviceID[1].SubclassWindow(GetDlgItem(IDC_EDIT_DEVICE_ID2));
	m_deviceID[1].SetReadOnly(TRUE);

	m_dasmPanel.Attach(GetDlgItem(IDC_EDIT_DASM_VIEW));
	m_dasmPanel.ShowWindow(SW_HIDE);

	int ci, ui;
	for (ci=0;ci<8;ci++)
		m_configWords[ci].SetLimitText(4);
	for (ui=0;ui<4;ui++)
		m_userID[ui].SetLimitText(4);

#if defined(_WIN32_WCE)
	m_deviceID[0].SetParent(m_configBox);
	m_deviceID[1].SetParent(m_configBox);
	for (ci=0;ci<8;ci++)
		m_configWords[ci].SetParent(m_configBox);
	for (ui=0;ui<4;ui++)
		m_userID[ui].SetParent(m_configBox);
#endif

	m_codeEditBrush.CreateSolidBrush(RGB(255, 255, 255));

	return TRUE;
}

LRESULT ChexPageDlg::OnCtlColorEdit(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	HDC dc = (HDC)wParam;
	HWND ctlHWnd = (HWND)lParam;

	if (ctlHWnd == m_hexPanel)
	{
		bHandled = TRUE;
		return (LRESULT)m_codeEditBrush.m_hBrush;
	}
	else
		return 0;
}

LRESULT ChexPageDlg::OnCtlColorStatic(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	HDC dc = (HDC)wParam;
	HWND ctlHWnd = (HWND)lParam;

	if (ctlHWnd == m_hexPanel || ctlHWnd == m_dataPanel)
	{
		bHandled = TRUE;
		return (LRESULT)m_codeEditBrush.m_hBrush;
	}
	else
		return 0;
}

void ChexPageDlg::SetCtrlFont(CFont &font)
{
	m_hexPanel.SetFont(font);
	m_dataPanel.SetFont(font);
	m_configWords[0].SetFont(font);
	m_configWords[1].SetFont(font);
	m_configWords[2].SetFont(font);
	m_configWords[3].SetFont(font);
	m_configWords[4].SetFont(font);
	m_configWords[5].SetFont(font);
	m_configWords[6].SetFont(font);
	m_configWords[7].SetFont(font);
	m_userID[0].SetFont(font);
	m_userID[1].SetFont(font);
	m_userID[2].SetFont(font);
	m_userID[3].SetFont(font);
	m_deviceID[0].SetFont(font);
	m_deviceID[1].SetFont(font);
}

int ChexPageDlg::sizeConfigBoxes(int width, int height, int charHeight, int innerPadding)
{
	int configWidth;
	RECT wRect, configRect;

#if defined(_WIN32_WCE)
	bool landscapeFlag = (m_inHighRes ? (width > 480) : (width > 240));
	int boxWidth  = (m_inHighRes ? 64 : 32);
	int boxHeight = (m_inHighRes ? 26 : 13);

	if (landscapeFlag)
		// Two columns of boxes for fuses and user IDs
		configWidth = (m_inHighRes ? 192 : 96);
	else
		// Single column
		configWidth = (m_inHighRes ? 96 : 48);

	wRect.left = width - configWidth; wRect.right  = width-1;
	wRect.top  = 0;                   wRect.bottom = height-1;
	m_configBox.MoveWindow(&wRect);
	m_configBox.GetClientRect(&configRect);

	int ci;
	int lineSpacing = (m_inHighRes ? 6 : 3);
	int horizSpacing = (m_inHighRes ? 4 : 2);
	int y = charHeight;
	// Figure out the offset needed to center the boxes in the group box
	int availConfigWidth = (configRect.right - configRect.left) - 2 * innerPadding;
	int centerX = innerPadding + (availConfigWidth - boxWidth) / 2;
	if (landscapeFlag)
	{
		int leftX = innerPadding + ((availConfigWidth - innerPadding) - 2 * boxWidth) / 2;
		int middleX = leftX + boxWidth + innerPadding;
		for (ci=0;ci<8;ci+=2)
		{
			m_configWords[ci].MoveWindow(leftX, y, boxWidth, boxHeight);
			m_configWords[ci+1].MoveWindow(middleX, y, boxWidth, boxHeight);
			y += boxHeight + lineSpacing;
		}

		// offset to user ID words
		y += (m_inHighRes ? 16 : 8);
		for (ci=0;ci<4;ci+=2)
		{
			m_userID[ci].MoveWindow(leftX, y, boxWidth, boxHeight);
			m_userID[ci+1].MoveWindow(middleX, y, boxWidth, boxHeight);
			y += boxHeight + lineSpacing;
		}

		// offset to device ID (two words, centered in config group box
		y += (m_inHighRes ? 16 : 8);
		m_deviceID[0].MoveWindow(leftX, y, boxWidth, boxHeight);
		m_deviceID[1].MoveWindow(middleX, y, boxWidth, boxHeight);
	}
	else
	{
		for (ci=0;ci<8;ci++)
		{
			m_configWords[ci].MoveWindow(centerX, y, boxWidth, boxHeight);
			y += boxHeight + lineSpacing;
		}

		// offset to user ID words
		y += (m_inHighRes ? 16 : 8);
		for (ci=0;ci<4;ci++)
		{
			m_userID[ci].MoveWindow(centerX, y, boxWidth, boxHeight);
			y += boxHeight + lineSpacing;
		}

		// offset to device ID (two words, centered in config group box
		y += (m_inHighRes ? 16 : 8);
		m_deviceID[0].MoveWindow(centerX, y, boxWidth, boxHeight);
		m_deviceID[1].MoveWindow(centerX, y, boxWidth, boxHeight);
	}
#else
	// Normal windows
	configWidth = (m_inHighRes ? 96 : 48);
	wRect.left = width - configWidth; wRect.right  = width-1;
	wRect.top  = 0;                   wRect.bottom = height-1;
	// m_configBox.MoveWindow(&wRect);
	m_configBox.GetClientRect(&configRect);
	configWidth = (configRect.right - configRect.left);
#endif

	return configWidth;
}

LRESULT ChexPageDlg::OnSize(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
{
	HRESULT hRes;
	LOGFONT lf;
	RECT wRect, codeRect, dataRect;
	int width = LOWORD(lParam);
	int height = HIWORD(lParam);

	// Get the size of the font in the group box
	CFont grpFont = m_codeBox.GetFont();
	if (grpFont.m_hFont != NULL)
		hRes = grpFont.GetLogFont(&lf);
	else
		hRes = 0;
	int charHeight = (hRes ? 4 + -lf.lfHeight : (m_inHighRes ? 32 : 16));
	int innerPadding = (m_inHighRes ? 6 : 3); // Inset within group box

	// Size and position config (fuse) words
	int configWidth = sizeConfigBoxes(width, height, charHeight, innerPadding);

	// Size and position code words (about 2/3's of the available height)
	int codeHeight = 2 * height / 3;
	int codeWidth  = width - configWidth;
	wRect.left = 0; wRect.right = codeWidth;
	wRect.top  = 0; wRect.bottom = codeHeight - 1;
	m_codeBox.MoveWindow(&wRect);

	// Size and position data word group box right under code box
	int dataHeight = height - codeHeight;
	int dataWidth  = width - configWidth;
	wRect.left = 0;               wRect.right = dataWidth;
	wRect.top = wRect.bottom + 4; wRect.bottom = height-1;
	m_dataBox.MoveWindow(&wRect);

	// Size and place the edit box containing the code words
	m_codeBox.GetClientRect(&codeRect);
	wRect = codeRect;
	wRect.left += innerPadding; wRect.right -= innerPadding;
	wRect.top += charHeight; wRect.bottom -= innerPadding;
	m_hexPanel.MoveWindow(&wRect);

	// Size and place the edit box containing the data words
	m_dataBox.GetClientRect(&dataRect);
	wRect = dataRect;
	wRect.left += innerPadding; wRect.right -= innerPadding;
	wRect.top += charHeight;    wRect.bottom -= innerPadding;
	m_dataPanel.MoveWindow(&wRect);

	// Size and place the edit box that contains disassembled hex
	wRect.left = codeRect.left;
	wRect.top = codeRect.top;
	wRect.bottom = dataRect.bottom;
	wRect.right = width-1;
	m_dasmPanel.MoveWindow(&wRect);

	return 0;
}
