// Firmware framework for USB I/O on PIC 18F2455 (and siblings)
// Copyright (C) 2005 Alexander Enzmann
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
#ifndef EVA_DEFS_H
#define EVA_DEFS_H

#include <pic18fregs.h>

#if !defined(byte)
#define byte unsigned char
#endif

// Allowed values for CLK_SPD: 48, 24, 20, 16, 10, 6.67, 5 (MHz)
// HS_PLL is needed for: 48, 24, 16.  HS is needed for: 20, 10, 6.67, 5.
// TBD: 48 MHz has at times been unreliable in this application - use only for testing.
//      24, 20 MHz seem pretty solid.
//      16 MHz works, but has had intermittent errors on high speed hub
// TBD:  10, 5 MHz don't get through enumeration on some hubs
// TBD: 5 MHz doesn't seem able to keep up when using RS232 - there are timeouts during I/O
// Note: If the clock speed is changed here, it will also need to be
// changed in EvaProg.asm - otherwise when it is loaded, it will make
// bad timing assumptions.
#define CLK_SPD 24

//
// Pin definitions
//
#define VBOOST_MASK TRISAbits.TRISA0
#define VBOOST PORTAbits.AN0
#define EXT_PWR_MASK TRISAbits.TRISA1
#define EXT_PWR PORTAbits.RA1
#define CTS_MASK TRISAbits.TRISA2
#define CTS PORTAbits.RA2
#define RTS_MASK TRISAbits.TRISA3
#define RTS PORTAbits.RA3
#define LED2_MASK TRISAbits.TRISA4
#define LED2 PORTAbits.RA4
#define LED1_MASK TRISAbits.TRISA5
#define LED1 PORTAbits.RA5
// RA6 - Tied to oscillator
// RA7 - N/A

#define SDA_MASK TRISBbits.TRISB0
#define SDA PORTBbits.RB0
#define SCL_MASK TRISBbits.TRISB1
#define SCL PORTBbits.RB1
#define VPP1_MASK TRISBbits.TRISB2
#define VPP1 PORTBbits.RB2
#define ICSP_CLK_MASK TRISBbits.TRISB3
#define ICSP_CLK PORTBbits.RB3
#define ICSP_DAT_MASK TRISBbits.TRISB4
#define ICSP_DAT PORTBbits.RB4
#define VDD_MASK TRISBbits.TRISB5
#define VDD PORTBbits.RB5
#define VPP2_MASK TRISBbits.TRISB6
#define VPP2 PORTBbits.RB6
#define LED3_MASK TRISBbits.TRISB7
#define LED3 PORTBbits.RB7

#define PRG_SEL_MASK TRISCbits.TRISC0
#define PRG_SEL PORTCbits.RC0
#define USB_PWR_MASK TRISCbits.TRISC1
#define USB_PWR PORTCbits.RC1
#define VPP_PWM_MASK TRISCbits.TRISC2
#define VPP_PWM PORTCbits.RC2
// RC3 - N/A
// RC4 - USB D-
// RC5 - USB D+
#define SERIAL_TX_MASK TRISCbits.TRISC6
#define SERIAL_TX PORTBbits.RC6
#define SERIAL_RX_MASK TRISCbits.TRISC7
#define SERIAL_RX PORTBbits.RC7


//
// Commands:
//
#define VERSION_CMD     0x01 // 
#define RESET_CMD       0x02 // Reset chip and start over
#define VPP_CMD         0x03 // Turn Vpp on/off
// (unused - ETX)
#define PRGSEL_CMD      0x05 // Turn PRG_SEL on/off
#define VDD_CMD         0x06 // Turn Vdd on/off
#define CLKDAT_CMD      0x07 // Set the ICSP CLK and DAT lines
#define READ_LINES_CMD  0x08 // Read current value of signal lines
#define READ_REGS_CMD   0x09 // Read the value of SFRs
#define ECHO_CMD        0x0a // Send back the input
#define BEGINPROG_CMD   0x0b // Enter prog mode: lower DAT/CLK, raise Vdd and Vpp
#define ENDPROG_CMD     0x0c // Exit prog mode: lower Vdd/Vpp, tristate CLK/DAT
#define PIC14_CMD       0x0d // Send command to 14 bit PIC
#define PIC14R_CMD      0x0e // Send cmd to 14 bit PIC, read response
// (unused - null command)
#define READ_ANALOG_CMD 0x10 // Read the current value of the A/D converter
#define SET_ANALOG_CMD  0x11 // Set desired voltage bounds
#define PIC16ADDR_CMD   0x12 // Set address of a 16 bit PIC
#define PIC16_CMD       0x13 // Send command to 16 bit PIC
#define PIC16R_CMD      0x14 // Send cmd to 16 bit PIC, read response
#define PIC16W_CMD      0x15 // Send write cmd to 16 bit PIC, hold CLK for time P9 (1-5 msec)
#define READ_MASKS_CMD  0x16 // Read current value of TRIS registers
#define READ_PORTS_CMD  0x17 // Read current value of PORT registers
#define READ_PWM_CMD    0x18 // Read current PWM duty cycle
#define PIC14RBP_CMD    0x19 // Read a block from a 14 bit PIC
#define PIC14WBP_CMD    0x1a // Write a block to a 14 bit PIC
#define PIC14RBE_CMD    0x1b // Read a block from a 14 bit PIC EEPROM
#define PIC14WBE_CMD    0x1c // Write a block to a 14 bit PIC EEPROM
#define PIC16RB_CMD     0x1d // Read a block from a 16 bit PIC
#define PIC16WB_CMD     0x1e // Write a block to a 16 bit PIC
#define PIC16RBE_CMD    0x1f // Read a block from a 16 bit PIC EEPROM
#define PIC16WBE_CMD    0x20 // Write a block to a 16 bit PIC EEPROM
#define PIC16WBUF_CMD   0x21 // Write one buffer of bytes to a 16 bit PIC
#define PIC16EBUF_CMD   0x22 // Erase one window of bytes on a 16 bit PIC
#define PIC16WEBYTE_CMD 0x23 // Write one byte to a 16 bit PIC EEPROM
#define PIC24_CMD       0x24 // Send cmd to 24 bit PIC (dsPIC)
#define PIC24R_CMD      0x25 // Send cmd to 24 bit PIC (dsPIC), read response
#define PIC24RBUF_CMD   0x26 // Read 16 instructions from a 24 bit PIC
#define PIC24WBUF_CMD   0x27 // Write 16 instruction to 24 bit PIC
#define PIC24WBSETUP    0x28 // Setup for write to 24 bit PIC
#define PIC24WBFINISH   0x29 // Finish buffer write to 24 bit PIC

#define LAST_CMD        0x29 // [Adjust if commands added/deleted]

// Commands associated with internal memory (rather than programming
// commands).  These are only for use when updating firmware via a
// USB connection.
#define IREAD_CMD        0x41 // Read programmer memory
#define IWRITE_CMD       0x42 // Write programmer memory
#define IERASE_CMD       0x43 // Erase programmer memory
#define IRUN_CMD         0x44 // Run code in internal programmer memory
#define IDEBUG_PRINT_CMD 0x45 // Turn serial debug printing on/off

#endif // EVA_DEFS_H
