#include "StdAfx.h"
#include "DeviceList.h"
#include "PIC/PICProgram.h"

// Device descriptions have the following form:
//   names[4]        - array of strings that can be used to describe the chip
//   wordSize        - number of bits in instructions (currently 14 or 16)
//   sizeAddr        - number of words in code memory
//   dataAddr        - base address of EEPROM data memory
//   sizeData        - number of bytes in EEPROM data memory
//   configAddr      - base address of configuration memory space (config, user ID, device ID)
//   configSize      - size of entire configuration memory
//   configWordsAddr - base address of configuration (fuse) words
//   configWordsSize - number of valid configuration (fuse) words
//   userIDAddr      - base address of user ID words
//   userIDSize      - number of valid user ID words
//   devIDAddr       - base address of device ID words
//   sizeDevID       - number of valid device ID words
//   devIDMask       - valid bits in device ID

// List of known devices
const PICDevice *DeviceList::Devices[] =
{
#if 0
	// TBD: Support for 12 bit devices
	new PICDevice(
	nameArr(_T("p12f508"), _T("12f508"), _T("pic12f508"), _T("f508")), 12,
	0x0200, 0x0000, 0x0000, 0x03fff, 1, 0x03ff, 1, 0x0200, 4, 0x0000, 0),
	new PICDevice(
	nameArr(_T("p12f509"), _T("12f509"), _T("pic12f509"), _T("f509")), 12,
	0x0400, 0x0000, 0x0000, 0x07ff, 1, 0x07ff, 1, 0x0400, 4, 0x0000, 0),
#endif
	new PICDevice(
	nameArr(_T("p12f629"), _T("12f629"), _T("pic12f629"), _T("f629")), 14,
	0x0400, 0x080, 8, 1, 2, 1, 0x0f9f),
	new PICDevice(
	nameArr(_T("p12f635"), _T("12f635"), _T("pic12f635"), _T("f635")), 14,
		0x0400, 0x080, 8, 1, 4, 1, 0x0fbf),
	new PICDevice(
	nameArr(_T("p12f675"), _T("12f675"), _T("pic12f675"), _T("f675")), 14,
	0x0400, 0x080, 8, 1, 2, 1, 0x0fdf),
	new PICDevice(
	nameArr(_T("p12f683"), _T("12f683"), _T("pic12f683"), _T("f683")), 14,
		0x0800, 0x100, 8, 1, 4, 1, 0x047f),

	new PICDevice(
	nameArr(_T("p16f630"), _T("16f630"), _T("pic16f630"), _T("f630")), 14,
	0x0400, 0x080, 8, 1, 2, 1, 0x10df),
	new PICDevice(
	nameArr(_T("p16f636"), _T("16f636"), _T("pic16f636"), _T("f636")), 14,
		0x0800, 0x080, 8, 1, 4, 1, 0x10bf),
	new PICDevice(
	nameArr(_T("p16f676"), _T("16f676"), _T("pic16f676"), _T("f676")), 14,
	0x0400, 0x080, 8, 1, 2, 1, 0x10ff),
	new PICDevice(
	nameArr(_T("p16f684"), _T("16f684"), _T("pic16f684"), _T("f684")), 14,
		0x0800, 0x100, 8, 1, 4, 1, 0x109f),
	new PICDevice(
	nameArr(_T("p16f687"), _T("16f687"), _T("pic16f687"), _T("f687")), 14,
	0x0800, 0x080, 8, 1, 4, 1, 0x133f),
	new PICDevice(
	nameArr(_T("p16f688"), _T("16f688"), _T("pic16f688"), _T("f688")), 14,
	0x1000, 0x080, 8, 1, 4, 1, 0x119f),
	new PICDevice(
	nameArr(_T("p16f73"), _T("16f73"), _T("pic16f73"), _T("f73")), 14,
	0x1000, 0x00, 8, 1, 4, 1, 0x061f),
	new PICDevice(
	nameArr(_T("p16f737"), _T("16f737"), _T("pic16f737"), _T("f737")), 14,
	0x1000, 0x00, 9, 2, 4, 1, 0x0bbf),
	new PICDevice(
	nameArr(_T("p16f74"), _T("16f74"), _T("pic16f74"), _T("f74")), 14,
	0x1000, 0x00, 8, 1, 4, 1, 0x063f, 0x02),
	new PICDevice(
	nameArr(_T("p16f747"), _T("16f747"), _T("pic16f747"), _T("f747")), 14,
	0x1000, 0x00, 9, 2, 4, 1, 0x0bff, 0x02),
	new PICDevice(
	nameArr(_T("p16f76"), _T("16f76"), _T("pic16f76"), _T("f76")), 14,
	0x2000, 0x00, 8, 1, 4, 1, 0x065f),
	new PICDevice(
	nameArr(_T("p16f767"), _T("16f767"), _T("pic16f767"), _T("f767")), 14,
	0x2000, 0x00, 9, 2, 4, 1, 0x0ebf),
	new PICDevice(
	nameArr(_T("p16f77"), _T("16f77"), _T("pic16f77"), _T("f77")), 14,
	0x2000, 0x00, 8, 1, 4, 1, 0x067f, 0x02),
	new PICDevice(
	nameArr(_T("p16f777"), _T("16f777"), _T("pic16f777"), _T("f777")), 14,
	0x2000, 0x00, 9, 2, 4, 1, 0x0dff),
	new PICDevice(
	nameArr(_T("p16f83"), _T("16f83"), _T("pic16f83"), _T("f83")), 14,
		0x0200, 0x040, 8, 1, 4, 1, 0xffff),
	new PICDevice(
	nameArr(_T("p16f84"), _T("16f84"), _T("pic16f84"), _T("f84")), 14,
		0x0400, 0x040, 8, 1, 4, 1, 0xffff),
	new PICDevice(
	nameArr(_T("p16f84A"), _T("16f84A"), _T("pic16f84A"), _T("f84A")), 14,
		0x0400, 0x040, 8, 1, 4, 1, 0x057f),
	new PICDevice(
	nameArr(_T("p16f87"), _T("16f87"), _T("pic16f87"), _T("f87")), 14,
	0x1000, 0x100, 8, 2, 4, 1, 0x072f),
	new PICDevice(
	nameArr(_T("p16f88"), _T("16f88"), _T("pic16f88"), _T("f88")), 14,
	0x1000, 0x100, 8, 2, 4, 1, 0x076f),
	new PICDevice(
	nameArr(_T("p16f913"), _T("16f913"), _T("pic16f913"), _T("f913")), 14,
	0x1000, 0x100, 8, 2, 4, 1, 0x13ef),
	new PICDevice(
	nameArr(_T("p16f916"), _T("16f916"), _T("pic16f916"), _T("f916")), 14,
	0x2000, 0x100, 8, 2, 4, 1, 0x13af),

	new PICDevice(
	nameArr(_T("p18f1220"), _T("18f1220"), _T("pic18f1220"), _T("1220")), 16,
	0x0800, 0x100, 8, 8, 4, 1, 0x07ff),
	new PICDevice(
	nameArr(_T("p18f1320"), _T("18f1320"), _T("pic18f1320"), _T("1320")), 16,
	0x1000, 0x100, 8, 8, 4, 1, 0x07df),
	new PICDevice(
	nameArr(_T("p18f2220"), _T("18f2220"), _T("pic18f2220"), _T("2220")), 16,
	0x0800, 0x100, 8, 8, 4, 1, 0x059f),
	new PICDevice(
	nameArr(_T("p18f2221"), _T("18f2221"), _T("pic18f2221"), _T("2221")), 16,
	0x0800, 0x100, 8, 8, 4, 1, 0x217f),
	new PICDevice(
	nameArr(_T("p18f2320"), _T("18f2320"), _T("pic18f2320"), _T("2320")), 16,
	0x1000, 0x100, 8, 8, 4, 1, 0x051f),
	new PICDevice(
	nameArr(_T("p18f2321"), _T("18f2321"), _T("pic18f2321"), _T("2321")), 16,
	0x1000, 0x100, 8, 8, 4, 1, 0x213f),
	new PICDevice(
	nameArr(_T("p18f2331"), _T("18f2331"), _T("pic18f2331"), _T("2331")), 16,
	0x1000, 0x100, 8, 8, 4, 1, 0x08ff),
	new PICDevice(
	nameArr(_T("p18f2410"), _T("18f2410"), _T("pic18f2410"), _T("2410")), 16,
	0x2000, 0x100, 8, 8, 4, 1, 0x117f),
	new PICDevice(
	nameArr(_T("p18f242"), _T("18f242"), _T("pic18f242"), _T("242")), 16,
	0x2000, 0x100, 8, 8, 4, 1, 0x049f),
	new PICDevice(
	nameArr(_T("p18f2420"), _T("18f2420"), _T("pic18f2420"), _T("2420")), 16,
	0x2000, 0x100, 8, 8, 4, 1, 0x115f),
	new PICDevice(
	nameArr(_T("p18f2431"), _T("18f2431"), _T("pic18f2431"), _T("2431")), 16,
	0x2000, 0x100, 8, 8, 4, 1, 0x08df),
	new PICDevice(
	nameArr(_T("p18f2439"), _T("18f2439"), _T("pic18f2439"), _T("2439")), 16,
	0x1800, 0x100, 8, 8, 4, 1, 0x0000),
	new PICDevice(
	nameArr(_T("p18f2450"), _T("18f2450"), _T("pic18f2450"), _T("2450")), 16,
	0x2000, 0x00, 8, 8, 4, 1, 0x142f),
	new PICDevice(
	nameArr(_T("p18f2455"), _T("18f2455"), _T("pic18f2455"), _T("2455")), 16,
	0x3000, 0x100, 8, 8, 4, 1, 0x127f),
	new PICDevice(
	nameArr(_T("p18f248"), _T("18f248"), _T("pic18f248"), _T("248")), 16,
	0x2000, 0x100, 8, 8, 4, 1, 0x081f),
	new PICDevice(
	nameArr(_T("p18f2480"), _T("18f2480"), _T("pic18f2480"), _T("2480")), 16,
	0x2000, 0x100, 8, 8, 4, 1, 0x1aff),
	new PICDevice(
	nameArr(_T("p18f2510"), _T("18f2510"), _T("pic18f2510"), _T("2510")), 16,
	0x4000, 0x100, 8, 8, 4, 1, 0x113f),
	new PICDevice(
	nameArr(_T("p18f252"), _T("18f252"), _T("pic18f252"), _T("252")), 16,
	0x4000, 0x100, 8, 8, 4, 1, 0x041f),
	new PICDevice(
	nameArr(_T("p18f2515"), _T("18f2515"), _T("pic18f2515"), _T("2515")), 16,
	0x6000, 0x100, 8, 8, 4, 1, 0x0cff),
	new PICDevice(
	nameArr(_T("p18f2520"), _T("18f2520"), _T("pic18f2520"), _T("2520")), 16,
	0x4000, 0x100, 8, 8, 4, 1, 0x111f),
	new PICDevice(
	nameArr(_T("p18f2525"), _T("18f2525"), _T("pic18f2525"), _T("2525")), 16,
	0x6000, 0x100, 8, 8, 4, 1, 0x0cbf),
	new PICDevice(
	nameArr(_T("p18f2539"), _T("18f2539"), _T("pic18f2539"), _T("2539")), 16,
	0x3000, 0x100, 8, 8, 4, 1, 0x0000),
	new PICDevice(
	nameArr(_T("p18f2550"), _T("18f2550"), _T("pic18f2550"), _T("2550")), 16,
	0x4000, 0x100, 8, 8, 4, 1, 0x125f),
	new PICDevice(
	nameArr(_T("p18f258"), _T("18f258"), _T("pic18f258"), _T("258")), 16,
	0x4000, 0x100, 8, 8, 4, 1, 0x085f),
	new PICDevice(
	nameArr(_T("p18f2580"), _T("18f2580"), _T("pic18f2580"), _T("2580")), 16,
	0x4000, 0x100, 8, 8, 4, 1, 0x1adf),
	new PICDevice(
	nameArr(_T("p18f2585"), _T("18f2585"), _T("pic18f2585"), _T("2585")), 16,
	0x6000, 0x100, 8, 8, 4, 1, 0x0eff),
	new PICDevice(
	nameArr(_T("p18f2610"), _T("18f2610"), _T("pic18f2610"), _T("2610")), 16,
	0x8000, 0x400, 8, 8, 4, 1, 0x0cbf),
	new PICDevice(
	nameArr(_T("p18f2620"), _T("18f2620"), _T("pic18f2620"), _T("2620")), 16,
	0x8000, 0x400, 8, 8, 4, 1, 0x0c9f),
	new PICDevice(
	nameArr(_T("p18f2680"), _T("18f2680"), _T("pic18f2680"), _T("2680")), 16,
	0x8000, 0x400, 8, 8, 4, 1, 0x0edf),
	new PICDevice(
	nameArr(_T("p18f4220"), _T("18f4220"), _T("pic18f4220"), _T("4220")), 16,
	0x0800, 0x100, 8, 8, 4, 1, 0x05bf, 0x02),
	new PICDevice(
	nameArr(_T("p18f4320"), _T("18f4320"), _T("pic18f4320"), _T("4320")), 16,
	0x1000, 0x100, 8, 8, 4, 1, 0x053f, 0x02),
	new PICDevice(
	nameArr(_T("p18f4221"), _T("18f4221"), _T("pic18f4221"), _T("4221")), 16,
	0x0800, 0x100, 8, 8, 4, 1, 0x215f, 0x02),
	new PICDevice(
	nameArr(_T("p18f4321"), _T("18f4321"), _T("pic18f4321"), _T("4321")), 16,
	0x1000, 0x100, 8, 8, 4, 1, 0x211f, 0x02),
	new PICDevice(
	nameArr(_T("p18f4331"), _T("18f4331"), _T("pic18f4331"), _T("4331")), 16,
	0x1000, 0x100, 8, 8, 4, 1, 0x08bf, 0x02),
	new PICDevice(
	nameArr(_T("p18f4410"), _T("18f4410"), _T("pic18f4410"), _T("4410")), 16,
	0x2000, 0x100, 8, 8, 4, 1, 0x10ff, 0x02),
	new PICDevice(
	nameArr(_T("p18f4420"), _T("18f4420"), _T("pic18f4420"), _T("4420")), 16,
	0x2000, 0x100, 8, 8, 4, 1, 0x10df, 0x02),
	new PICDevice(
	nameArr(_T("p18f442"), _T("18f442"), _T("pic18f442"), _T("442")), 16,
	0x2000, 0x100, 8, 8, 4, 1, 0x04bf, 0x02),
	new PICDevice(
	nameArr(_T("p18f4431"), _T("18f4431"), _T("pic18f4431"), _T("4431")), 16,
	0x2000, 0x100, 8, 8, 4, 1, 0x089f, 0x02),
	new PICDevice(
	nameArr(_T("p18f4439"), _T("18f4439"), _T("pic18f4439"), _T("4439")), 16,
	0x1800, 0x100, 8, 8, 4, 1, 0x0000, 0x02),
	new PICDevice(
	nameArr(_T("p18f4450"), _T("18f4450"), _T("pic18f4450"), _T("4450")), 16,
	0x2000, 0x00, 8, 8, 4, 1, 0x140f, 0x02),
	new PICDevice(
	nameArr(_T("p18f4455"), _T("18f4455"), _T("pic18f4455"), _T("4455")), 16,
	0x3000, 0x100, 8, 8, 4, 1, 0x123f, 0x02),
	new PICDevice(
	nameArr(_T("p18f448"), _T("18f448"), _T("pic18f448"), _T("448")), 16,
	0x2000, 0x100, 8, 8, 4, 1, 0x083f, 0x02),
	new PICDevice(
	nameArr(_T("p18f4480"), _T("18f4480"), _T("pic18f4480"), _T("4480")), 16,
	0x2000, 0x100, 8, 8, 4, 1, 0x1abf, 0x02),
	new PICDevice(
	nameArr(_T("p18f4510"), _T("18f4510"), _T("pic18f4510"), _T("4510")), 16,
	0x8000, 0x100, 8, 8, 4, 1, 0x10bf, 0x02),
	new PICDevice(
	nameArr(_T("p18f4515"), _T("18f4515"), _T("pic18f4515"), _T("4515")), 16,
	0x6000, 0x100, 8, 8, 4, 1, 0x0c7f, 0x02),
	new PICDevice(
	nameArr(_T("p18f452"), _T("18f452"), _T("pic18f452"), _T("452")), 16,
	0x4000, 0x100, 8, 8, 4, 1, 0x043f, 0x02),
	new PICDevice(
	nameArr(_T("p18f4520"), _T("18f4520"), _T("pic18f4520"), _T("4520")), 16,
	0x8000, 0x100, 8, 8, 4, 1, 0x109f, 0x02),
	new PICDevice(
	nameArr(_T("p18f4525"), _T("18f4525"), _T("pic18f4525"), _T("4525")), 16,
	0x6000, 0x100, 8, 8, 4, 1, 0x0c5f, 0x02),
	new PICDevice(
	nameArr(_T("p18f4539"), _T("18f4539"), _T("pic18f4539"), _T("4539")), 16,
	0x3000, 0x100, 8, 8, 4, 1, 0x043f, 0x02), // Note: Dev ID same as 18F452
	new PICDevice(
	nameArr(_T("p18f4550"), _T("18f4550"), _T("pic18f4550"), _T("4550")), 16,
	0x4000, 0x100, 8, 8, 4, 1, 0x121f, 0x02),
	new PICDevice(
	nameArr(_T("p18f458"), _T("18f458"), _T("pic18f458"), _T("458")), 16,
	0x4000, 0x100, 8, 8, 4, 1, 0x087f, 0x02),
	new PICDevice(
	nameArr(_T("p18f4580"), _T("18f4580"), _T("pic18f4580"), _T("4580")), 16,
	0x4000, 0x100, 8, 8, 4, 1, 0x1a9f, 0x02),
	new PICDevice(
	nameArr(_T("p18f4585"), _T("18f4585"), _T("pic18f4585"), _T("4585")), 16,
	0x6000, 0x100, 8, 8, 4, 1, 0x0ebf, 0x02),
	new PICDevice(
	nameArr(_T("p18f4610"), _T("18f4610"), _T("pic18f4610"), _T("4610")), 16,
	0x8000, 0x400, 8, 8, 4, 1, 0x0c3f, 0x02),
	new PICDevice(
	nameArr(_T("p18f4620"), _T("18f4620"), _T("pic18f4620"), _T("4620")), 16,
	0x8000, 0x400, 8, 8, 4, 1, 0x0c1f, 0x02),
	new PICDevice(
	nameArr(_T("p18f4680"), _T("18f4680"), _T("pic18f4680"), _T("4680")), 16,
	0x8000, 0x400, 8, 8, 4, 1, 0x0e9f, 0x02),

	new PICDevice(
	nameArr(_T("p30f2010"), _T("30f2010"), _T("dspic30f2010"), _T("2010")), 24,
	0x2000, 0x400, 8, 8, 0x600, 2, 0x00c0),
	new PICDevice(
	nameArr(_T("p30f2011"), _T("30f2011"), _T("dspic30f2011"), _T("2011")), 24,
	0x2000, 0x00, 8, 8, 0x600, 2, 0x0240),
	new PICDevice(
	nameArr(_T("p30f2012"), _T("30f2012"), _T("dspic30f2012"), _T("2012")), 24,
	0x2000, 0x00, 8, 8, 0x600, 2, 0x0241),
	new PICDevice(
	nameArr(_T("p30f3012"), _T("30f3012"), _T("dspic30f3012"), _T("3012")), 24,
	0x4000, 0x00, 8, 8, 0x600, 2, 0x00c1),
	new PICDevice(
	nameArr(_T("p30f3013"), _T("30f3013"), _T("dspic30f3013"), _T("3013")), 24,
	0x4000, 0x00, 8, 8, 0x600, 2, 0x00c3),
	new PICDevice(
	nameArr(_T("p30f4011"), _T("30f4011"), _T("dspic30f4011"), _T("4011")), 24,
	0x8000, 0x400, 8, 8, 0x600, 2, 0x0101),
	new PICDevice(
	nameArr(_T("p30f4012"), _T("30f4012"), _T("dspic30f4012"), _T("4012")), 24,
	0x8000, 0x400, 8, 8, 0x600, 2, 0x0100),
	new PICDevice(
	nameArr(_T("p30f4013"), _T("30f4013"), _T("dspic30f4013"), _T("4013")), 24,
	0x8000, 0x400, 8, 8, 0x600, 2, 0x0141),


	NULL
};

LPCTSTR *DeviceList::nameArr(LPCTSTR n1, LPCTSTR n2, LPCTSTR n3, LPCTSTR n4)
{
	LPCTSTR *names = new LPCTSTR[4];
	names[0] = n1;
	names[1] = n2;
	names[2] = n3;
	names[3] = n4;

	return names;
}
const int DeviceList::DeviceIndex(LPCTSTR deviceName)
{
	// Find the device index the hard way - look at every spelling
	// of every device we know about.
	int entryCount = sizeof(Devices)/sizeof(Devices[0]);
	for (int i=0;i<entryCount;i++)
	{
		if (Devices[i] == NULL)
			continue;
		if (!_tcsicmp(deviceName, Devices[i]->name[0])) return i;
		if (!_tcsicmp(deviceName, Devices[i]->name[1])) return i;
		if (!_tcsicmp(deviceName, Devices[i]->name[2])) return i;
		if (!_tcsicmp(deviceName, Devices[i]->name[3])) return i;
	}

	return -1;
}

int DeviceList::deviceCount(void)
{
	return sizeof(Devices)/sizeof(Devices[0]);
}
