#include "stdafx.h"
#include <assert.h>
#include "pic18fxx31program.h"

PIC18FXX31Program PIC18FXX31Program::m_Driver;

// Get the singleton programmer instance for this class of chips
const PICProgram *PIC18FXX31Program::GetDriver()
{
	return &m_Driver;
}

#if 1

// The PIC18FXX31 devices require a 8 byte write.
int PIC18FXX31Program::WriteCode(const PICDevice *device, CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	uint addr, i;
	if (statusIF) statusIF->InitProgress(_T("Writing Program Memory"));

	// read eeprom data
	PICOpen(port, device->devVppMask, false);

	if (!retryOpen(device, port))
	{
		// Issue a warning that we aren't getting anything from the chip
		return 1;
	}

	// Setup for write to code memory (single panel writes)
	i = setupWriteCode(device, port, hexFile);

#if 1
	// Move to the beginning of code memory
	i = SetTablePointer(port, 0x0000);

	// Write code memory in CMD_BLOCKSIZE byte chunks.
	byte outBuf[128], inBuf[128];
	uint block, resLen;
	uint blockSize  = 64;
	// Note: write buffer is 8 bytes for 18Fxx31 chips.
	uint bufferSize = 8;
	for (addr=0,block=0;addr<device->programWordsSize;block+=blockSize)
	{
		uint byteAddr = addr * 2;
		outBuf[0] = byteAddr & 0xff;
		outBuf[1] = (byteAddr & 0xff00) >> 8;
		outBuf[2] = (byteAddr & 0xff0000) >> 16;
		outBuf[3] = bufferSize; // # of bytes per pass within the block write routine (adjust by processor)

		ushort *valBuf = (ushort *)&outBuf[4]; // Location of first value in response buffer

		for (uint i0=0;i0<blockSize;i0+=2)
		{
			*valBuf++ = hexFile->programWords[addr++];
		}
		i = PICProgram::SendPacket(port, PICProgram::EVA_PIC16WB_CMD,
			blockSize+4, outBuf, 128, &resLen, inBuf, 4*blockSize);
		if (i) { PICClose(port); return addr+1; }
		if (inBuf[0] == 0xee)
		{
			// Write failed
			return addr+1;
		}
		if (memcmp(&outBuf[4], &inBuf[6], blockSize) != 0)
		{
			// Buffers don't match - programming failed
			return addr+1;
		}

		if (statusIF && !statusIF->UpdateProgress(addr, device->programWordsSize))
		{
			PICClose(port);
			return 1;
		}
	}
#else
	uint j;
	// Loop through memory writing 4 words (8 bytes) at a time.
	// for(addr=0;addr<device->programWordsSize;addr+=4)
	for(addr=0;addr<128;addr+=4)
	{
		uint baddr = 2 * addr;
#if 1
		// Set the code address pointer
		byte TBLPTRU = (baddr & 0xff0000) >> 16;
		byte TBLPTRH = (baddr & 0xff00) >> 8;
		byte TBLPTRL = (baddr & 0xff);
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | (0x0E00 | TBLPTRU), &j);
		i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EF8, &j);
		i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | (0x0E00 | TBLPTRH), &j);
		i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EF7, &j);
		i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | (0x0E00 | TBLPTRL), &j);
		i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EF6, &j);
		i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE, &j);

		// Enable memory writes
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x84A6, &j); // bsf EECON1, WREN
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x88A6, &j); // bsf EECON1, FREE

		// Perform required sequence
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0E55, &j); // movlw 0x55
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EA7, &j); // movwf EECON2
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0EAA, &j); // movlw 0xaa
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EA7, &j); // movwf EECON2

		// Initiate erase
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x82A6, &j); // bsf EECON1, WR
		i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE, &j);     // NOP

		::Sleep(5);

		// Initiate erase
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x94A6, &j); // bcf EECON1, WREN
#endif

		i = SetTablePointer(port, baddr);

		// Load the write buffer at the current address
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_PBLWRT_POSTINC | hexFile->programWords[addr], &j);
		i += TxCmd(port, EVA_PIC16_CMD, PIC16_PBLWRT_POSTINC | hexFile->programWords[addr+1], &j);
		i += TxCmd(port, EVA_PIC16_CMD, PIC16_PBLWRT_POSTINC | hexFile->programWords[addr+2], &j);
		i += TxCmd(port, EVA_PIC16_CMD, PIC16_STARTPROG | hexFile->programWords[addr+3], &j);

		// At this point we need to emit a NOP command and hold the CLK line high
		// for the duration of the write.

		i += TxCmd(port, EVA_PIC16W_CMD, PIC16_CORE | 0x0000, &j);

		if (i) { PICClose(port); return addr+1; }

		if (statusIF && !statusIF->UpdateProgress(addr, device->programWordsSize))
		{
			PICClose(port);
			return 1;
		}
	}
#endif

	PICClose(port);

	return 0;
}
#endif
