#include "StdAfx.h"
#include <assert.h>
#include "PIC18FXXXProgram.h"

PIC18FXXXProgram::PIC18FXXXProgram(void)
{
}

PIC18FXXXProgram::~PIC18FXXXProgram(void)
{
}

PIC18FXXXProgram PIC18FXXXProgram::m_Driver;

// Get the singleton programmer instance for this class of chips
const PICProgram *PIC18FXXXProgram::GetDriver()
{
	return &m_Driver;
}

bool PIC18FXXXProgram::retryOpen(const PICDevice *device, CIOPort *port) const
{
#if 0
	return true;
#else
	uint b0, b1;
	for (int r=0;r<4;r++)
	{
		// read device ID
		SetTablePointer(port, 2*device->deviceIDAddr);
		TxCmd(port, EVA_PIC16R_CMD, PIC16_TBLRD_POSTINC, &b0);
		TxCmd(port, EVA_PIC16R_CMD, PIC16_TBLRD_POSTINC, &b1);
		b0 &= 0xff;
		b1 &= 0xff;
		if ((b0 != 0xff && b0 != 0x80 && b0 != 0x00)
		 || (b1 != 0xff && b1 != 0x80 && b1 != 0x00))
			return true;

		// Not seeing good values - lower and raise MCLR and try again
		PICClose(port);
		PICOpen(port, device->devVppMask, false);
	}

	return false;
#endif
}

int PIC18FXXXProgram::getWriteBufferSize(const PICDevice *device) const
{
	int bufferSize = 8;
	const TCHAR *devName = device->name[2];
	if (!_tcsicmp(devName, _T("pic18f2221"))) bufferSize = 8;
	if (!_tcsicmp(devName, _T("pic18f2321"))) bufferSize = 8;
	if (!_tcsicmp(devName, _T("pic18f4221"))) bufferSize = 8;
	if (!_tcsicmp(devName, _T("pic18f4321"))) bufferSize = 8;

	if (!_tcsicmp(devName, _T("pic18f2410"))) bufferSize = 32;
	if (!_tcsicmp(devName, _T("pic18f2510"))) bufferSize = 32;
	if (!_tcsicmp(devName, _T("pic18f4410"))) bufferSize = 32;
	if (!_tcsicmp(devName, _T("pic18f2420"))) bufferSize = 32;
	if (!_tcsicmp(devName, _T("pic18f2520"))) bufferSize = 32;
	if (!_tcsicmp(devName, _T("pic18f4420"))) bufferSize = 32;
	if (!_tcsicmp(devName, _T("pic18f4520"))) bufferSize = 32;
	if (!_tcsicmp(devName, _T("pic18f2480"))) bufferSize = 32;
	if (!_tcsicmp(devName, _T("pic18f2580"))) bufferSize = 32;
	if (!_tcsicmp(devName, _T("pic18f4480"))) bufferSize = 32;
	if (!_tcsicmp(devName, _T("pic18f4580"))) bufferSize = 32;
	if (!_tcsicmp(devName, _T("pic18f2455"))) bufferSize = 32;
	if (!_tcsicmp(devName, _T("pic18f2550"))) bufferSize = 32;
	if (!_tcsicmp(devName, _T("pic18f4455"))) bufferSize = 32;
	if (!_tcsicmp(devName, _T("pic18f4550"))) bufferSize = 32;

	if (!_tcsicmp(devName, _T("pic18f2515"))) bufferSize = 64;
	if (!_tcsicmp(devName, _T("pic18f2610"))) bufferSize = 64;
	if (!_tcsicmp(devName, _T("pic18f4515"))) bufferSize = 64;
	if (!_tcsicmp(devName, _T("pic18f4610"))) bufferSize = 64;
	if (!_tcsicmp(devName, _T("pic18f2525"))) bufferSize = 64;
	if (!_tcsicmp(devName, _T("pic18f2620"))) bufferSize = 64;
	if (!_tcsicmp(devName, _T("pic18f4525"))) bufferSize = 64;
	if (!_tcsicmp(devName, _T("pic18f4620"))) bufferSize = 64;
	if (!_tcsicmp(devName, _T("pic18f2585"))) bufferSize = 64;
	if (!_tcsicmp(devName, _T("pic18f2680"))) bufferSize = 64;
	if (!_tcsicmp(devName, _T("pic18f4585"))) bufferSize = 64;
	if (!_tcsicmp(devName, _T("pic18f4680"))) bufferSize = 64;

	return bufferSize;
}

int PIC18FXXXProgram::getEraseBufferSize(const PICDevice *device) const
{
	return 64;
}

int PIC18FXXXProgram::ReadCode(const PICDevice *device, CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	uint addr, i;
	if (statusIF) statusIF->InitProgress(_T("Reading Program Memory"));

	// Reset PIC.  After this is executed we will be at address 0 and can
	// increment to get to the address we actually want to read.
	PICOpen(port, device->devVppMask, false);

	if (!retryOpen(device, port))
	{
		// Issue a warning that we aren't getting anything from the chip
	}

#if 1
	// Read code memory a block at a time.  Using a large block (max 64 bytes in current
	// firmware) reduces the overhead associated with sending/recieving packets over the
	// serial port.
	byte outBuf[4];
	byte inBuf[128];

	uint block, resLen;
	uint blockSize = CMD_BLOCKSIZE;
	for (addr=0,block=0;addr<device->programWordsSize;block+=blockSize)
	{
		uint byteAddr = addr * 2;
		outBuf[0] = byteAddr & 0xff;
		outBuf[1] = (byteAddr & 0xff00) >> 8;
		outBuf[2] = (byteAddr & 0xff0000) >> 16;
		outBuf[3] = blockSize;
#if 1
		i = PICProgram::SendPacket(port, PICProgram::EVA_PIC16RB_CMD,
			4, outBuf, 128, &resLen, inBuf, 4*blockSize);
		if (i)
		{
			PICClose(port);
			return addr+1;
		}
#else
		i = 1;
		for (int retryCtr=0;i!=0 && retryCtr<8;retryCtr++)
		{
			i = PICProgram::SendPacket(port, PICProgram::EVA_PIC16RB_CMD,
				4, outBuf, 128, &resLen, inBuf, 2*blockSize);
			if (i)
			{
				if (retryCtr<3)
					continue;
				PICClose(port);
				return addr+1;
			}
			else if (inBuf[0] == 0x00 && inBuf[1] == 0x00 && inBuf[2] == 0x00
				&& inBuf[3] == 0x00 && inBuf[4] == 0x00 && inBuf[5] == 0x00)
			{
				if (retryCtr < 8)
				{
					i = -1;
					continue;
				}
				// Read gave back garbage - USB problem?  This only seems
				// to happen on fast hubs.
				return addr+1;
			}
			else
				break;
		}
#endif
		// Move the block response into the result array
		ushort *valBuf = (ushort *)&inBuf[6]; // Location of first value in response buffer
		for (uint i0=0;i0<blockSize;i0+=2)
		{
			// The programmer sends back the data LSB first so we can just pull
			// the values right out of the buffer.
			hexFile->programWords[addr++] = *valBuf++;
		}

		if (statusIF && !statusIF->UpdateProgress(addr, device->programWordsSize))
		{
			PICClose(port);
			return 1;
		}
	}
#else
	// Read the contents of the code memory one word at a time.  Very inefficient because
	// there is the overhead of sending/receiving an entire packet for each word.  This
	// routine is very reliable and can be used when testing the block based read routine above.
	uint b0, b1;
	uint wordVal;

	// Set the starting address to 0x000000
	i = SetTablePointer(port, 0x000000);
	if (i) { PICClose(port); return(1); }

	// read code
	for (addr=0;addr<device->programWordsSize;addr++)
	{
		// Read the current address and increment the table pointer
		i  = TxCmd(port, EVA_PIC16R_CMD, PIC16_TBLRD_POSTINC, &b0);
		i += TxCmd(port, EVA_PIC16R_CMD, PIC16_TBLRD_POSTINC, &b1);

		wordVal = ((b1 & 0x00ff) << 8) | (b0 & 0xff);

		// Check for any errors
		if (i) { PICClose(port); return(1); }

		assert(addr < device->programWordsSize);

		// Save the contents of the address.  Note that the response contains the
		// start and stop bit from the PIC.  To get the final 1 bit contents
		// we need to shift and mask the extra bits.
		hexFile->programWords[addr] = wordVal;

		if (statusIF && !statusIF->UpdateProgress(addr, device->programWordsSize))
		{
			PICClose(port);
			return 1;
		}
	}
#endif

	PICClose(port);

	return 0;
}

int PIC18FXXXProgram::ReadConfig(const PICDevice *device, CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	uint b0, b1;
	uint addr, wordVal, i;
	if (statusIF) statusIF->InitProgress(_T("Reading Config Bits"));

	// Reset PIC
	PICOpen(port, device->devVppMask, false);

	if (!retryOpen(device, port))
	{
		// Issue a warning that we aren't getting anything from the chip
	}

	// Set the starting address to 0x300000
	i = SetTablePointer(port, 2*device->configAddr);
	if (i) { PICClose(port); return(1); }

	// read configuration bits (TBD: config is read a byte at a time - need
	// to assemble the bytes as they come in.)
	for (addr=0;addr<device->configWordsSize;addr++)
	{
		// Read the current address and increment the table pointer
		i  = TxCmd(port, EVA_PIC16R_CMD, PIC16_TBLRD_POSTINC, &b0);
		i += TxCmd(port, EVA_PIC16R_CMD, PIC16_TBLRD_POSTINC, &b1);

		// wordVal = (b1 & 0xff00) | (b0 >> 8);
		wordVal = ((b1 & 0x00ff) << 8) | (b0 & 0xff);
		// wordVal = (b1 | b0);

		// Check for any errors
		if (i)
		{
			PICClose(port);
			return(1);
		}

		assert(addr < device->configWordsSize);

		// Save the contents of the address.  Note that the response contains the
		// start and stop bit from the PIC.  To get the final 1 bit contents
		// we need to shift and mask the extra bits.
		hexFile->configWords[addr] = wordVal;

		if (statusIF && !statusIF->UpdateProgress(addr, device->configWordsSize))
		{
			PICClose(port);
			return 1;
		}
	}

	// read device ID
	SetTablePointer(port, 2*device->deviceIDAddr);
	i  = TxCmd(port, EVA_PIC16R_CMD, PIC16_TBLRD_POSTINC, &b0);
	i += TxCmd(port, EVA_PIC16R_CMD, PIC16_TBLRD_POSTINC, &b1);
	hexFile->deviceID[0] = ((b1 & 0x00ff) << 8) | (b0 & 0xff);

	// read user ID
	if (statusIF) statusIF->InitProgress(_T("Reading User ID Values"));
	SetTablePointer(port, 2*device->userIDWordsAddr);
	for (addr=0;addr<device->userIDWordsSize;addr++)
	{
		// Read the current address and increment the table pointer
		i  = TxCmd(port, EVA_PIC16R_CMD, PIC16_TBLRD_POSTINC, &b0);
		i += TxCmd(port, EVA_PIC16R_CMD, PIC16_TBLRD_POSTINC, &b1);

		wordVal = ((b1 & 0x00ff) << 8) | (b0 & 0xff);

		// Check for any errors
		if (i) { PICClose(port); return 1; }

		assert(addr < device->userIDWordsSize);

		// Save the contents of the address.  Note that the response contains the
		// start and stop bit from the PIC.  To get the final 1 bit contents
		// we need to shift and mask the extra bits.
		hexFile->userIDWords[addr] = wordVal;

		if (statusIF && !statusIF->UpdateProgress(addr, device->configWordsSize))
		{
			PICClose(port);
			return 1;
		}
	}

	PICClose(port);

	return 0;
}

// Read a single byte from the data EEPROM.  Note: the programmer MUST have already been
// set up in programming mode.
int PIC18FXXXProgram::ReadDataByte(const PICDevice *device, CIOPort *port, uint addr, byte *val) const
{
	uint i, j;

	// read data
	uint b0;

	// Enable direct access to data EEPROM
	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x9EA6, &j);
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x9CA6, &j);

	// Set address of the byte we want to read
	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0E00 | (addr & 0xFF), &j);
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EA9, &j);

	// Set address of the byte we want to read
	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0E00 | ((addr & 0xFF00) >> 8), &j);
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EAA, &j);

	// Initiate a memory read
	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x80A6, &j);

	// Load the data into the serial data holding register
	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x50A8, &j);
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EF5, &j);

	// Shift out the data
	i  = TxCmd(port, EVA_PIC16R_CMD, PIC16_SHIFT_TBLAT, &b0);

	*val = (b0 & 0xff);

	return i;
}

int PIC18FXXXProgram::ReadData(const PICDevice *device, CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	int i;
	uint addr;
	if (statusIF) statusIF->InitProgress(_T("Reading Data Memory"));

	// Reset PIC
	PICOpen(port, device->devVppMask, false);

	if (!retryOpen(device, port))
	{
		// Issue a warning that we aren't getting anything from the chip
	}

#if 1
	byte outBuf[4], inBuf[128];
	uint block, resLen;
	uint blockSize = CMD_BLOCKSIZE;
	for (addr=0,block=0;addr<device->EEPROMDataSize;block+=blockSize)
	{
		uint byteAddr = addr;
		outBuf[0] = byteAddr & 0xff;
		outBuf[1] = (byteAddr & 0xff00) >> 8;
		outBuf[2] = (byteAddr & 0xff0000) >> 8;
		outBuf[3] = blockSize;
		i = PICProgram::SendPacket(port, PICProgram::EVA_PIC16RBE_CMD,
			4, outBuf, 128, &resLen, inBuf, 2*blockSize);
		if (i) { PICClose(port); return addr+1; }

		// Move the block response into the result array
		byte *valBuf = &inBuf[6]; // Location of first value in response buffer
		for (uint i0=0;i0<blockSize;i0++)
		{
			// The programmer sends back byte data so we can pull
			// the values right out of the buffer.
			hexFile->EEPROMData[addr++] = *valBuf++;
		}

		if (statusIF && !statusIF->UpdateProgress(addr, device->EEPROMDataSize))
		{
			PICClose(port);
			return 1;
		}
	}
#else
	uint wordVal, j;

	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x9EA6, &j);
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x9CA6, &j);

	// read data
	uint b0;
	for (addr=0;addr<device->EEPROMDataSize;addr++)
	{
		uint bAddr = (addr >= 0 ? addr : 0);

		// Set address of the byte we want to read
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0E00 | (bAddr & 0xFF), &j);
		i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EA9, &j);

		// Set address of the byte we want to read
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0E00 | ((bAddr & 0xFF00) >> 8), &j);
		i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EAA, &j);

		// Initiate a memory read
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x80A6, &j);

		// Load the data into the serial data holding register
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x50A8, &j);
		i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EF5, &j);

		// i = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0000, &j);

		// Shift out the data
		i  = TxCmd(port, EVA_PIC16R_CMD, PIC16_SHIFT_TBLAT, &b0);

		wordVal = (b0 & 0xff);

		// Check for any errors
		if (i) { PICClose(port); return(1); }

		assert(bAddr < device->EEPROMDataSize);

		// Save the contents of the address.
		hexFile->EEPROMData[bAddr] = wordVal;

		if (statusIF && !statusIF->UpdateProgress(bAddr, device->EEPROMDataSize))
		{
			PICClose(port);
			return 1;
		}
	}
#endif

	PICClose(port);

	return 0;
}


int PIC18FXXXProgram::PartialRead(CIOPort *port, PICHexFile *hexFile, bool codeFlag, bool dataFlag, bool configFlag, CProgrammerProgressIF *statusIF) const
{
	const PICDevice *device = hexFile->Processor;

    if (!SUCCEEDED(ResetProgrammer(port)))
	{
		// Failed to get the programmer into a known good state.
		return 1;
	}

	int res = 0;
	if (codeFlag && (res = ReadCode(device, port, hexFile, statusIF))) return res;
	if (configFlag && (res == 0) && (res = ReadConfig(device, port, hexFile, statusIF))) return res;
	if (dataFlag && (res == 0) && (res = ReadData(device, port, hexFile, statusIF))) return res;

	PICClose(port);

	return 0;
}

int PIC18FXXXProgram::Read(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	const PICDevice *device = hexFile->Processor;

    if (!SUCCEEDED(ResetProgrammer(port)))
	{
		// Failed to get the programmer into a known good state.
		if (!SUCCEEDED(ResetProgrammer(port)))
			return 1;
	}

	int res;
	if ((res = ReadCode(device, port, hexFile, statusIF)) != 0) return res;
	if ((res = ReadConfig(device, port, hexFile, statusIF)) != 0) return res;
	if ((res = ReadData(device, port, hexFile, statusIF)) != 0) return res;

	// done
	PICClose(port);

	return 0;
}

int PIC18FXXXProgram::setupWriteCode(const PICDevice *device, CIOPort *port, PICHexFile *hexFile) const
{
	uint i, j;

	// Direct access to config memory
	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x8EA6, &j); // bsf EECON1, EEPGD
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x8CA6, &j); // bcf EECON1, CFGS

	// Configure device for single panel writes
	i += SetTablePointer(port, 0x3c0006);
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_TBLWRT, &j);        // Write 0x00 to 0x3C0006 to enable single-panel writes

	// Direct access to code memory
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x8EA6, &j); // bsf EECON1, EEPGD
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x9CA6, &j); // bcf EECON1, CFGS

	return i;
}

int PIC18FXXXProgram::WriteCode(const PICDevice *device, CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	uint addr, i;
	if (statusIF) statusIF->InitProgress(_T("Writing Program Memory"));

	// read eeprom data
	PICOpen(port, device->devVppMask, false);

	if (!retryOpen(device, port))
	{
		// Issue a warning that we aren't getting anything from the chip
		return 1;
	}

#if 1
	// Setup for write to code memory (single panel writes)
	i = setupWriteCode(device, port, hexFile);
#else
	uint j;
	// Direct access to code memory
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x8EA6, &j); // bsf EECON1, EEPGD
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x9CA6, &j); // bcf EECON1, CFGS
#endif

	// Move to the beginning of code memory
	i = SetTablePointer(port, 0x0000);

#if 1
#if 1
	// Write code memory in CMD_BLOCKSIZE byte chunks.
	byte outBuf[128], inBuf[128];
	uint block, resLen;
	uint blockSize = CMD_BLOCKSIZE;
	uint bufferSize = getWriteBufferSize(device);
	uint endAddr = device->programWordsSize;
	for (addr=0,block=0;addr<endAddr;block+=blockSize)
	{
		uint byteAddr = addr * 2;
		outBuf[0] = byteAddr & 0xff;
		outBuf[1] = (byteAddr & 0xff00) >> 8;
		outBuf[2] = (byteAddr & 0xff0000) >> 16;
		outBuf[3] = bufferSize; // # of words per pass within the block write routine (adjust by processor)

		// i = SetTablePointer(port, byteAddr);

		ushort *valBuf = (ushort *)&outBuf[4]; // Location of first value in response buffer

		for (uint i0=0;i0<blockSize;i0+=2)
		{
			*valBuf++ = hexFile->programWords[addr++];
		}
#if 1
		i = 1;
		for (int retryCtr=0;i!=0 && retryCtr<8;retryCtr++)
		{
			i = PICProgram::SendPacket(port, PICProgram::EVA_PIC16WB_CMD,
				blockSize+4, outBuf, 128, &resLen, inBuf, 4*blockSize);
			if (i)
			{
				if (retryCtr<3)
					continue;
				PICClose(port);
				return addr+1;
			}
			if (inBuf[0] == 0xee)
			{
				if (retryCtr < 3)
					continue;
				// Write failed
				return addr+1;
			}
			if (memcmp(&outBuf[4], &inBuf[6], blockSize) != 0)
			{
				if (retryCtr < 3)
					continue;
				// Buffers don't match - programming failed
				return addr+1;
			}

			if (statusIF && !statusIF->UpdateProgress(addr, device->programWordsSize))
			{
				PICClose(port);
				return 1;
			}
		}
#else
		i = PICProgram::SendPacket(port, PICProgram::EVA_PIC16WB_CMD,
			blockSize+4, outBuf, 128, &resLen, inBuf, 64*blockSize);
		if (i)
		{
			PICClose(port);
			return addr+1;
		}
		if (inBuf[0] == 0xee)
		{
			// Write failed
			return addr+1;
		}
		if (memcmp(&outBuf[4], &inBuf[6], blockSize) != 0)
		{
			// Buffers don't match - programming failed
			return addr+1;
		}

		if (statusIF && !statusIF->UpdateProgress(addr, device->programWordsSize))
		{
			PICClose(port);
			return 1;
		}
#endif
	}
#else
	// Write code memory in 8 byte chunks.
	byte outBuf[128], inBuf[128];
	uint block, resLen;
	uint bufferSize = 8;
	uint endAddr = device->programWordsSize;
	for (addr=0;addr<endAddr;)
	{
		uint byteAddr = addr * 2;
		outBuf[0] = byteAddr & 0xff;
		outBuf[1] = (byteAddr & 0xff00) >> 8;
		outBuf[2] = (byteAddr & 0xff0000) >> 16;
		outBuf[3] = bufferSize/2; // # of bytes per pass within the block write routine (adjust by processor)

		ushort *valBuf = (ushort *)&outBuf[4]; // Location of first value in response buffer

		for (uint i0=0;i0<bufferSize;i0+=2)
		{
			*valBuf++ = hexFile->programWords[addr++];
		}

		i = PICProgram::SendPacket(port, PICProgram::EVA_PIC16WBUF_CMD,
			bufferSize+4, outBuf, 64, &resLen, inBuf, 4*bufferSize);

		if (i) { PICClose(port); return addr+1; }
		if (inBuf[0] == 0xee)
		{
			// Write failed
			return addr+1;
		}
		if (memcmp(&outBuf[4], &inBuf[6], bufferSize) != 0)
		{
			// Buffers don't match - programming failed
			// return addr+1;
		}

		if (statusIF && !statusIF->UpdateProgress(addr, device->programWordsSize))
		{
			PICClose(port);
			return 1;
		}
	}
#endif
#else
	i = SetTablePointer(port, 0x0000);

	// Loop through memory writing 4 words (8 bytes) at a time.
	uint endAddr = device->programWordsSize; // 128;
	for(addr=0;addr<endAddr;addr+=4)
	{
		// Write the four words
		i = WriteWord16(port, &hexFile->programWords[addr], RETRY_LIMIT);
		if (i) { PICClose(port); return addr+1; }

		if (statusIF && !statusIF->UpdateProgress(addr, device->programWordsSize))
		{
			PICClose(port);
			return 1;
		}
	}
#endif

	PICClose(port);

	return 0;
}

int PIC18FXXXProgram::WriteData(const PICDevice *device, CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	uint addr, i;
	if (statusIF) statusIF->InitProgress(_T("Writing EEPROM Memory"));

	// Open programmer
	PICOpen(port, device->devVppMask, false);

	if (!retryOpen(device, port))
	{
		// Issue a warning that we aren't getting anything from the chip
	}

	uint j;
	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x9EA6, &j); // bsf EECON1, EEPGD
	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x9CA6, &j); // bsf EECON1, CFGS

#if 1
	// Write data memory in CMD_BLOCKSIZE byte chunks.
	byte outBuf[128], inBuf[128];
	uint block, resLen;
	uint blockSize = CMD_BLOCKSIZE;
	for (addr=0,block=0;addr<device->EEPROMDataSize;block+=blockSize)
	{
		uint byteAddr = addr;
		outBuf[0] = byteAddr & 0xff;
		outBuf[1] = (byteAddr & 0xff00) >> 8;
		outBuf[2] = (byteAddr & 0xff0000) >> 16;
		outBuf[3] = CMD_BLOCKSIZE; // # of words per pass within the block write routine (adjust by processor)

		byte *valBuf = &outBuf[4]; // Location of first value in response buffer

		for (uint i0=0;i0<blockSize;i0++)
		{
			*valBuf++ = (hexFile->EEPROMData[addr++] & 0xff);
		}
		i = 1;
		for (int retryCtr=0;i!=0 && retryCtr<4;retryCtr++)
		{
			i = PICProgram::SendPacket(port, PICProgram::EVA_PIC16WBE_CMD,
				blockSize+4, outBuf, 128, &resLen, inBuf, 8*blockSize);
			if (i)
			{
				if (retryCtr<3)
					continue;
				PICClose(port);
				return addr+1;
			}
			if (inBuf[0] == 0xee)
			{
				if (retryCtr < 3)
					continue;
				// Write failed
				return addr+1;
			}
			if (memcmp(&outBuf[4], &inBuf[6], blockSize) != 0)
			{
				if (retryCtr < 3)
					continue;
				// Buffers don't match - programming failed
				return addr+1;
			}

			if (statusIF && !statusIF->UpdateProgress(addr, device->EEPROMDataSize))
			{
				PICClose(port);
				return 1;
			}
		}
	}
#else
	// Step through EEPROM values, writing and verifying one at a time
	for(addr=0;addr<device->EEPROMDataSize;addr++)
	{
		// Set the data EEPROM address pointer
		byte addrH = (addr & 0xff00) >> 8;
		byte addrL = (addr & 0xff);
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0E00 | addrL, &j);
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EA9, &j);       // movwf EEADR
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0E00 | addrH, &j);
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EAA, &j);       // movwf EEADRH

		// Load the data to be written
		byte data = (hexFile->EEPROMData[addr] & 0xff);
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0E00 | data, &j);  // movlw <Data>
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EA8, &j);       // movwf EEDATA

		// Enable memory writes
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x84A6, &j); // bsf EECON1, WREN

		// Perform required sequence
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0E55, &j); // movlw 0x55
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EA7, &j); // movwf EECON2
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0EAA, &j); // movlw 0xaa
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EA7, &j); // movwf EECON2

		// Initiate write
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x82A6, &j); // bsf EECON1, WR

		// Poll WR bit, repeat until clear
		for (int WRDelay=0;WRDelay<16;WRDelay++)
		{
			i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x50A6, &j); // movf EECON1, W, 0
			i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EF5, &j); // movwf TABLAT
			i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0000, &j); // NOP
			i  = TxCmd(port, EVA_PIC16R_CMD, PIC16_SHIFT_TBLAT, &j);  // shift out data
			byte EECON1 = j & 0xff;
			if (EECON1 & 0x02)
			{
				::Sleep(1);
			}
			else
				break;
		}

		// Disable writes
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x94A6, &j); // bcf EECON1, WREN

		if (statusIF && !statusIF->UpdateProgress(addr, device->EEPROMDataSize))
		{
			PICClose(port);
			return 1;
		}
	}
#endif
	// Shut down connection to programmer
	PICClose(port);

	return 0;
}

int PIC18FXXXProgram::WriteConfig(const PICDevice *device, CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	uint addr, i, j;

	if (statusIF) statusIF->InitProgress(_T("Writing Config Memory"));

	PICOpen(port, device->devVppMask, false);

	if (!retryOpen(device, port))
	{
		// Issue a warning that we aren't getting anything from the chip
	}

	// Write ID locations
	// Setup for write to config memory (single panel writes)
	i = setupWriteCode(device, port, hexFile);
	if (i) { PICClose(port); return 1; }

	// Move to the location of ID memory and write it
	i  = SetTablePointer(port, 2 * device->userIDWordsAddr);
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_PBLWRT_POSTINC | hexFile->userIDWords[0], &j); // bsf EECON1, EEPGD
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_PBLWRT_POSTINC | hexFile->userIDWords[1], &j); // bsf EECON1, EEPGD
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_PBLWRT_POSTINC | hexFile->userIDWords[2], &j); // bsf EECON1, EEPGD
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_STARTPROG | hexFile->userIDWords[3], &j); // bsf EECON1, EEPGD
	i += TxCmd(port, EVA_PIC16W_CMD, PIC16_CORE, &j);            
	if (i)
	{
		PICClose(port);
		return 1;
	}

	// Write config bytes
	// Direct access to config memory
	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x8EA6, &j); // bsf EECON1, EEPGD
	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x8CA6, &j); // bsf EECON1, CFGS
	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0xef00, &j); // goto 100000h
	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0xf800, &j);

	// Move to the beginning of code memory
	i = SetTablePointer(port, 2*device->configWordsAddr);

	// Step through config values, writing one byte at a time
	for(addr=0;addr<device->configSize;addr++)
	{
		// Extract the individual bytes from the config word
		byte fuseH = (hexFile->configWords[addr] & 0xff00) >> 8;
		byte fuseL = (hexFile->configWords[addr] & 0xff);
		ushort fuse = hexFile->configWords[addr];

		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_STARTPROG | fuse, &j); // Load <LSB> and start programming
		i  = TxCmd(port, EVA_PIC16W_CMD, PIC16_CORE, &j);            
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x2af6, &j);    // incf TBLPTRL
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_STARTPROG | fuse, &j);
		i  = TxCmd(port, EVA_PIC16W_CMD, PIC16_CORE, &j);
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x2af6, &j);    // incf TBLPTRL

		if (statusIF && !statusIF->UpdateProgress(addr, device->configSize))
		{
			PICClose(port);
			return 1;
		}
	}

	PICClose(port);

	return 0;
}

int PIC18FXXXProgram::WriteUser(const PICDevice *device, CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	if (statusIF) statusIF->InitProgress(_T("Writing User IDs"));

	PICOpen(port, device->devVppMask, false);

	PICClose(port);

	return 0;
}

int PIC18FXXXProgram::Program(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	int res;

	const PICDevice *device = hexFile->Processor;

    if (!SUCCEEDED(ResetProgrammer(port)))
	{
		// Failed to get the programmer into a known good state.
		return 1;
	}

	// Send the contents of the hex file to the PIC
	if ((res = WriteCode(device, port, hexFile, statusIF)) != 0) return res;
	if ((res = WriteData(device, port, hexFile, statusIF)) != 0) return res;
	if ((res = WriteConfig(device, port, hexFile, statusIF)) != 0) return res;

	// Ensure everything shut down
	PICClose(port);

	return 0;
}

int PIC18FXXXProgram::PartialProgram(CIOPort *port, PICHexFile *hexFile, bool codeFlag, bool dataFlag, bool configFlag, CProgrammerProgressIF *statusIF) const
{
	int res;

	const PICDevice *device = hexFile->Processor;

    if (!SUCCEEDED(ResetProgrammer(port)))
	{
		// Failed to get the programmer into a known good state.
		return 1;
	}

	// Send the contents of the hex file to the PIC
	if (codeFlag && (res = WriteCode(device, port, hexFile, statusIF)) != 0) return res;
	if (dataFlag && (res = WriteData(device, port, hexFile, statusIF)) != 0) return res;
	if (configFlag && (res = WriteConfig(device, port, hexFile, statusIF)) != 0) return res;

	// Ensure everything shut down
	PICClose(port);

	return 0;
}


int PIC18FXXXProgram::EraseCmd(CIOPort *port, PICHexFile *hexFile, uint cmd) const
{
	int i;
	uint j;

	const PICDevice *device = hexFile->Processor;

    if (!SUCCEEDED(ResetProgrammer(port)))
		// Failed to get the programmer into a known good state.
		return 1;

	PICOpen(port, device->devVppMask, false);

	if (!retryOpen(device, port))
	{
		// Issue a warning that we aren't getting anything from the chip
		return 1;
	}

#if 1
	uint cmdH = (cmd & 0xFF00) | ((cmd & 0xFF00) >> 8);
	uint cmdL = (cmd & 0x00FF) | ((cmd & 0x00FF) << 8);
	i = SetTablePointer(port, 0x3c0005);
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_TBLWRT | cmdH, &j);
	i = SetTablePointer(port, 0x3c0004);
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_TBLWRT | cmdL, &j);
#else
	i = SetTablePointer(port, 0x3c0004);
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_TBLWRT | cmd, &j);
#endif
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0000, &j);
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0000, &j);
	::Sleep(5);

	PICClose(port);
	return i;
}

int PIC18FXXXProgram::EraseBoot(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	return EraseCmd(port, hexFile, 0x0083); // Erase boot block
}

int PIC18FXXXProgram::EraseNonBoot(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	int i;
	i  = EraseCmd(port, hexFile, 0x0088); // Erase panel 1
	i += EraseCmd(port, hexFile, 0x008A); // Erase panel 2
	i += EraseCmd(port, hexFile, 0x008B); // Erase panel 3
	i += EraseCmd(port, hexFile, 0x008C); // Erase panel 4
	return i;
}

int PIC18FXXXProgram::EraseCode(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
#if 0
	int i;
	i  =  EraseCmd(port, hexFile, 0x0083); // Erase boot block
	i +=  EraseCmd(port, hexFile, 0x0088); // Erase panel 1
	i +=  EraseCmd(port, hexFile, 0x0089); // Erase panel 2
	i +=  EraseCmd(port, hexFile, 0x008A); // Erase panel 3
	i +=  EraseCmd(port, hexFile, 0x008B); // Erase panel 4
	i +=  EraseCmd(port, hexFile, 0x008C); // Erase panel 5
	i +=  EraseCmd(port, hexFile, 0x008D); // Erase panel 6
	i +=  EraseCmd(port, hexFile, 0x008E); // Erase panel 7
	i +=  EraseCmd(port, hexFile, 0x008F); // Erase panel 8
#else

#if 1
	uint i, j;

	const PICDevice *device = hexFile->Processor;
	PICOpen(port, device->devVppMask, false);

	if (statusIF) statusIF->InitProgress(_T("Erase code memory"));

	// Step through code, erasing one 64 byte block at a time
	uint addr, endAddr = 2 * device->programWordsSize;
	uint resLen, blockSize = 64;
	byte outBuf[9], inBuf[128];
	for(addr=0;addr<endAddr;addr+=blockSize)
	{
#if 1
		outBuf[0] = addr & 0xff;
		outBuf[1] = (addr & 0xff00) >> 8;
		outBuf[2] = (addr & 0xff0000) >> 8;
		outBuf[3] = blockSize;
		i = PICProgram::SendPacket(port, PICProgram::EVA_PIC16EBUF_CMD,
			4, outBuf, 16, &resLen, inBuf, 2*blockSize);
		if (i) { PICClose(port); return addr+1; }
#else
		// Move code pointer to address
		i = SetTablePointer(port, addr);

		// Enable memory writes and enable erase
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x84A6, &j); // bsf EECON1, WREN
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x88A6, &j); // bsf EECON1, FREE

		// Flash unlock sequence
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0e55, &j); //
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6ea7, &j); //
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0eaa, &j); //
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6ea7, &j); //

		// Initiate erase
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x82a6, &j); //
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0000, &j); // NOP
#endif
		if (statusIF && !statusIF->UpdateProgress(addr, endAddr))
		{
			PICClose(port);
			return 1;
		}
	}
#endif

	// Set config bytes to all 1's
	// Direct access to config memory
	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x8EA6, &j); // bsf EECON1, EEPGD
	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x8CA6, &j); // bsf EECON1, CFGS
	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0xef00, &j); // goto 100000h
	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0xf800, &j);

	// Move to the beginning of code memory
	i = SetTablePointer(port, 2*device->configWordsAddr);

	// Step through config values, writing one byte at a time
	for(addr=0;addr<device->configSize;addr++)
	{
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_STARTPROG | 0xffff, &j); // Load <LSB> and start programming
		i  = TxCmd(port, EVA_PIC16W_CMD, PIC16_CORE, &j);            
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x2af6, &j);    // incf TBLPTRL
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_STARTPROG | 0xffff, &j);
		i  = TxCmd(port, EVA_PIC16W_CMD, PIC16_CORE, &j);
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x2af6, &j);    // incf TBLPTRL
	}

	PICClose(port);
#endif

	return i;
}

int PIC18FXXXProgram::EraseData(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
#if 0
	return EraseCmd(port, hexFile, 0x0081); // Bulk erase EEPROM
#else
	const PICDevice *device = hexFile->Processor;

	// Erase the Data EEPROM by writing 0xFF into every location (the data
	// EEPROM automatically erases a location prior to a write).
	uint addr, i;
	if (statusIF) statusIF->InitProgress(_T("Erasing Data EEPROM"));

	// Open programmer
	PICOpen(port, device->devVppMask, false);

	if (!retryOpen(device, port))
	{
		// Issue a warning that we aren't getting anything from the chip
	}

	uint j;
	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x9EA6, &j); // bsf EECON1, EEPGD
	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x9CA6, &j); // bsf EECON1, CFGS

	// Write data memory in CMD_BLOCKSIZE byte chunks.
	byte outBuf[128], inBuf[128];
	uint resLen;
	uint blockSize = CMD_BLOCKSIZE;
	byte *valBuf = &outBuf[4]; // Location of first value in response buffer
	for (uint i0=0;i0<blockSize;i0++)
		*valBuf++ = 0xff;
	for (addr=0;addr<device->EEPROMDataSize;addr+=blockSize)
	{
		uint byteAddr = addr;
		outBuf[0] = byteAddr & 0xff;
		outBuf[1] = (byteAddr & 0xff00) >> 8;
		outBuf[2] = (byteAddr & 0xff0000) >> 16;
		outBuf[3] = CMD_BLOCKSIZE; // # of words per pass within the block write routine (adjust by processor)

		i = PICProgram::SendPacket(port, PICProgram::EVA_PIC16WBE_CMD,
			blockSize+4, outBuf, 128, &resLen, inBuf, 8*blockSize);
		if (i) { PICClose(port); return addr+1; }

		if (statusIF && !statusIF->UpdateProgress(addr, device->EEPROMDataSize))
		{
			PICClose(port);
			return 1;
		}
	}

	// Shut down connection to programmer
	PICClose(port);

	return 0;
#endif
}

int PIC18FXXXProgram::Erase(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	int res = EraseCmd(port, hexFile, 0x0080); // Erase entire chip

	return res;
}

int PIC18FXXXProgram::PartialErase(CIOPort *port, PICHexFile *hexFile, bool bootFlag, bool codeFlag, bool dataFlag, bool configFlag, CProgrammerProgressIF *statusIF) const
{
	int i = 0;

	if (bootFlag)
		i = EraseBoot(port, hexFile, statusIF);
	if (codeFlag)
		i += EraseCode(port, hexFile, statusIF);
	if (dataFlag)
		i += EraseData(port, hexFile, statusIF);

	return i;
}

int PIC18FXXXProgram::Verify(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	return 0;
}
