#pragma once
#include "PIC/PICProgram.h"

class PIC18FXXXProgram :
	public PICProgram
{
public:
	PIC18FXXXProgram(void);
	~PIC18FXXXProgram(void);

public:
	// Get the singleton instance of this class of PIC programming code
	static const PICProgram *PIC18FXXXProgram::GetDriver();

	int Read(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF = NULL) const;
	int PartialRead(CIOPort *port, PICHexFile *hexFile, bool codeFlag, bool dataFlag, bool configFlag, CProgrammerProgressIF *statusIF) const;
	int Program(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF = NULL) const;
	int PartialProgram(CIOPort *port, PICHexFile *hexFile, bool codeFlag = false, bool dataFlag = false, bool configFlag = false, CProgrammerProgressIF *statusIF = NULL) const;
	virtual int Erase(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF = NULL) const;
	int PartialErase(CIOPort *port, PICHexFile *hexFile, bool bootFlag = false, bool codeFlag = false, bool dataFlag = false, bool configFlag = false, CProgrammerProgressIF *statusIF = NULL) const;
	int Verify(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF = NULL) const;

	bool retryOpen(const PICDevice *device, CIOPort *port) const;

protected:
	static PIC18FXXXProgram m_Driver;

	int getWriteBufferSize(const PICDevice *device) const;
	int getEraseBufferSize(const PICDevice *device) const;

	virtual int ReadCode(const PICDevice *device, CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF = NULL) const;
	virtual int ReadConfig(const PICDevice *device, CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF = NULL) const;
	virtual int ReadDataByte(const PICDevice *device, CIOPort *port, uint addr, byte *val) const;
	virtual int ReadData(const PICDevice *device, CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF = NULL) const;

	int setupWriteCode(const PICDevice *device, CIOPort *port, PICHexFile *hexFile) const;
	virtual int WriteCode(const PICDevice *device, CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF = NULL) const;
	virtual int WriteData(const PICDevice *device, CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF = NULL) const;
	virtual int WriteConfig(const PICDevice *device, CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF = NULL) const;
	virtual int WriteUser(const PICDevice *device, CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF = NULL) const;

	virtual int EraseCmd(CIOPort *port, PICHexFile *hexFile, uint cmd) const;
	virtual int EraseBoot(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF = NULL) const;
	virtual int EraseNonBoot(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF = NULL) const;
	virtual int EraseCode(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF = NULL) const;
	virtual int EraseData(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF = NULL) const;
};
