#include "StdAfx.h"
#include "PICDevice.h"

	// Start addresss of EEPROM data memory
const uint PICDevice::PIC14_EEPROM_ADDRESS = 0x2100;
const uint PICDevice::PIC16_EEPROM_ADDRESS = 0x780000;
const uint PICDevice::PIC24_EEPROM_ADDRESS = 0x7ffc00;
	// Start addresss of configuration memory
const uint PICDevice::PIC14_CONFIG_ADDRESS = 0x2000;
const uint PICDevice::PIC16_CONFIG_ADDRESS = 0x180000;
const uint PICDevice::PIC24_CONFIG_ADDRESS = 0xf80000;
	// Start address of fuses
const uint PICDevice::PIC14_FUSE_ADDRESS = 0x02007;
const uint PICDevice::PIC16_FUSE_ADDRESS = 0x180000;
const uint PICDevice::PIC24_FUSE_ADDRESS = 0xf80000;
	// Start address of user ID words
const uint PICDevice::PIC14_USER_ADDRESS = 0x2000;
const uint PICDevice::PIC16_USER_ADDRESS = 0x100000;
const uint PICDevice::PIC24_USER_ADDRESS = 0x800000; // (Executive code)
	// Start address of device ID words
const uint PICDevice::PIC14_DEVID_ADDRESS = 0x2006;
const uint PICDevice::PIC16_DEVID_ADDRESS = 0x1FFFFF;
const uint PICDevice::PIC24_DEVID_ADDRESS = 0xff0000;

#define NAME_COPY 0

#if 1
PICDevice::PICDevice(LPCTSTR pNames[4], uint pwordSize, uint psizeAddr,
			uint psizeData,
			uint pconfigSize,
			uint pconfigWordsSize,
			uint puserIDSize,
			uint psizeDevID,
			uint pdevIDMask,
			unsigned char pdevVppMask)
#else
PICDevice::PICDevice(LPCTSTR pNames[4], uint pwordSize, uint psizeAddr,
			uint pdataAddr, uint psizeData,
			uint pconfigAddr, uint pconfigSize,
			uint pconfigWordsAddr, uint pconfigWordsSize,
			uint puserIDAddr, uint puserIDSize,
			uint pdevIDAddr, uint psizeDevID,
			uint pdevIDMask,
			unsigned char pdevVppMask)
#endif
{
#if NAME_COPY
	int nameCnt = sizeof(name) / sizeof(name[0]);
	for (int i=0;i<nameCnt;i++)
	{
		name[i] = new TCHAR[_tcslen(pNames[i]) + 1];
		_tcscpy(name[i], pNames[i]);
	}
#else
	name[0]          = pNames[0];
	name[1]          = pNames[1];
	name[2]          = pNames[2];
	name[3]          = pNames[3];
#endif
	wordSize         = pwordSize;
	programWordsSize = psizeAddr;
	EEPROMDataSize   = psizeData;
	configSize       = pconfigSize;
	configWordsSize  = pconfigWordsSize;
	deviceIDSize     = psizeDevID;
	deviceIDMask     = pdevIDMask;
	userIDWordsSize  = puserIDSize;

	devVppMask       = pdevVppMask;

#if 1
	switch (wordSize)
	{
	case 14:
		configAddr = PICDevice::PIC14_CONFIG_ADDRESS;
		configWordsAddr = PICDevice::PIC14_FUSE_ADDRESS;
		EEPROMDataAddr = PICDevice::PIC14_EEPROM_ADDRESS;
		userIDWordsAddr = PICDevice::PIC14_USER_ADDRESS;
		deviceIDAddr = PICDevice::PIC14_DEVID_ADDRESS;
		break;
	case 16:
		configAddr = PICDevice::PIC16_CONFIG_ADDRESS;
		configWordsAddr = PICDevice::PIC16_FUSE_ADDRESS;
		EEPROMDataAddr = PICDevice::PIC16_EEPROM_ADDRESS;
		userIDWordsAddr = PICDevice::PIC16_USER_ADDRESS;
		deviceIDAddr = PICDevice::PIC16_DEVID_ADDRESS;
		break;
	case 24:
		configAddr = PICDevice::PIC24_CONFIG_ADDRESS;
		configWordsAddr = PICDevice::PIC24_FUSE_ADDRESS;
		EEPROMDataAddr = PICDevice::PIC24_EEPROM_ADDRESS;
		userIDWordsAddr = PICDevice::PIC24_USER_ADDRESS;
		deviceIDAddr = PICDevice::PIC24_DEVID_ADDRESS;
		break;
	default:
		configAddr = PICDevice::PIC14_CONFIG_ADDRESS;
		configWordsAddr = PICDevice::PIC14_FUSE_ADDRESS;
		EEPROMDataAddr = PICDevice::PIC14_EEPROM_ADDRESS;
		userIDWordsAddr = PICDevice::PIC14_USER_ADDRESS;
		deviceIDAddr = PICDevice::PIC14_DEVID_ADDRESS;
		break;
	}
#else
	EEPROMDataAddr   = pdataAddr;
	configAddr       = pconfigAddr;
	configWordsAddr  = pconfigWordsAddr;
	deviceIDAddr     = pdevIDAddr;
	userIDWordsAddr  = puserIDAddr;
#endif
}

PICDevice::~PICDevice(void)
{
#if NAME_COPY
	// Free memory associated with array of processor name aliases
	for (int i=0;i<sizeof(name)/sizeof(name[0]);i++)
	{
		delete []name[i];
	}
#endif
}
