#pragma once

#if !defined(uint)
#define uint unsigned int
#endif

#if !defined(byte)
#define byte unsigned char
#endif

class PICDevice
{
public:
#if 1
	PICDevice(LPCTSTR *pNames, uint pwordSize, uint psizeAddr,
			uint psizeData,
			uint pconfigSize,
			uint pconfigWordsSize,
			uint puserIDSize,
			uint psizeDevID,
			uint pdevIDMask,
			unsigned char pdevVppMask = 0x03);
#else
	PICDevice(LPCTSTR *pNames, uint pwordSize, uint psizeAddr,
			uint pdataAddr, uint psizeData,
			uint pconfigAddr, uint pconfigSize,
			uint pconfigWordsAddr, uint pconfigWordsSize,
			uint puserIDAddr, uint puserIDSize,
			uint pdevIDAddr, uint psizeDevID,
			uint pdevIDMask,
			unsigned char pdevVppMask = 0x03);
#endif
	~PICDevice(void);

	LPCTSTR name[4];          // Array of aliases for the processor name

	uint wordSize;            // Number of bits in a program word (14 or 16)

	uint programWordsSize;    // Number of words (not bytes) in program memory

	uint configAddr;          // First address of entire configuration memory (user id, device id, configuration word)
	uint configSize;          // Number of words in entire configuration memory

	uint configWordsAddr;     // First address of configuration word(s)
	uint configWordsSize;     // Number of config words

	// Note that the userIDWords values are used by dsPIC processors to
	// hold the address of the program executive memory space.
	uint userIDWordsAddr;     // First address of User ID memory
	uint userIDWordsSize;     // Number of words of User ID memory

	uint deviceIDAddr;        // First address of words in device ID memory
	uint deviceIDSize;        // Max number of words in device ID memory
	uint deviceIDMask;        // Bit pattern for device specific ID

	uint EEPROMDataAddr;	  // First address of a byte intended for EEPROM memory
	uint EEPROMDataSize;	  // Maximum # of bytes in EEPROM data memory

	unsigned char devVppMask; // Which Vpp line(s) are used to program this chip?
	// Note: ICSP uses Vpp2, so don't turn it off unless you don't need ICSP.

	// Start addresss of EEPROM data memory
	static const uint PIC14_EEPROM_ADDRESS; //       = 0x2100;
	static const uint PIC16_EEPROM_ADDRESS; //       = 0x780000;
	static const uint PIC24_EEPROM_ADDRESS; //       = 0x7ffc00;
	// Start addresss of configuration memory
	static const uint PIC14_CONFIG_ADDRESS; //       = 0x2000;
	static const uint PIC16_CONFIG_ADDRESS; //       = 0x180000;
	static const uint PIC24_CONFIG_ADDRESS; //       = 0xf80000;
	// Start address of fuses
	static const uint PIC14_FUSE_ADDRESS;   //       = 0x02007;
	static const uint PIC16_FUSE_ADDRESS;   //       = 0x180000;
	static const uint PIC24_FUSE_ADDRESS;   //       = 0xf80000;
	// Start address of user ID words
	static const uint PIC14_USER_ADDRESS;   //       = 0x2000;
	static const uint PIC16_USER_ADDRESS;   //       = 0x100000;
	static const uint PIC24_USER_ADDRESS;   //       = 0x800000; // (Executive code)
	// Start address of device ID words
	static const uint PIC14_DEVID_ADDRESS;  //       = 0x2006;
	static const uint PIC16_DEVID_ADDRESS;  //       = 0x1FFFFF;
	static const uint PIC24_DEVID_ADDRESS;  //       = 0xff0000;

};
