#include "StdAfx.h"
#include <assert.h>
#include "PICHexFile.h"
#include "HexFileIO.h"

// 14 bit PIC Instructions
instructionSetEntry PICHexFile::PIC14Instructions[] = {
	// Byte-Oriented File Register Operations
  { _T("addwf"),  byteFileOp,  0x3f00, 0x0700}, // 00 0111 dfff ffff
  { _T("andwf"),  byteFileOp,  0x3f00, 0x0500}, // 00 0101 dfff ffff
  { _T("clrf"),   byteFileOpA, 0x3f80, 0x0180}, // 00 0001 1fff ffff
  { _T("clrw"),   noArgOp,     0x3f80, 0x0100}, // 00 0001 0xxx xxxx
  { _T("comf"),   byteFileOp,  0x3f00, 0x0900}, // 00 1001 dfff ffff
  { _T("decf"),   byteFileOp,  0x3f00, 0x0300}, // 00 0011 dfff ffff
  { _T("decfsz"), byteFileOp,  0x3f00, 0x0b00}, // 00 1011 dfff ffff
  { _T("incf"),   byteFileOp,  0x3f00, 0x0a00}, // 00 1010 dfff ffff
  { _T("incfsz"), byteFileOp,  0x3f00, 0x0f00}, // 00 1111 dfff ffff
  { _T("iorwf"),  byteFileOp,  0x3f00, 0x0400}, // 00 0100 dfff ffff
  { _T("movf"),   byteFileOp,  0x3f00, 0x0800}, // 00 1000 dfff ffff
  { _T("movwf"),  byteFileOpA, 0x3f80, 0x0080}, // 00 0000 1fff ffff
  { _T("nop"),    noArgOp,     0x3f9f, 0x0000}, // 00 0000 0xx0 0000
  { _T("rlf"),    byteFileOp,  0x3f00, 0x0d00}, // 00 1101 dfff ffff
  { _T("rrf"),    byteFileOp,  0x3f00, 0x0c00}, // 00 1100 dfff ffff
  { _T("subwf"),  byteFileOp,  0x3f00, 0x0200}, // 00 0010 dfff ffff
  { _T("swapf"),  byteFileOp,  0x3f00, 0x0e00}, // 00 1110 dfff ffff
  { _T("xorwf"),  byteFileOp,  0x3f00, 0x0600}, // 00 0110 dfff ffff

	// Bit-Oriented File Register Operations
  { _T("bcf"),    bitFileOp,   0x3c00, 0x1000}, // 01 00bb bfff ffff
  { _T("bsf"),    bitFileOp,   0x3c00, 0x1400}, // 01 01bb bfff ffff
  { _T("btfsc"),  bitFileOp,   0x3c00, 0x1800}, // 01 10bb bfff ffff
  { _T("btfss"),  bitFileOp,   0x3c00, 0x1c00}, // 01 11bb bfff ffff

  // Literal and Control Operations
  { _T("addlw"),  literalOp,   0x3e00, 0x3e00}, // 11 111x kkkk kkkk
  { _T("andlw"),  literalOp,   0x3f00, 0x3900}, // 11 1001 kkkk kkkk
  { _T("call"),   controlOp11, 0x3800, 0x2000}, // 10 0kkk kkkk kkkk
  { _T("clrwdt"), noArgOp ,    0x3fff, 0x0064}, // 00 0000 0110 0100
  { _T("goto"),   controlOp11, 0x3800, 0x2800}, // 10 1kkk kkkk kkkk
  { _T("iorlw"),  literalOp,   0x3f00, 0x3800}, // 11 1000 kkkk kkkk
  { _T("movlw"),  literalOp,   0x3c00, 0x3000}, // 11 00xx kkkk kkkk
  { _T("option"), noArgOp,     0x3fff, 0x0062}, // 
  { _T("retfie"), noArgOp,     0x3fff, 0x0009}, // 00 0000 0000 1001
  { _T("retlw"),  literalOp,   0x3c00, 0x3400}, // 11 01xx kkkk kkkk
  { _T("return"), noArgOp,     0x3fff, 0x0008}, // 00 0000 0000 1000
  { _T("sleep"),  noArgOp,     0x3fff, 0x0063}, // 00 0000 0110 0011
  { _T("sublw"),  literalOp,   0x3e00, 0x3c00}, // 11 110x kkkk kkkk
  { _T("tris"),   byteFileOpA, 0x3ff8, 0x0060}, // 
  { _T("xorlw"),  literalOp,   0x3f00, 0x3a00}, // 11 1010 kkkk kkkk
};

instructionSetEntry PICHexFile::PIC16Instructions[] = {
	// Byte-Oriented File Register Operations
  { _T("addwf"),  byteFileOp,  0xfc00, 0x2400}, // 0010 01da ffff ffff
  { _T("addwfc"), byteFileOp,  0xfc00, 0x2000}, // 0010 00da ffff ffff
  { _T("andwf"),  byteFileOp,  0xfc00, 0x1400}, // 0010 00da ffff ffff
  { _T("clrf"),   byteFileOpA, 0xfe00, 0x6a00}, // 0110 101a ffff ffff
  { _T("comf"),   byteFileOp,  0xfc00, 0x1c00}, // 0001 11da ffff ffff
  { _T("cpfseq"), byteFileOpA, 0xfe00, 0x6200}, // 0110 001a ffff ffff
  { _T("cpfsgt"), byteFileOpA, 0xfe00, 0x6400}, // 0110 010a ffff ffff
  { _T("cpfslt"), byteFileOpA, 0xfe00, 0x6000}, // 0110 000a ffff ffff
  { _T("decf"),   byteFileOp,  0xfc00, 0x0400}, // 0000 01da ffff ffff
  { _T("decfsz"), byteFileOp,  0xfc00, 0x2c00}, // 0010 11da ffff ffff
  { _T("dcfsnz"), byteFileOp,  0xfc00, 0x4c00}, // 0100 11da ffff ffff
  { _T("incf"),   byteFileOp,  0xfc00, 0x2800}, // 0010 10da ffff ffff
  { _T("incfsz"), byteFileOp,  0xfc00, 0x3c00}, // 0011 11da ffff ffff
  { _T("infsnz"), byteFileOp,  0xfc00, 0x4800}, // 0100 10da ffff ffff
  { _T("iorwf"),  byteFileOp,  0xfc00, 0x1000}, // 0001 00da ffff ffff
  { _T("movf"),   byteFileOp,  0xfc00, 0x5000}, // 0101 00da ffff ffff
  { _T("movff"),  byteToByte,  0xf000, 0xc000}, // 1100 ffff ffff ffff + 1111 ffff ffff ffff
  { _T("movwf"),  byteFileOpA, 0xfe00, 0x6e00}, // 0110 111a ffff ffff
  { _T("mulwf"),  byteFileOpA, 0xfe00, 0x0200}, // 0000 001a ffff ffff
  { _T("negf"),   byteFileOpA, 0xfe00, 0x6c00}, // 0110 110a ffff ffff
  { _T("rlcf"),   byteFileOp,  0xfc00, 0x3400}, // 0011 01da ffff ffff
  { _T("rlncf"),  byteFileOp,  0xfc00, 0x4400}, // 0100 01da ffff ffff
  { _T("rrcf"),   byteFileOp,  0xfc00, 0x3000}, // 0011 00da ffff ffff
  { _T("rrncf"),  byteFileOp,  0xfc00, 0x4000}, // 0100 00da ffff ffff
  { _T("setf"),   byteFileOpA, 0xfe00, 0x6800}, // 0110 100a ffff ffff
  { _T("subfwb"), byteFileOp,  0xfc00, 0x5400}, // 0101 01da ffff ffff
  { _T("subwf"),  byteFileOp,  0xfc00, 0x5c00}, // 0101 11da ffff ffff
  { _T("subwfb"), byteFileOp,  0xfc00, 0x5800}, // 0101 10da ffff ffff
  { _T("swapf"),  byteFileOp,  0xfc00, 0x3800}, // 0011 10da ffff ffff
  { _T("tstfsz"), byteFileOpA, 0xfe00, 0x6600}, // 0110 011a ffff ffff
  { _T("xorwf"),  byteFileOp,  0xfc00, 0x1800}, // 0001 10da ffff ffff

	// Bit-Oriented File Register Operations
  { _T("bcf"),    bitFileOp,   0xf000, 0x9000}, // 1001 bbba ffff ffff
  { _T("bsf"),    bitFileOp,   0xf000, 0x8000}, // 1000 bbba ffff ffff
  { _T("btfsc"),  bitFileOp,   0xf000, 0xb000}, // 1011 bbba ffff ffff
  { _T("btfss"),  bitFileOp,   0xf000, 0xa000}, // 1010 bbba ffff ffff
  { _T("btg"),    bitFileOp,   0xf000, 0x7000}, // 0111 bbba ffff ffff

    // Control Operations
  { _T("bc"),     controlOp8,  0xff00, 0xe200}, // 1110 0010 nnnn nnnn
  { _T("bn"),     controlOp8,  0xff00, 0xe600}, // 1110 0110 nnnn nnnn
  { _T("bnc"),    controlOp8,  0xff00, 0xe300}, // 1110 0011 nnnn nnnn
  { _T("bnn"),    controlOp8,  0xff00, 0xe700}, // 1110 0111 nnnn nnnn
  { _T("bnov"),   controlOp8,  0xff00, 0xe500}, // 1110 0101 nnnn nnnn
  { _T("bnz"),    controlOp8,  0xff00, 0xe100}, // 1110 0001 nnnn nnnn
  { _T("bov"),    controlOp8,  0xff00, 0xe400}, // 1110 0100 nnnn nnnn
  { _T("bra"),    controlOp11, 0xf800, 0xd000}, // 1101 0nnn nnnn nnnn
  { _T("bz"),     controlOp8,  0xff00, 0xe000}, // 1110 0000 nnnn nnnn
  { _T("call"),   controlOp20S,0xfe00, 0xec00}, // 1110 110s kkkk kkkk + 1111 kkkk kkkk kkkk
  { _T("clrwdt"), noArgOp,     0xffff, 0x0004}, // 0000 0000 0000 0100
  { _T("daw"),    noArgOp,     0xffff, 0x0007}, // 0000 0000 0000 0111
  { _T("goto"),   controlOp20, 0xff00, 0xef00}, // 1110 1111 kkkk kkkk + 1111 kkkk kkkk kkkk
  { _T("nop"),    noArgOp,     0xffff, 0x0000}, // 0000 0000 0000 0000
  { _T("nop"),    noArgOp,     0xf000, 0xf000}, // 1111 xxxx xxxx xxxx
  { _T("pop"),    noArgOp,     0xffff, 0x0006}, // 0000 0000 0000 0110
  { _T("push"),   noArgOp,     0xffff, 0x0005}, // 0000 0000 0000 0101
  { _T("rcall"),  controlOp11, 0xf800, 0xd800}, // 1101 1nnn nnnn nnnn
  { _T("reset"),  noArgOp,     0xffff, 0x00ff}, // 0000 0000 1111 1111
  { _T("retfie"), controlOpS,  0xfffe, 0x0010}, // 0000 0000 0001 000s
  { _T("retlw"),  literalOp,   0xff00, 0x0c00}, // 0000 1100 kkkk kkkk
  { _T("return"), controlOpS,  0xfffe, 0x0012}, // 0000 0000 0001 001s
  { _T("sleep"),  noArgOp,     0xffff, 0x0003}, // 0000 0000 0000 0011

	// Literal Operations
  { _T("addlw"),  literalOp,   0xff00, 0x0f00}, // 0000 1111 kkkk kkkk
  { _T("andlw"),  literalOp,   0xff00, 0x0b00}, // 0000 1011 kkkk kkkk
  { _T("iorlw"),  literalOp,   0xff00, 0x0900}, // 0000 1011 kkkk kkkk
  { _T("lfsr"),   literalOp12, 0xffc0, 0xee00}, // 1110 1110 00ff kkkk + 1111 0000 kkkk kkkk
  { _T("movlb"),  literalOp4,  0xff00, 0x0100}, // 0000 0001 0000 kkkk
  { _T("movlw"),  literalOp,   0xff00, 0x0e00}, // 0000 1110 kkkk kkkk
  { _T("mullw"),  literalOp,   0xff00, 0x0d00}, // 0000 1101 kkkk kkkk
  { _T("sublw"),  literalOp,   0xff00, 0x0800}, // 0000 1000 kkkk kkkk
  { _T("xorlw"),  literalOp,   0xff00, 0x0a00}, // 0000 1010 kkkk kkkk

	// Data Memory <-> Memory Operations
  { _T("tblrd*"), noArgOp,     0xfffc, 0x0008}, // 0000 0000 0000 1000
  { _T("tblrd*+"),noArgOp,     0xfffc, 0x0009}, // 0000 0000 0000 1001
  { _T("tblrd*-"),noArgOp,     0xfffc, 0x000a}, // 0000 0000 0000 1010
  { _T("tblrd+*"),noArgOp,     0xfffc, 0x000b}, // 0000 0000 0000 1011
  { _T("tblwt*"), noArgOp,     0xfffc, 0x000c}, // 0000 0000 0000 1100
  { _T("tblwt*+"),noArgOp,     0xfffc, 0x000d}, // 0000 0000 0000 1101
  { _T("tblwt*-"),noArgOp,     0xfffc, 0x000e}, // 0000 0000 0000 1110
  { _T("tblwt+*"),noArgOp,     0xfffc, 0x000f}, // 0000 0000 0000 1111
};

// Find all addresses that are targets of call/goto
void PICHexFile::findCallTargets(uint startAddr, uint endAddr, callTargetArray &callTargets)
{
	uint addr, f;
	int i;
	struct insn *instruction;
	instructionSetEntry *opCode;
	int instructionLength;

	callTargets.clear();
	int numOpCodes14 = sizeof(PIC14Instructions) / sizeof(PIC14Instructions[0]);
	int numOpCodes16 = sizeof(PIC16Instructions) / sizeof(PIC16Instructions[0]);

	for (addr=startAddr;addr<endAddr;addr++)
	{
		instruction = NULL;
		opCode = NULL;
		ushort memWord = programWords[addr];

		if (Processor->wordSize == 14)
		{
			for(i = 0; i < numOpCodes14; i++)
			{
				if((memWord & PIC14Instructions[i].opCodeMask) == PIC14Instructions[i].opCode)
				{
					opCode = &PIC14Instructions[i];
					break;
				}
			}
			if (opCode == NULL)
				continue;
			if (opCode->opCodeType == controlOp11)
			{
				// Found a call/goto.
				uint f = (memWord & 0x7ff);
				callTargets.insert(callTargetEntry(f, 0));
			}
		}
		else if (Processor->wordSize == 16)
		{
			// 16 bit PIC
			for(i = 0; i < numOpCodes16; i++)
			{
				if((memWord & PIC16Instructions[i].opCodeMask) == PIC16Instructions[i].opCode)
				{
					opCode = &PIC16Instructions[i];
					break;
				}
			}

			if (opCode == NULL)
				continue;

			switch (opCode->opCodeType)
			{
			case byteToByte:
				// Two word instruction - bump address forward
				addr++;
				instructionLength = 2;
				break;
			case controlOp20:
				f = (memWord & 0xff) | ((programWords[addr+1] & 0xfff) << 8);
				callTargets.insert(callTargetEntry(f, 0));
				addr++;
				break;
			case controlOp20S:
				f = (memWord & 0xff) | ((programWords[addr+1] & 0xfff) << 8);
				callTargets.insert(callTargetEntry(f, 0));
				addr++;
				break;
			case controlOp11:
				//The 2's complement number 2N is added to PC.
				f = (memWord & 0x7ff);
				// If f is negative, then complement and add 1
				if (f & 0x400)
					f = ~(f & 0x400);
				f = 2 * (addr + f + 1);
				callTargets.insert(callTargetEntry(f, 0));
				break;
			case controlOp8:
				//The 2's complement number 2N is added to PC.
				f = 2 * (addr + (char)(memWord & 0xff) + 1);
				callTargets.insert(callTargetEntry(f, 0));
				break;
			}
		}
	}

	callTargetArray::iterator aItr;
	uint index=0;
	for (aItr=callTargets.begin();aItr!=callTargets.end();aItr++)
	{
		aItr->second = index++;
	}
	for (aItr=callTargets.begin();aItr!=callTargets.end();aItr++)
	{
		uint val1 = aItr->second;
		uint val0 = aItr->first;
	}
}

int PICHexFile::emitOpcode(uint addr, ushort memWord, TCHAR *buf, callTargetArray &callTargets)
{
	int i;
	struct insn *instruction = NULL;
	instructionSetEntry *opCode = NULL;
	int instructionLength = 1;
	callTargetArray::iterator aItr;

	if (Processor->wordSize == 14)
	{
		// 14 bit PIC
		int numOpCodes = sizeof(PIC14Instructions) / sizeof(PIC14Instructions[0]);
		for(i = 0; i < numOpCodes; i++) {
			if((memWord & PIC14Instructions[i].opCodeMask) == PIC14Instructions[i].opCode)
			{
				opCode = &PIC14Instructions[i];
				break;
			}
		}

		if (opCode == NULL)
		{
			_stprintf(buf, _T("dw %#lx  ; unknown instruction"), memWord);
			return instructionLength;
		}

		uint b, d, f, k; // Argument names used in datasheets
		switch (opCode->opCodeType)
		{
			case noArgOp:      // [opcode<13:0>]
				_stprintf(buf, _T("%s"),  opCode->name);
				break;
			case byteFileOp:   // [opcode<13:8>|d<7>|f<6:0>]
				f = (memWord & 0x7f);
				d = (memWord & 0x80) >> 7;
				_stprintf(buf, _T("%s %#x, %#x"),  opCode->name, f, d);
			case byteFileOpA:  // [opcode<13:7>|f<6:0>]
				f = (memWord & 0x7f);
				_stprintf(buf, _T("%s %#x"),  opCode->name, f);
				break;
			case bitFileOp:    // [opcode<13:10>|b<9:7>|f<6:0>]
				f = (memWord & 0x7f);
				b = (memWord & 0x380) >> 7;
				_stprintf(buf, _T("%s %#x, %#x"),  opCode->name, f, b);
				break;
			case literalOp:    // [opcode<13:8>|k<7:0>]
				k = (memWord & 0xff);
				_stprintf(buf, _T("%s %#x"),  opCode->name, k);
				break;
			case controlOp11:  // [opcode<13:11>|n<10:0>]
				k = (memWord & 0x7ff);
				aItr = callTargets.find(k);
				if (aItr != callTargets.end())
					_stprintf(buf, _T("%s label%d"),  opCode->name, aItr->second);
				else
					// The target of any call/goto should always be on the
					// list.  We shouldn't get here.
					_stprintf(buf, _T("%s %#x"),  opCode->name, k);
				break;
			default:
				// TBD: Unknown opcode (which shouldn't have happened if we match
				// against the known opcode table above).
				assert(false);
		}
	}
	else if (Processor->wordSize == 16)
	{
		// 16 bit PIC
		int numOpCodes = sizeof(PIC16Instructions) / sizeof(PIC16Instructions[0]);
		for(i = 0; i < numOpCodes; i++) {
			if((memWord & PIC16Instructions[i].opCodeMask) == PIC16Instructions[i].opCode)
			{
				opCode = &PIC16Instructions[i];
				break;
			}
		}

		if (opCode == NULL)
		{
			if (((memWord >= -1) && (memWord < 256)) && isprint(memWord))
			{
				_stprintf(buf, _T("dw %#lx  ; '%c'"), memWord, (char)memWord);
			}
			else
			{
				_stprintf(buf, _T("dw %#lx  ; unknown instruction"), memWord);
			}
			return instructionLength;
		}

		uint a, b, d, f, f1, k, s; // Argument names used in datasheets
		int n;
		switch (opCode->opCodeType)
		{
			case noArgOp:      // [opcode<15:0>]
				_stprintf(buf, _T("%s"),  opCode->name);
				break;
			case byteFileOp:   // [opcode<15:10>|d<9>|a<8>|f<7:0>]
				f = (memWord & 0xff);
				d = (memWord & 0x200) >> 9;
				a = (memWord & 0x100) >> 8;
				if (a == 0)
					_stprintf(buf, _T("%s %#x, %#x, %#x"),  opCode->name, f, d, a);
				else
					_stprintf(buf, _T("%s %#x, %#x"),  opCode->name, f, d);
			case byteFileOpA:  // [opcode<15:9>|a<8>|f<7:0>]
				f = (memWord & 0xff);
				a = (memWord & 0x100) >> 8;
				if (a == 0)
					_stprintf(buf, _T("%s %#x, %#x"),  opCode->name, f, a);
				else
					_stprintf(buf, _T("%s %#x, %#x"),  opCode->name, f, a);
				break;
			case byteToByte:   // [opcode<15:12>|f<11:0>]+[0xF<15:12>|f<11:0>]
				f = (memWord & 0xfff);
				f1 = (programWords[addr+1] & 0xfff);
				_stprintf(buf, _T("%s %#x, %#x"),  opCode->name, f, f1);
				instructionLength = 2;
				break;
			case bitFileOp:    // [opcode<15:12>|b<11:9>|a<8>|f<7:0>]
				f = (memWord & 0xff);
				b = (memWord & 0xe00) >> 9;
				a = (memWord & 0x100) >> 8;
				if (a == 0)
					_stprintf(buf, _T("%s %#x, %#0x"),  opCode->name, f, b);
				else
					_stprintf(buf, _T("%s %#x, %#0x, 1"),  opCode->name, f, b);
				break;
			case literalOp:    // [opcode<15:8>|k<7:0>]
				k = (memWord & 0xff);
				_stprintf(buf, _T("%s %#x"),  opCode->name, k);
				break;
			case literalOp4:   // [opcode<15:4>|k<3:0>]
				k = (memWord & 0xf);
				_stprintf(buf, _T("%s %#x"),  opCode->name, k);
				break;
			case literalOp12:  // [opcode<15:6>|f<5:4>|k<3:0>]+[0xf0<15:8>|k<7:0>]
				k = (memWord & 0xfff);
				_stprintf(buf, _T("%s %#x"),  opCode->name, k);
				break;
			case controlOp20:  // [opcode<15:8>|n<7:0>]+[0xF|n<11:0>]
				f = (memWord & 0xff) | ((programWords[addr+1] & 0xfff) << 8);
				_stprintf(buf, _T("%s %#x"),  opCode->name, 2*f);
				instructionLength = 2;
				break;
			case controlOp20S: // [opcode<15:9>|S<8>|n<7:0>]+[unset?<15:12>|n<11:0>]
				f = (memWord & 0xff) | ((programWords[addr+1] & 0xfff) << 8);
				s = (memWord & 0x100) >> 8;
				if (s != 0)
					_stprintf(buf, _T("%s %#x, %#x"),  opCode->name, 2*f, s);
				else
					_stprintf(buf, _T("%s %#x"),  opCode->name, 2*f);
				instructionLength = 2;
				break;
			case controlOp11:  // [opcode<15:11>|n<10:0>]
				//The 2's complement number 2N is added to PC.
				n = (memWord & 0x7ff);
				// If n is negative, then complement and add 1
				if (n & 0x400)
					n = ~(n & 0x400);
				n = 2 * (addr + n + 1);
				_stprintf(buf, _T("%s %#x"),  opCode->name, n);
				break;
			case controlOp8:   // [opcode<15:8>|n<7:0>]
				//The 2's complement number 2N is added to PC.
				n = 2 * (addr + (char)(memWord & 0xff) + 1);
				_stprintf(buf, _T("%s %#x"),  opCode->name, n);
				break;
			case controlOpS:   // [opcode<15:1>|s<0>]
				s = (memWord & 0x1);
				if (s != 0)
					_stprintf(buf, _T("%s %#x"),  opCode->name, s);
				else
					_stprintf(buf, _T("%s"),  opCode->name);
				break;
			default:
				// TBD: Unknown opcode (which shouldn't have happened if we match
				// against the known opcode table above).
				assert(false);
		}

	}
	else
	{
		// Unknown/unsupported instruction length.  [If we got here, it's because
		// support was added for a new PIC without adding the corresponding
		// instruction set table above.]
		return 0;
	}

    return instructionLength;
}

PICHexFile::PICHexFile(const PICDevice *proc)
{
	fileName = NULL;
	// TBD: Get base addresses and limits from a configuration file or array.

	// Initialize storage for the hex file
	Processor = proc;

	configWords      = new ushort[proc->configWordsSize];
	programWords     = new ushort[proc->programWordsSize];
	programWordsUsed.resize((int)proc->programWordsSize);
	userIDWords      = new ushort[proc->userIDWordsSize];
	deviceID         = new ushort[proc->deviceIDSize];
	EEPROMData = new ushort[proc->EEPROMDataSize];

	ushort emptyVal = (proc->wordSize == 14 ? 0x3FFF : 0xFFFF);

	// Initialize all values to 0xFF
	for (uint configAddr=0;configAddr<proc->configWordsSize;configAddr++)
		configWords[configAddr] = emptyVal;
	for (uint programAddr=0;programAddr<proc->programWordsSize;programAddr++)
		programWords[programAddr] = emptyVal;
	for (uint EEPROMAddr=0;EEPROMAddr<proc->EEPROMDataSize;EEPROMAddr++)
		EEPROMData[EEPROMAddr] = 0xFF;
	for (uint deviceAddr=0;deviceAddr<proc->deviceIDSize;deviceAddr++)
		deviceID[deviceAddr] = emptyVal;
	for (uint userIDAddr=0;userIDAddr<proc->userIDWordsSize;userIDAddr++)
		userIDWords[userIDAddr] = emptyVal;

#if 1
	// Clearing the bit array this way was painfully slow - seconds
	// for just a few K bytes.
	for (int i=0;i<(int)proc->programWordsSize;i++)
		programWordsUsed[i] = false;
#else
	std::vector<bool>::iterator pwItr;
	for (pwItr=programWordsUsed.begin();pwItr!=programWordsUsed.end();pwItr++)
		*pwItr = false;
#endif
}

PICHexFile::~PICHexFile(void)
{
	if (fileName != NULL) delete[] fileName;
	if (configWords != NULL) delete[] configWords;
	if (programWords != NULL) delete[] programWords;
	if (userIDWords != NULL) delete[] userIDWords;
	if (deviceID != NULL) delete[] deviceID;
	if (EEPROMData != NULL) delete[] EEPROMData;
}

int PICHexFile::LoadFromFile(LPCTSTR hexFileName)
{
	// Free memory associated with fileName
	if (fileName != NULL)
	{
		delete[] fileName;
		fileName = NULL;
	}

	// Save a copy of the new file name
	if (hexFileName == NULL)
	{
		// No file name - return error
		return -1;
	}
	fileName = new TCHAR[_tcslen(hexFileName) + 1];
	_tcscpy(fileName, hexFileName);

	// Read the hex file.
	HexFileIO hexFileReader;
	int res = hexFileReader.Load(fileName, this);

	return res;
}

// Add a string to the end of a list of strings.  The string "str" is
// copied before being put in the list.
void PICHexFile::appendStr(TCHAR *str, CTextList *lineList)
{
	// Don't add NULL strings
	if (str == NULL)
		return;

	// Copy and append.
	TCHAR *tStr = new TCHAR[_tcslen(str)+1];
	_tcscpy(tStr, str);
	lineList->push_back(tStr);
}

CTextList *PICHexFile::Disassemble(bool showAddr)
{
	if (Processor == NULL)
	{
		MessageBox(NULL, _T("Processor not selected"), _T("Dissasembly Error"), MB_OK);
		return NULL;
	}

	// Create a queue for the output lines
	CTextList *lineList = new CTextList;
	std::list<uint> lineAddrList;

	// Figure out the first and last addresses used
	uint addr;
	uint startAddr = Processor->programWordsSize;
	uint endAddr   = 0;
	for (addr=0;addr<Processor->programWordsSize;addr++)
	{
		// Skip memory we know is unused (probably because we read in the hex file
		// and know which addresses have something in them).
		if (!programWordsUsed[addr])
			continue;

		// Skip NOP's
		ushort memWord = programWords[addr];
		if (Processor->wordSize == 14)
		{
			if ((memWord & 0x3fff) == 0x3fff)
				// NOP for a 14 bit PIC
				continue;
		}
		else if (Processor->wordSize == 16)
		{
			if (memWord == 0xffff)
				continue;
		}
		else if (Processor->wordSize == 24)
		{
			// Must be a dsPIC.  Not supported yet.
			return NULL;
		}
		else
		{
			// TBD: This is an error - shouldn't have an unrecognized
			// word size.
			assert(false);
		}

		// See if the current address is before/after existing low/high addresses
		if (addr < startAddr)
			startAddr = addr;
		if (addr+1 > endAddr)
			endAddr = addr + 1;
	}

	// If no memory locations are used then report an error.
	// if ((endAddr == 0) && (fileName == NULL))
	if (endAddr == 0)
	{
		MessageBox(NULL, _T("Empty File"), _T("Dissasembly Error"), MB_OK);
		return NULL;
	}

	// Track all locations that are targets of call/goto/branch statements
	callTargetArray callTargets;
	findCallTargets(startAddr, endAddr, callTargets);

	size_t offset;
	uint lastAddr, tAddr;
	TCHAR tBuf[128];
	ushort memWord;
	callTargetArray::iterator aItr;

	for (addr=startAddr,lastAddr=-1;addr<endAddr;addr++)
	{
		tAddr = (Processor->wordSize == 16) ? (2 * addr) : addr;

		// If this address is the target of goto/call/branch, then emit
		// a label.
		aItr = callTargets.find(tAddr);
		if (aItr != callTargets.end())
		{
			// It's a target.  Emit a label.
			uint index = aItr->second;
			if (showAddr)
				_stprintf(tBuf, _T("                label%d"), index);
			else
				_stprintf(tBuf, _T("label%d"), index);
			appendStr(tBuf, lineList);
		}

		// Skip unused memory
		if (!programWordsUsed[addr])
		{
			continue;
		}

		// If we skipped any locations, then reset the code origin
		if (lastAddr != (addr - 1))
		{
			// 16 bit processors use byte addressing (but word instructions).  Adjust
			// to byte address for output.
			tAddr = (Processor->wordSize == 16) ? (2 * addr) : addr;
			if (showAddr)
				_stprintf(tBuf, _T("                    org %#x"), tAddr);
			else
				_stprintf(tBuf, _T("    org %#x"), tAddr);
			appendStr(tBuf, lineList);
			lineAddrList.push_back(addr);
			lastAddr++;
		}

		// Pull the new hex opcode/data from code memory
		memWord = programWords[addr];

		// Emit the current address and actual opcode
		if (showAddr)
		{
			offset = _stprintf(tBuf, _T("%#06x:  %#06x     "), tAddr, memWord);
		}
		else
		{
			_tcscpy(tBuf, _T("    "));
			offset = 4;
		}

		// Decode the instruction into something legible.  Add it to the
		// end of the line we just created.
		int nWords = emitOpcode(addr, memWord, &tBuf[offset], callTargets);

		// Add line to output
		appendStr(tBuf, lineList);
		lineAddrList.push_back(addr);

		// Move to next opcode
		lastAddr = addr;

		if (nWords > 1)
		{
			// TBD: better be 2 if it isn't 1 word.
			addr++;
			lastAddr = addr;
			if (showAddr)
			{
				tAddr = (Processor->wordSize == 16) ? (2 * addr) : addr;

				// Emit the following word
				_stprintf(tBuf, _T("%#06x:  %#06x"), tAddr, programWords[addr]);

				// Add line to output
				appendStr(tBuf, lineList);
				lineAddrList.push_back(addr);
			}
		}
	}

	// We now have a list of statements, a list of addresses for those statements,
	// as well as a map of all statements that are targets of call/goto/branches.
	// Add lines to the list with targets of the call/goto.

	return lineList;
}
