#pragma once

// Interface that must be implemented for tracking progress of the device
// programmer.
class CProgrammerProgressIF
{
public:
	// Perform any necessary startup operations and display title text.
	virtual void InitProgress(TCHAR *statusText) = 0;
	// Intermediate status update - programmer is at location "addr" out of "maxAddr"
	virtual bool UpdateProgress(int addr, int maxAddr) = 0;
	// Called when the programming operation should be terminated.  Can be invoked
	// by either the main application (user requested cancel) or by the programming
	// code itself (error during programming).
	virtual void Cancel() = 0;
	// Method to see if programming should stop.  Normally used by the programming
	// code to see if the main application has requested termination.
	virtual bool IsCanceled() = 0;
};
