#include "StdAfx.h"
#include <assert.h>
#include "dsPIC30FXXXProgram.h"

dsPIC30FXXXProgram::dsPIC30FXXXProgram(void)
{
}

dsPIC30FXXXProgram::~dsPIC30FXXXProgram(void)
{
}

dsPIC30FXXXProgram dsPIC30FXXXProgram::m_Driver;

// Get the singleton programmer instance for this class of chips
const PICProgram *dsPIC30FXXXProgram::GetDriver()
{
	return &m_Driver;
}

int dsPIC30FXXXProgram::getWriteBufferSize(const PICDevice *device) const
{
	int bufferSize = 8;
	const TCHAR *devName = device->name[2];
	if (!_tcsicmp(devName, _T("dspic30f4012"))) bufferSize = 8;

	return bufferSize;
}

int dsPIC30FXXXProgram::getEraseBufferSize(const PICDevice *device) const
{
	return 64;
}

int dsPIC30FXXXProgram::ReadCode(const PICDevice *device, CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	int i = -1;

	if (statusIF) statusIF->InitProgress(_T("Reading Program Memory"));

	uint addr = 0;

	// Reset PIC.  After this is executed we will be at address 0 and can
	// increment to get to the address we actually want to read.  Make sure
	// to turn on the bit that indicates this is a dsPIC so we can select
	// the correct PGC/PGD lines (and pull the others to ground).
	PICOpen(port, device->devVppMask | 0x04, false);

	uint j;

	// Exit reset vector
	i = TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
	i = TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
	i = TxCmd(port, EVA_PIC24_CMD, 0x040100, &j);  // goto 0x100
	i = TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop

	uint instr;
	ushort LSW0, MSB0, LSW1, MSB1;
	ushort LSW2, MSB2, LSW3, MSB3;
	uint endAddr = device->programWordsSize; // 0x1000;
#if 1
	// Initialize TBLPAG and the read pointer (W0) for TBLRD instruction
	instr = 0x200000 | ((addr & 0x00ff0000) >> 12);
	i  = TxCmd(port, EVA_PIC24_CMD, instr, &j);  // mov <addr 23:16>, W0
	i += TxCmd(port, EVA_PIC24_CMD, 0x880190, &j);  // mov W0, TBLPAG
	instr = 0x200006 | ((addr & 0x0000ffff) << 4);
	i += TxCmd(port, EVA_PIC24_CMD, instr, &j);  // mov <addr 15:0>, W6

	byte outBuf[4];
	byte inBuf[128];
	uint resLen;
	for (addr=0;addr<endAddr;)
	{
		uint byteAddr = addr * 2;
		outBuf[0] = byteAddr & 0xff;
		outBuf[1] = (byteAddr & 0xff00) >> 8;
		outBuf[2] = (byteAddr & 0xff0000) >> 16;
		outBuf[3] = 0;

		i = PICProgram::SendPacket(port, PICProgram::EVA_PIC24RBUF_CMD,
			4, outBuf, 128, &resLen, inBuf, 100);
		if (i)
		{
			PICClose(port);
			return addr+1;
		}

		if (statusIF && !statusIF->UpdateProgress(addr, device->programWordsSize))
		{
			PICClose(port);
			return 1;
		}

        // Move code words into array
		int blkCount = inBuf[1] / 12;
		ushort *bufPtr = (ushort *)&inBuf[6];
		for (int blkCtr=0;blkCtr<blkCount;blkCtr++)
		{
			ushort val = *bufPtr++;
			LSW0 = (val & 0xffff);
			val = *bufPtr++;
			MSB1 = (val & 0xff00) >> 8;
			MSB0 = (val & 0xff);
			val = *bufPtr++;
			LSW1 = (val & 0xffff);
			val = *bufPtr++;
			LSW2 = (val & 0xffff);
			val = *bufPtr++;
			MSB3 = (val & 0xff00) >> 8;
			MSB2 = (val & 0xff);
			val = *bufPtr++;
			LSW3 = (val & 0xffff);

			hexFile->programWords[addr++] = LSW0;
			hexFile->programWords[addr++] = MSB0;
			hexFile->programWords[addr++] = LSW1;
			hexFile->programWords[addr++] = MSB1;
			hexFile->programWords[addr++] = LSW2;
			hexFile->programWords[addr++] = MSB2;
			hexFile->programWords[addr++] = LSW3;
			hexFile->programWords[addr++] = MSB3;
		}

		if (statusIF && !statusIF->UpdateProgress(addr, device->programWordsSize))
		{
			PICClose(port);
			return 1;
		}
	}
#else
	uint res;
	// Initialize TBLPAG and the read pointer (W6) for TBLRD instruction
	instr = 0x200000 | ((addr & 0x00ff0000) >> 12);
	i  = TxCmd(port, EVA_PIC24_CMD, instr, &j);  // mov <addr 23:16>, W0
	i += TxCmd(port, EVA_PIC24_CMD, 0x880190, &j);  // mov W0, TBLPAG
	instr = 0x200006 | ((addr & 0x0000ffff) << 4);
	i += TxCmd(port, EVA_PIC24_CMD, instr, &j);  // mov <addr 15:0>, W6

	// Read code memory 4 instructions (8 words) at a time
	for (addr=0;addr<endAddr;)
	{
		// Set the read pointer (W6) and load the write latches
		i += TxCmd(port, EVA_PIC24_CMD, 0xeb0380, &j);  // clr W7
		i += TxCmd(port, EVA_PIC24_CMD, 0xba1b96, &j);  // TBLRDL [W6], [W7++]
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i += TxCmd(port, EVA_PIC24_CMD, 0xbadbb6, &j);  // TBLRDH, B [W6++], [W7++]
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i += TxCmd(port, EVA_PIC24_CMD, 0xbadbd6, &j);  // TBLRDH, B [++W6], [W7++]
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i += TxCmd(port, EVA_PIC24_CMD, 0xba1bb6, &j);  // TBLRDL [W6++], [W7++]
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i += TxCmd(port, EVA_PIC24_CMD, 0xba1b96, &j);  // TBLRDL [W6], [W7++]
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i += TxCmd(port, EVA_PIC24_CMD, 0xbadbb6, &j);  // TBLRDH, B [W6++], [W7++]
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i += TxCmd(port, EVA_PIC24_CMD, 0xbadbd6, &j);  // TBLRDH, B [++W6], [++W7]
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i += TxCmd(port, EVA_PIC24_CMD, 0xba0bb6, &j);  // TBLRDL [W6++], [W7]
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop

		// Output W0:W5 using the VISI register and REGOUT command
		i += TxCmd(port, EVA_PIC24_CMD, 0x883c20, &j);    // mov W0, VISI
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);    // nop
		i += TxCmd(port, EVA_PIC24R_CMD, 0x000000, &res); // Read register
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);    // nop
		LSW0 = (res & 0xffff);

		i += TxCmd(port, EVA_PIC24_CMD, 0x883c21, &j);    // mov W1, VISI
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);    // nop
		i += TxCmd(port, EVA_PIC24R_CMD, 0x000000, &res); // Read register
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);    // nop
		MSB1 = (res & 0xff00) >> 8;
		MSB0 = (res & 0xff);

		i += TxCmd(port, EVA_PIC24_CMD, 0x883c22, &j);    // mov W2, VISI
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);    // nop
		i += TxCmd(port, EVA_PIC24R_CMD, 0x000000, &res); // Read register
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);    // nop
		LSW1 = (res & 0xffff);

		i += TxCmd(port, EVA_PIC24_CMD, 0x883c23, &j);    // mov W3, VISI
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);    // nop
		i += TxCmd(port, EVA_PIC24R_CMD, 0x000000, &res); // Read register
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);     // nop
		LSW2 = (res & 0xffff);

		i += TxCmd(port, EVA_PIC24_CMD, 0x883c24, &j);    // mov W4, VISI
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);    // nop
		i += TxCmd(port, EVA_PIC24R_CMD, 0x000000, &res); // Read register
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);    // nop
		MSB3 = (res & 0xff00) >> 8;
		MSB2 = (res & 0xff);

		i += TxCmd(port, EVA_PIC24_CMD, 0x883c25, &j);    // mov W5, VISI
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);    // nop
		i += TxCmd(port, EVA_PIC24R_CMD, 0x000000, &res); // Read register
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);    // nop
		LSW3 = (res & 0xffff);

		i += TxCmd(port, EVA_PIC24_CMD, 0x040100, &res);  // goto 0x100
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &res);  // nop

        // Move code words into array
		hexFile->programWords[addr++] = LSW0;
		hexFile->programWords[addr++] = MSB0;
		hexFile->programWords[addr++] = LSW1;
		hexFile->programWords[addr++] = MSB1;
		hexFile->programWords[addr++] = LSW2;
		hexFile->programWords[addr++] = MSB2;
		hexFile->programWords[addr++] = LSW3;
		hexFile->programWords[addr++] = MSB3;

		if (i)
		{
			PICClose(port);
			return addr+1;
		}

		if (statusIF && !statusIF->UpdateProgress(addr, device->programWordsSize))
		{
			PICClose(port);
			return 1;
		}
	}
#endif

	PICClose(port);

	return 0;
}

int dsPIC30FXXXProgram::ReadConfig(const PICDevice *device, CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	int i = -1;
	uint res, j;

	if (statusIF) statusIF->InitProgress(_T("Reading Config Bits"));

	// Reset PIC
	PICOpen(port, device->devVppMask | 0x04, false);

	// Exit reset vector
	i = TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
	i = TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
	i = TxCmd(port, EVA_PIC24_CMD, 0x040100, &j);  // goto 0x100
	i = TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop

	// Initialize TBLPAG, the read pointer (W6), and the write
	// pointer (W7) for TBLRD instruction
	i = TxCmd(port, EVA_PIC24_CMD, 0x200f80, &j);  // mov 0xf8, W0
	i = TxCmd(port, EVA_PIC24_CMD, 0x880190, &j);  // mov W0, TBLPAG
	i = TxCmd(port, EVA_PIC24_CMD, 0xeb0300, &j);  // clr W6
	i = TxCmd(port, EVA_PIC24_CMD, 0xeb0380, &j);  // clr W7

#if 1
	uint cRegs[16];
	for (uint addr=0;addr<device->configWordsSize;addr++)
	{
		// Read the configuration register and write it to the VISI register
		// (located at 0x784)
		i = TxCmd(port, EVA_PIC24_CMD, 0xba0bb6, &j);  // TBLRDL [W6++], [W7]
		i = TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i = TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i = TxCmd(port, EVA_PIC24_CMD, 0x883c20, &j);  // mov W0, VISI
		i = TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop

		// Output the VISI register using the REGOUT command
		i = TxCmd(port, EVA_PIC24R_CMD, 0x000000, &res); // Read register
		cRegs[addr] = res;
		hexFile->configWords[addr] = res;
		i = TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop

		// Reset device internal PC
		i = TxCmd(port, EVA_PIC24_CMD, 0x040100, &j);  // goto 0x100
		i = TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop

		if (statusIF && !statusIF->UpdateProgress(addr, device->configWordsSize))
		{
			PICClose(port);
			return 1;
		}
	}
#endif

	// Read the device ID registers at address 0xFF0000:0xff0002
	// Exit reset vector
	i = TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
	i = TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
	i = TxCmd(port, EVA_PIC24_CMD, 0x040100, &j);  // goto 0x100
	i = TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop

	// Initialize TBLPAG, the read pointer (W6), and the write
	// pointer (W7) for TBLRD instruction
	i = TxCmd(port, EVA_PIC24_CMD, 0x200ff0, &j);  // mov 0xff, W0
	i = TxCmd(port, EVA_PIC24_CMD, 0x880190, &j);  // mov W0, TBLPAG
	i = TxCmd(port, EVA_PIC24_CMD, 0xeb0300, &j);  // clr W6
	i = TxCmd(port, EVA_PIC24_CMD, 0xeb0380, &j);  // clr W7

	for (uint addr=0;addr<device->deviceIDSize;addr++)
	{
		// Read the configuration register and write it to the VISI register
		// (located at 0x784)
		i = TxCmd(port, EVA_PIC24_CMD, 0xba0bb6, &j);  // TBLRDL [W6++], [W7]
		i = TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i = TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i = TxCmd(port, EVA_PIC24_CMD, 0x883c20, &j);  // mov W0, VISI
		i = TxCmd(port, EVA_PIC24_CMD, 0xeb0380, &j);  // clr W7

		// Output the VISI register using the REGOUT command
		i = TxCmd(port, EVA_PIC24R_CMD, 0x000000, &res); // Read register
		hexFile->deviceID[addr] = res;
		i = TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop

		// Reset device internal PC
		i = TxCmd(port, EVA_PIC24_CMD, 0x040100, &j);  // goto 0x100
		i = TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
	}

#if 0
	uint b0, b1;
	uint addr, wordVal;

	// Reset PIC
	PICOpen(port, device->devVppMask | 0x04, false);

	// Set the starting address to 0x300000
	i = SetTablePointer(port, 2*device->configAddr);
	if (i) { PICClose(port); return(1); }

	// read configuration bits (TBD: config is read a byte at a time - need
	// to assemble the bytes as they come in.)
	for (addr=0;addr<device->configWordsSize;addr++)
	{
		// Read the current address and increment the table pointer
		i  = TxCmd(port, EVA_PIC16R_CMD, PIC16_TBLRD_POSTINC, &b0);
		i += TxCmd(port, EVA_PIC16R_CMD, PIC16_TBLRD_POSTINC, &b1);

		// wordVal = (b1 & 0xff00) | (b0 >> 8);
		wordVal = ((b1 & 0x00ff) << 8) | (b0 & 0xff);
		// wordVal = (b1 | b0);

		// Check for any errors
		if (i)
		{
			PICClose(port);
			return(1);
		}

		assert(addr < device->configWordsSize);

		// Save the contents of the address.  Note that the response contains the
		// start and stop bit from the PIC.  To get the final 1 bit contents
		// we need to shift and mask the extra bits.
		hexFile->configWords[addr] = wordVal;

		if (statusIF && !statusIF->UpdateProgress(addr, device->configWordsSize))
		{
			PICClose(port);
			return 1;
		}
	}

	// read device ID
	SetTablePointer(port, 2*device->deviceIDAddr);
	i  = TxCmd(port, EVA_PIC16R_CMD, PIC16_TBLRD_POSTINC, &b0);
	i += TxCmd(port, EVA_PIC16R_CMD, PIC16_TBLRD_POSTINC, &b1);
	hexFile->deviceID[0] = ((b1 & 0x00ff) << 8) | (b0 & 0xff);

	// read user ID
	if (statusIF) statusIF->InitProgress(_T("Reading User ID Values"));
	SetTablePointer(port, 2*device->userIDWordsAddr);
	for (addr=0;addr<device->userIDWordsSize;addr++)
	{
		// Read the current address and increment the table pointer
		i  = TxCmd(port, EVA_PIC16R_CMD, PIC16_TBLRD_POSTINC, &b0);
		i += TxCmd(port, EVA_PIC16R_CMD, PIC16_TBLRD_POSTINC, &b1);

		wordVal = ((b1 & 0x00ff) << 8) | (b0 & 0xff);

		// Check for any errors
		if (i) { PICClose(port); return 1; }

		assert(addr < device->userIDWordsSize);

		// Save the contents of the address.  Note that the response contains the
		// start and stop bit from the PIC.  To get the final 1 bit contents
		// we need to shift and mask the extra bits.
		hexFile->userIDWords[addr] = wordVal;

		if (statusIF && !statusIF->UpdateProgress(addr, device->configWordsSize))
		{
			PICClose(port);
			return 1;
		}
	}
#endif

	PICClose(port);

	return 0;
}

// Read a single byte from the data EEPROM.  Note: the programmer MUST have already been
// set up in programming mode.
int dsPIC30FXXXProgram::ReadDataByte(const PICDevice *device, CIOPort *port, uint addr, byte *val) const
{
	int i = -1;

	*val = 0x00;

	return i;
}

int dsPIC30FXXXProgram::ReadData(const PICDevice *device, CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	int i = -1;

	if (statusIF) statusIF->InitProgress(_T("Reading Data Memory"));

#if 0
	int addr;
	uint j;

	// Reset PIC
	PICOpen(port, device->devVppMask | 0x04, false);

#if 1
	byte outBuf[4], inBuf[128];
	uint block, resLen;
	uint blockSize = CMD_BLOCKSIZE;
	for (addr=0,block=0;addr<device->EEPROMDataSize;block+=blockSize)
	{
		uint byteAddr = addr;
		outBuf[0] = byteAddr & 0xff;
		outBuf[1] = (byteAddr & 0xff00) >> 8;
		outBuf[2] = (byteAddr & 0xff0000) >> 8;
		outBuf[3] = blockSize;
		i = PICProgram::SendPacket(port, PICProgram::EVA_PIC16RBE_CMD,
			4, outBuf, 128, &resLen, inBuf, 2*blockSize);
		if (i) { PICClose(port); return addr+1; }

		// Move the block response into the result array
		byte *valBuf = &inBuf[6]; // Location of first value in response buffer
		for (uint i0=0;i0<blockSize;i0++)
		{
			// The programmer sends back byte data so we can pull
			// the values right out of the buffer.
			hexFile->EEPROMData[addr++] = *valBuf++;
		}

		if (statusIF && !statusIF->UpdateProgress(addr, device->EEPROMDataSize))
		{
			PICClose(port);
			return 1;
		}
	}
#else
	uint wordVal;

	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x9EA6, &j);
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x9CA6, &j);

	// read data
	uint b0;
	for (addr=0;addr<device->EEPROMDataSize;addr++)
	{
		uint bAddr = (addr >= 0 ? addr : 0);

		// Set address of the byte we want to read
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0E00 | (bAddr & 0xFF), &j);
		i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EA9, &j);

		// Set address of the byte we want to read
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0E00 | ((bAddr & 0xFF00) >> 8), &j);
		i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EAA, &j);

		// Initiate a memory read
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x80A6, &j);

		// Load the data into the serial data holding register
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x50A8, &j);
		i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EF5, &j);

		// i = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0000, &j);

		// Shift out the data
		i  = TxCmd(port, EVA_PIC16R_CMD, PIC16_SHIFT_TBLAT, &b0);

		wordVal = (b0 & 0xff);

		// Check for any errors
		if (i) { PICClose(port); return(1); }

		assert(bAddr < device->EEPROMDataSize);

		// Save the contents of the address.
		hexFile->EEPROMData[bAddr] = wordVal;

		if (statusIF && !statusIF->UpdateProgress(bAddr, device->EEPROMDataSize))
		{
			PICClose(port);
			return 1;
		}
	}
#endif

	PICClose(port);
#endif

	return 0;
}


int dsPIC30FXXXProgram::PartialRead(CIOPort *port, PICHexFile *hexFile, bool codeFlag, bool dataFlag, bool configFlag, CProgrammerProgressIF *statusIF) const
{
	const PICDevice *device = hexFile->Processor;

    if (!SUCCEEDED(ResetProgrammer(port)))
	{
		// Failed to get the programmer into a known good state.
		return 1;
	}

	int res = 0;
	if (codeFlag && (res = ReadCode(device, port, hexFile, statusIF))) return res;
	if (configFlag && (res == 0) && (res = ReadConfig(device, port, hexFile, statusIF))) return res;
	if (dataFlag && (res == 0) && (res = ReadData(device, port, hexFile, statusIF))) return res;

	PICClose(port);

	return 0;
}

int dsPIC30FXXXProgram::Read(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	const PICDevice *device = hexFile->Processor;

    if (!SUCCEEDED(ResetProgrammer(port)))
	{
		// Failed to get the programmer into a known good state.
		if (!SUCCEEDED(ResetProgrammer(port)))
			return 1;
	}

	int res;
	if ((res = ReadCode(device, port, hexFile, statusIF)) != 0) return res;
	if ((res = ReadConfig(device, port, hexFile, statusIF)) != 0) return res;
	if ((res = ReadData(device, port, hexFile, statusIF)) != 0) return res;

	// done
	PICClose(port);

	return 0;
}

int dsPIC30FXXXProgram::setupWriteCode(const PICDevice *device, CIOPort *port, PICHexFile *hexFile) const
{
	int i = -1;
#if 0
	uint j;

	// Direct access to config memory
	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x8EA6, &j); // bsf EECON1, EEPGD
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x8CA6, &j); // bcf EECON1, CFGS

	// Configure device for single panel writes
	i += SetTablePointer(port, 0x3c0006);
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_TBLWRT, &j);        // Write 0x00 to 0x3C0006 to enable single-panel writes

	// Direct access to code memory
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x8EA6, &j); // bsf EECON1, EEPGD
	i += TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x9CA6, &j); // bcf EECON1, CFGS
#endif
	return i;
}

int dsPIC30FXXXProgram::WriteCode(const PICDevice *device, CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	int i = -1;

	if (statusIF) statusIF->InitProgress(_T("Writing Program Memory"));

	uint addr;

	// Reset PIC.  After this is executed we will be at address 0 and can
	// increment to get to the address we actually want to read.
	PICOpen(port, device->devVppMask | 0x04, false);

	uint j;

	// Exit reset vector
	i = TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
	i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
	i += TxCmd(port, EVA_PIC24_CMD, 0x040100, &j);  // goto 0x100
	i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop

	// Set NVMCON to program 32 instruction words
	i += TxCmd(port, EVA_PIC24_CMD, 0x24001a, &j);  // mov 0x4001, W10
	i += TxCmd(port, EVA_PIC24_CMD, 0x883b0a, &j);  // mov W10, NVMCON

	// Write code memory in CMD_BLOCKSIZE byte chunks.
	byte outBuf[128], inBuf[128];
	uint resLen;
	uint bufferSize = getWriteBufferSize(device);
	uint endAddr = device->programWordsSize; // 0x400;
	for (addr=0;addr<endAddr;)
	{
		uint byteAddr = addr * 2;
		uint instr;
		ushort LSW0, MSB0, LSW1, MSB1;
		ushort LSW2, MSB2, LSW3, MSB3;

#if 1
#if 1
		uint txAddr = ((addr & 0x00ffff00) >> 8) | ((addr & 0x000000ff) << 16);
		i += PICProgram::TxCmd(port, EVA_PIC24WBUF_SETUP, txAddr, &j);
#else
		outBuf[0] = addr & 0xff;
		outBuf[1] = (addr & 0xff00) >> 8;
		outBuf[2] = (addr & 0xff0000) >> 16;
		outBuf[3] = 0;
		i += PICProgram::SendPacket(port, EVA_PIC24WBUF_SETUP,
			4, outBuf, 128, &resLen, inBuf, 10);
#endif
#else
		// Set NVMCON to program 32 instruction words
		i += TxCmd(port, EVA_PIC24_CMD, 0x24001a, &j);  // mov 0x4001, W10
		i += TxCmd(port, EVA_PIC24_CMD, 0x883b0a, &j);  // mov W10, NVMCON

		// Initialize the write pointer (W7) for TBLWT instruction
		instr = 0x200000 | ((addr & 0x00ff0000) >> 12);
		i += TxCmd(port, EVA_PIC24_CMD, instr, &j);  // mov <addr 23:16>, W0
		i += TxCmd(port, EVA_PIC24_CMD, 0x880190, &j);  // mov W0, TBLPAG
		instr = 0x200007 | ((addr & 0x0000ffff) << 4);
		i += TxCmd(port, EVA_PIC24_CMD, instr, &j);  // mov <addr 15:0>, W7
#endif

#if 1
		for (int ctr=0;ctr<2;ctr++)
		{
			byteAddr = addr * 2;
			outBuf[0] = byteAddr & 0xff;
			outBuf[1] = (byteAddr & 0xff00) >> 8;
			outBuf[2] = (byteAddr & 0xff0000) >> 16;
			outBuf[3] = 0;
			byte *bufPtr = &outBuf[4];

			for (int ctr2=0;ctr2<4;ctr2++)
			{
				// Initialize read pointer (W6) and load W0:W5 with the next 4 instruction
				// words to program
				LSW0 = hexFile->programWords[addr++];
				MSB0 = hexFile->programWords[addr++];
				LSW1 = hexFile->programWords[addr++];
				MSB1 = hexFile->programWords[addr++];
				LSW2 = hexFile->programWords[addr++];
				MSB2 = hexFile->programWords[addr++];
				LSW3 = hexFile->programWords[addr++];
				MSB3 = hexFile->programWords[addr++];

				*bufPtr++ = (LSW0 & 0xff);
				*bufPtr++ = (LSW0 >> 8) & 0xff;
				*bufPtr++ = (MSB0 & 0xff);
				*bufPtr++ = (MSB1 & 0xff);
				*bufPtr++ = (LSW1 & 0xff);
				*bufPtr++ = (LSW1 >> 8) & 0xff;
				*bufPtr++ = (LSW2 & 0xff);
				*bufPtr++ = (LSW2 >> 8) & 0xff;
				*bufPtr++ = (MSB2 & 0xff);
				*bufPtr++ = (MSB3 & 0xff);
				*bufPtr++ = (LSW3 & 0xff);
				*bufPtr++ = (LSW3 >> 8) & 0xff;
			}

			i += PICProgram::SendPacket(port, PICProgram::EVA_PIC24WBUF_CMD,
				4 + 4 * 12, outBuf, 128, &resLen, inBuf, 100);
		}
#else
		for (int ctr=0;ctr<8;ctr++)
		{
			// Initialize read pointer (W6) and load W0:W5 with the next 4 instruction
			// words to program
			LSW0 = hexFile->programWords[addr++];
			MSB0 = hexFile->programWords[addr++];
			LSW1 = hexFile->programWords[addr++];
			MSB1 = hexFile->programWords[addr++];
			LSW2 = hexFile->programWords[addr++];
			MSB2 = hexFile->programWords[addr++];
			LSW3 = hexFile->programWords[addr++];
			MSB3 = hexFile->programWords[addr++];

			instr = 0x200000 | (LSW0 << 4);
			i += TxCmd(port, EVA_PIC24_CMD, instr, &j);  // mov LSW0, W0
			instr = 0x200001 | ((MSB1 & 0x00ff) << 12) | ((MSB0 & 0xff) << 4);
			i += TxCmd(port, EVA_PIC24_CMD, instr, &j);  // mov <MSB1:MSB0>, W1
			instr = 0x200002 | (LSW1 << 4);
			i += TxCmd(port, EVA_PIC24_CMD, instr, &j);  // mov LSW1, W2
			instr = 0x200003 | (LSW2 << 4);
			i += TxCmd(port, EVA_PIC24_CMD, instr, &j);  // mov LSW2, W3
			instr = 0x200004 | ((MSB3 & 0x00ff) << 12) | ((MSB2 & 0xff) << 4);
			i += TxCmd(port, EVA_PIC24_CMD, instr, &j);  // mov <MSB3:MSB2>, W4
			instr = 0x200005 | (LSW3 << 4);
			i += TxCmd(port, EVA_PIC24_CMD, instr, &j);  // mov LSW3, W5

			// Set the read pointer (W6) and load the write latches
			i += TxCmd(port, EVA_PIC24_CMD, 0xeb0300, &j);  // clr W6
			i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
			i += TxCmd(port, EVA_PIC24_CMD, 0xbb0bb6, &j);  // TBLWRT [W6++], [W7]
			i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
			i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
			i += TxCmd(port, EVA_PIC24_CMD, 0xbbdbb6, &j);  // TBLWTH, B [W6++], [W7++]
			i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
			i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
			i += TxCmd(port, EVA_PIC24_CMD, 0xbbebb6, &j);  // TBLWTH, B [W6++], [++W7]
			i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
			i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
			i += TxCmd(port, EVA_PIC24_CMD, 0xbb1bb6, &j);  // TBLWTL [W6++], [W7++]
			i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
			i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
			i += TxCmd(port, EVA_PIC24_CMD, 0xbb0bb6, &j);  // TBLWTL [W6++], [W7]
			i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
			i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
			i += TxCmd(port, EVA_PIC24_CMD, 0xbbdbb6, &j);  // TBLWTH, B [W6++], [W7++]
			i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
			i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
			i += TxCmd(port, EVA_PIC24_CMD, 0xbbebb6, &j);  // TBLWTH, B [W6++], [++W7]
			i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
			i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
			i += TxCmd(port, EVA_PIC24_CMD, 0xbb1bb6, &j);  // TBLWTL [W6++], [W7++]
			i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
			i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		}
#endif

#if 1
		i += TxCmd(port, EVA_PIC24WBUF_FINISH, 0x0000, &j);
#else
		// Unlock the NVMCON for writing
		i += TxCmd(port, EVA_PIC24_CMD, 0x200558, &j);  // mov 0x55, W8
		i += TxCmd(port, EVA_PIC24_CMD, 0x883b38, &j);  // mov W8, NVMKEY
		i += TxCmd(port, EVA_PIC24_CMD, 0x200aa9, &j);  // mov 0xaa, W9
		i += TxCmd(port, EVA_PIC24_CMD, 0x883b39, &j);  // mov W9, NVMKEY

		// Initiate the write cycle
		i += TxCmd(port, EVA_PIC24_CMD, 0xa8e761, &j);  // bset NVMCON, WR
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop

		::Sleep(2); // Externally timed 2 msec

		i += TxCmd(port, EVA_PIC24_CMD, 0xa9e761, &j);  // bclr NVMCON, WR
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop

		// Reset the device internal PC
		i += TxCmd(port, EVA_PIC24_CMD, 0x040100, &j);  // goto 0x100
		i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
#endif

		if (statusIF && !statusIF->UpdateProgress(addr, device->programWordsSize))
		{
			PICClose(port);
			return 1;
		}
	}

	PICClose(port);

	return 0;
}

int dsPIC30FXXXProgram::WriteData(const PICDevice *device, CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	int i = -1;

	if (statusIF) statusIF->InitProgress(_T("Writing EEPROM Memory"));

#if 0
	uint addr;

	// Open programmer
	PICOpen(port, device->devVppMask | 0x04, false);

	uint j;
	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x9EA6, &j); // bsf EECON1, EEPGD
	i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x9CA6, &j); // bsf EECON1, CFGS

#if 1
	// Write data memory in CMD_BLOCKSIZE byte chunks.
	byte outBuf[128], inBuf[128];
	uint block, resLen;
	uint blockSize = CMD_BLOCKSIZE;
	for (addr=0,block=0;addr<device->EEPROMDataSize;block+=blockSize)
	{
		uint byteAddr = addr;
		outBuf[0] = byteAddr & 0xff;
		outBuf[1] = (byteAddr & 0xff00) >> 8;
		outBuf[2] = (byteAddr & 0xff0000) >> 16;
		outBuf[3] = CMD_BLOCKSIZE; // # of words per pass within the block write routine (adjust by processor)

		byte *valBuf = &outBuf[4]; // Location of first value in response buffer

		for (uint i0=0;i0<blockSize;i0++)
		{
			*valBuf++ = (hexFile->EEPROMData[addr++] & 0xff);
		}
		i = 1;
		for (int retryCtr=0;i!=0 && retryCtr<4;retryCtr++)
		{
			i = PICProgram::SendPacket(port, PICProgram::EVA_PIC16WBE_CMD,
				blockSize+4, outBuf, 128, &resLen, inBuf, 8*blockSize);
			if (i)
			{
				if (retryCtr<3)
					continue;
				PICClose(port);
				return addr+1;
			}
			if (inBuf[0] == 0xee)
			{
				if (retryCtr < 3)
					continue;
				// Write failed
				return addr+1;
			}
			if (memcmp(&outBuf[4], &inBuf[6], blockSize) != 0)
			{
				if (retryCtr < 3)
					continue;
				// Buffers don't match - programming failed
				return addr+1;
			}

			if (statusIF && !statusIF->UpdateProgress(addr, device->EEPROMDataSize))
			{
				PICClose(port);
				return 1;
			}
		}
	}
#else
	// Step through EEPROM values, writing and verifying one at a time
	for(addr=0;addr<device->EEPROMDataSize;addr++)
	{
		// Set the data EEPROM address pointer
		byte addrH = (addr & 0xff00) >> 8;
		byte addrL = (addr & 0xff);
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0E00 | addrL, &j);
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EA9, &j);       // movwf EEADR
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0E00 | addrH, &j);
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EAA, &j);       // movwf EEADRH

		// Load the data to be written
		byte data = (hexFile->EEPROMData[addr] & 0xff);
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0E00 | data, &j);  // movlw <Data>
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EA8, &j);       // movwf EEDATA

		// Enable memory writes
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x84A6, &j); // bsf EECON1, WREN

		// Perform required sequence
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0E55, &j); // movlw 0x55
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EA7, &j); // movwf EECON2
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0EAA, &j); // movlw 0xaa
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EA7, &j); // movwf EECON2

		// Initiate write
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x82A6, &j); // bsf EECON1, WR

		// Poll WR bit, repeat until clear
		for (int WRDelay=0;WRDelay<16;WRDelay++)
		{
			i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x50A6, &j); // movf EECON1, W, 0
			i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x6EF5, &j); // movwf TABLAT
			i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x0000, &j); // NOP
			i  = TxCmd(port, EVA_PIC16R_CMD, PIC16_SHIFT_TBLAT, &j);  // shift out data
			byte EECON1 = j & 0xff;
			if (EECON1 & 0x02)
			{
				::Sleep(1);
			}
			else
				break;
		}

		// Disable writes
		i  = TxCmd(port, EVA_PIC16_CMD, PIC16_CORE | 0x94A6, &j); // bcf EECON1, WREN

		if (statusIF && !statusIF->UpdateProgress(addr, device->EEPROMDataSize))
		{
			PICClose(port);
			return 1;
		}
	}
#endif
	// Shut down connection to programmer
	PICClose(port);
#endif

	return 0;
}

int dsPIC30FXXXProgram::WriteConfig(const PICDevice *device, CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	int i = -1;

	if (statusIF) statusIF->InitProgress(_T("Writing Config Memory"));

	return 0;
}

int dsPIC30FXXXProgram::WriteUser(const PICDevice *device, CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	if (statusIF) statusIF->InitProgress(_T("Writing User IDs"));

	PICOpen(port, device->devVppMask | 0x04, false);

	PICClose(port);

	return 0;
}

int dsPIC30FXXXProgram::Program(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	int res;

	const PICDevice *device = hexFile->Processor;

    if (!SUCCEEDED(ResetProgrammer(port)))
	{
		// Failed to get the programmer into a known good state.
		return 1;
	}

	// Send the contents of the hex file to the PIC
	if ((res = WriteCode(device, port, hexFile, statusIF)) != 0) return res;
	if ((res = WriteData(device, port, hexFile, statusIF)) != 0) return res;
	if ((res = WriteConfig(device, port, hexFile, statusIF)) != 0) return res;

	// Ensure everything shut down
	PICClose(port);

	return 0;
}

int dsPIC30FXXXProgram::PartialProgram(CIOPort *port, PICHexFile *hexFile, bool codeFlag, bool dataFlag, bool configFlag, CProgrammerProgressIF *statusIF) const
{
	int res;

	const PICDevice *device = hexFile->Processor;

    if (!SUCCEEDED(ResetProgrammer(port)))
	{
		// Failed to get the programmer into a known good state.
		return 1;
	}

	// Send the contents of the hex file to the PIC
	if (codeFlag && (res = WriteCode(device, port, hexFile, statusIF)) != 0) return res;
	if (dataFlag && (res = WriteData(device, port, hexFile, statusIF)) != 0) return res;
	if (configFlag && (res = WriteConfig(device, port, hexFile, statusIF)) != 0) return res;

	// Ensure everything shut down
	PICClose(port);

	return 0;
}

int dsPIC30FXXXProgram::EraseBoot(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	return -1;
}

int dsPIC30FXXXProgram::EraseNonBoot(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	int i = -1;

	return i;
}

int dsPIC30FXXXProgram::EraseCode(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	int i = -1;

	return i;
}

int dsPIC30FXXXProgram::EraseData(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	const PICDevice *device = hexFile->Processor;

	// Erase the Data EEPROM by writing 0xFF into every location (the data
	// EEPROM automatically erases a location prior to a write).
	if (statusIF) statusIF->InitProgress(_T("Erasing Data EEPROM"));

	return 0;
}

int dsPIC30FXXXProgram::Erase(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	const PICDevice *device = hexFile->Processor;
	int i = -1;

	if (statusIF) statusIF->InitProgress(_T("Reading Program Memory"));

	// Reset PIC.  After this is executed we will be at address 0 and can
	// increment to get to the address we actually want to read.
	PICOpen(port, device->devVppMask | 0x04, false);

	uint j;

	// Exit reset vector
	i = TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
	i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
	i += TxCmd(port, EVA_PIC24_CMD, 0x040100, &j);  // goto 0x100
	i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop

	// Set NVMCON to program the RESERVED1 Configuration register
	i += TxCmd(port, EVA_PIC24_CMD, 0x24008a, &j);  // mov #0x4008, W10
	i += TxCmd(port, EVA_PIC24_CMD, 0x883b0a, &j);  // W10, MVMCON

	// Initialize the TBLPAG and write pointer (W7) for TBLWT instruction
	// for Configuration register
	// Initialize TBLPAG and the read pointer (W0) for TBLRD instruction
	i += TxCmd(port, EVA_PIC24_CMD, 0x200f80, &j);  // mov 0xf8, W0
	i += TxCmd(port, EVA_PIC24_CMD, 0x880190, &j);  // mov W0, TBLPAG
	i += TxCmd(port, EVA_PIC24_CMD, 0x200067, &j);  //mov #0x6, W7

	// Load the Configuration Register data to W6
	i += TxCmd(port, EVA_PIC24_CMD, 0xeb0300, &j);  // clr W6

#if 0
	// Programming spec says this block only necessary for 30F5011 & 30F5013
	for (int ctr=0;ctr<2;ctr++)
	{
		// Load the Configuration Register write latch.  Advance W7 to point
		// to next Configuration register.
		i = TxCmd(port, EVA_PIC24_CMD, 0xbb1b86, &j);  // TBLWTL W6, [W7++]

		// Unlock the NVMCON for programming the Configuration register

		// Initiate the programming cycle

		// Repeeat last three steps to program 0x0000 to RESERVED2 Configuration register
	}
#endif

	// Set the NVMCON to erase all program memory
	i += TxCmd(port, EVA_PIC24_CMD, 0x2407fa, &j);  // mov #0x407f, W10
	i += TxCmd(port, EVA_PIC24_CMD, 0x883b0a, &j);  // mov W10, NVMCON

	// Unlock the NVMCON for programming
	i += TxCmd(port, EVA_PIC24_CMD, 0x200558, &j);  // mov 0x55, W8
	i += TxCmd(port, EVA_PIC24_CMD, 0x883b38, &j);  // mov W8, NVMKEY
	i += TxCmd(port, EVA_PIC24_CMD, 0x200aa9, &j);  // mov 0xaa, W9
	i += TxCmd(port, EVA_PIC24_CMD, 0x883b39, &j);  // mov W9, NVMKEY

	// Initiate the erase cycle
	i += TxCmd(port, EVA_PIC24_CMD, 0xa8e761, &j);  // bset NVMCON, WR
	i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
	i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop

	::Sleep(2); // Externally timed 2 msec

	i += TxCmd(port, EVA_PIC24_CMD, 0xa9e761, &j);  // bclr NVMCON, WR
	i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop
	i += TxCmd(port, EVA_PIC24_CMD, 0x000000, &j);  // nop

	return i;
}

int dsPIC30FXXXProgram::PartialErase(CIOPort *port, PICHexFile *hexFile, bool bootFlag, bool codeFlag, bool dataFlag, bool configFlag, CProgrammerProgressIF *statusIF) const
{
	int i = 0;

	if (bootFlag)
		i = EraseBoot(port, hexFile, statusIF);
	if (codeFlag)
		i += EraseCode(port, hexFile, statusIF);
	if (dataFlag)
		i += EraseData(port, hexFile, statusIF);

	return i;
}

int dsPIC30FXXXProgram::Verify(CIOPort *port, PICHexFile *hexFile, CProgrammerProgressIF *statusIF) const
{
	return 0;
}
