#include "StdAfx.h"
#include "DefaultsManager.h"

TCHAR *CDefaultsManager::EvaProgOptionKeys =
	_T("Software\\Xander\\Eva\\Options");
static defaultsIntList *allowedBaudRates = NULL;
static defaultsIntList *allowedAsmViews = NULL;
static defaultsLabelList *allowedAsmViewLabels = NULL;


// Registry keys and defaults values to use if they aren't in the registry.
#define COMPortKey _T("PortName")
#define defaultCOMPort _T("COM1:")
#define ProcessorNameKey _T("ProcessorName")
#define defaultProcessorName _T("pic16f684")
#define BaudRateKey _T("BaudRate")
#define defaultBaudRate 115200
#define AsmDisplayKey _T("DisasmView")
#define defaultAsmView 1

CDefaultsManager::CDefaultsManager(void)
{
	portName = new TCHAR[_tcslen(defaultCOMPort)+1];
	_tcscpy(portName, defaultCOMPort);
	processorName = new TCHAR[_tcslen(defaultProcessorName)+1];
	_tcscpy(processorName, defaultProcessorName);
	baudRate = defaultBaudRate;
	asmView = defaultAsmView;

	if (allowedBaudRates == NULL)
	{
		allowedBaudRates = new defaultsIntList();
		allowedBaudRates->push_back(CBR_38400);
		allowedBaudRates->push_back(CBR_115200);
	}
	if (allowedAsmViews == NULL)
	{
		allowedAsmViews = new defaultsIntList();
		allowedAsmViews->push_back(0);
		allowedAsmViews->push_back(1);
	}
	if (allowedAsmViewLabels == NULL)
	{
		allowedAsmViewLabels = new defaultsLabelList();
		allowedAsmViewLabels->push_back(_T("Assembly Only"));
		allowedAsmViewLabels->push_back(_T("Addresses"));
	}
}

CDefaultsManager::~CDefaultsManager(void)
{
	if (portName != NULL) delete[] portName;
	if (processorName != NULL) delete[] processorName;
}

bool CDefaultsManager::Load()
{
	TCHAR strBuf[64];
	HKEY key;
	DWORD keytype, keysize;
	int tInt;

	LONG keystatus = RegOpenKeyEx(HKEY_CURRENT_USER, EvaProgOptionKeys, 0, KEY_READ, &key);
	if (keystatus != ERROR_SUCCESS) {
		// The main key wasn't there.  We create it and fill it with default
		// options.
		DWORD createstatus;
		keystatus = RegCreateKeyEx(HKEY_CURRENT_USER, EvaProgOptionKeys, NULL, NULL,
			REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &key, &createstatus);
		if (keystatus == ERROR_SUCCESS) {
			// Created the key - now let's put in the default values for the key
			RegCloseKey(key);
			Save();
		} else {
			// Eliminate this error message to see if it gets past NT problems...
			// RegError(keystatus);
		}

		return true;
	}

	// Get the last COM port used
	keytype = REG_SZ;
	keysize = sizeof(strBuf)-1;
	// Ask for the size of the string
	keystatus = RegQueryValueEx(key, COMPortKey, NULL, &keytype, (BYTE *)strBuf, &keysize);
	if (keystatus == ERROR_SUCCESS) {
		if (portName != NULL) delete[] portName;
		portName = new TCHAR[keysize+1];
		_tcscpy(portName, strBuf);
	}
	else if (keystatus == ERROR_MORE_DATA)
	{
		TCHAR *buf = new TCHAR[keysize+1];
		// Get the actual string
		keystatus = RegQueryValueEx(key, COMPortKey, NULL, &keytype, (BYTE *)buf, &keysize);
		if (keystatus == ERROR_SUCCESS)
		{
			if (portName != NULL) delete[] portName;
			portName = buf;
		}
	}

	// Get the last processor used
	keytype = REG_SZ;
	keysize = sizeof(strBuf)-1;
	// Ask for the size of the string
	keystatus = RegQueryValueEx(key, ProcessorNameKey, NULL, &keytype, (BYTE *)strBuf, &keysize);
	if (keystatus == ERROR_SUCCESS)
	{
		if (processorName != NULL) delete[] processorName;
		processorName = new TCHAR[keysize+1];
		_tcscpy(processorName, strBuf);
	}
	else if (keystatus == ERROR_MORE_DATA)
	{
		TCHAR *buf = new TCHAR[keysize+1];
		// Get the actual string
		keystatus = RegQueryValueEx(key, ProcessorNameKey, NULL, &keytype, (BYTE *)buf, &keysize);
		if (keystatus == ERROR_SUCCESS)
		{
			if (processorName != NULL) delete[] processorName;
			processorName = buf;
		}
	}

	// Get an integer from the registry
	keysize = sizeof(int);
	if ((keystatus = RegQueryValueEx(key, BaudRateKey, NULL, &keytype,
										(BYTE *)&tInt, &keysize)) == ERROR_SUCCESS)
	{
		// Error check the key value
		if (tInt != CBR_38400 && tInt != CBR_115200)
			baudRate = defaultBaudRate;
		else
			baudRate = tInt;
	}

	// Get an integer from the registry
	keysize = sizeof(int);
	if ((keystatus = RegQueryValueEx(key, AsmDisplayKey, NULL, &keytype,
										(BYTE *)&tInt, &keysize)) == ERROR_SUCCESS)
	{
		// Error check the key value
		if (tInt < 0 || tInt > 1)
			asmView = defaultAsmView;
		else
			asmView = tInt;
	}

#if 0
	// Get an integer from the registry
	keysize = sizeof(int);
	if ((keystatus = RegQueryValueEx(key, "intVal", NULL, &keytype,
										(BYTE *)&tint, &keysize)) == ERROR_SUCCESS) {
		intVal = tint;
		// Error check the key value
		if (intVal < lowVal || intVal > highVal)
			intVal = defaultVal;
	}
#endif

	RegCloseKey(key);

	return true;
}

bool CDefaultsManager::Save()
{
	HKEY key;

	LONG keystatus = RegOpenKeyEx(HKEY_CURRENT_USER, EvaProgOptionKeys, 0, KEY_WRITE, &key);
	if (keystatus != ERROR_SUCCESS)
		// For some reason we were unable to open the registry
		return false;

	keystatus = RegSetValueEx(key, COMPortKey, NULL, REG_SZ, (BYTE *)portName, (DWORD)(_tcslen(portName)+1)*sizeof(TCHAR));
	keystatus = RegSetValueEx(key, ProcessorNameKey, NULL, REG_SZ, (BYTE *)processorName, (DWORD)(_tcslen(processorName)+1)*sizeof(TCHAR));
	keystatus = RegSetValueEx(key, BaudRateKey, NULL, REG_DWORD, (BYTE *)&baudRate, (DWORD)sizeof(DWORD));
	keystatus = RegSetValueEx(key, AsmDisplayKey, NULL, REG_DWORD, (BYTE *)&asmView, (DWORD)sizeof(DWORD));


 	RegCloseKey(key);

	return true;
}

void CDefaultsManager::setCOMPort(LPCTSTR newPortName)
{
	if (processorName == NULL)
	{
		// Set to default..
		portName = new TCHAR[_tcslen(defaultCOMPort)+1];
		_tcscpy(portName, defaultCOMPort);
	}
	else
	{
		size_t len = _tcslen(newPortName);
		TCHAR *newStr = new TCHAR[len+1];
		_tcscpy(newStr, newPortName);
		if (portName != NULL) delete[] portName;
		portName = newStr;
	}

	Save();
}

void CDefaultsManager::setProcessorName(LPCTSTR newProcName)
{
	if (processorName == NULL)
	{
		// Set to default..
		processorName = new TCHAR[_tcslen(defaultProcessorName)+1];
		_tcscpy(processorName, defaultProcessorName);
	}
	else
	{
		size_t len = _tcslen(newProcName);
		TCHAR *newStr = new TCHAR[len+1];
		_tcscpy(newStr, newProcName);
		if (processorName != NULL) delete[] processorName;
		processorName = newStr;
	}

	Save();
}

// Set the default Baud rate
void CDefaultsManager::setBaudRate(DWORD newBaudRate)
{
	baudRate = newBaudRate;
}

// Get the list of allowed Baud rates
defaultsIntList &CDefaultsManager::getAllowedBaudRates()
{
	return *allowedBaudRates;
}

// Set the default disassembly view
void CDefaultsManager::setAsmView(DWORD newAsmView)
{
	asmView = newAsmView;
}

// Get the list of allowed view codes
defaultsIntList &CDefaultsManager::getAllowedAsmViews()
{
	return *allowedAsmViews;
}

// Get the list of allowed view names (one for each allowed view code)
defaultsLabelList &CDefaultsManager::getAllowedAsmViewLabels()
{
	return *allowedAsmViewLabels;
}

