#pragma once

#include <list>

typedef std::list<DWORD> defaultsIntList;
typedef std::list<TCHAR *> defaultsLabelList;

// Read/Write default values
class CDefaultsManager
{
public:
	CDefaultsManager(void);
	~CDefaultsManager(void);

	// I/O routines to populate or write the defaults
	bool Load();
	bool Save();

	// Methods to modify properties

	// Set the default processor name
	void setProcessorName(LPCTSTR processorName);

	// Set the default COM port name
	void setCOMPort(LPCTSTR newPortName);

	// Set the default Baud rate
	void setBaudRate(DWORD newBaudRate);

	// Get the list of allowed Baud rates
	defaultsIntList &getAllowedBaudRates();

	// Set the default disassembly view
	void setAsmView(DWORD newAsmView);

	// Get the list of allowed view codes
	defaultsIntList &getAllowedAsmViews();

	// Get the list of allowed view names (one for each allowed view code)
	defaultsLabelList &getAllowedAsmViewLabels();

	// Default values.  Note that string values are destroyed by this
	// class' destructor.  If you change, make sure to allocate memory
	// for them so it doesn't get deallocated twice (or use property
	// methods above to do the allocation for you).
	TCHAR *portName;		// Last COM port used
	TCHAR *processorName;	// Last processor selected
	DWORD baudRate;         // 38400 or 115200
	DWORD asmView;          // 0 = asm only, 1 = full listing (hex addresses & instructions)

private:
	static TCHAR *EvaProgOptionKeys;
};
