//	Serial0.h - Simple RS232 Serial I/O class
//
// Support 8 bit communications only
//
#ifndef __SERIAL0_H
#define __SERIAL0_H

#include <list>
#include <windows.h>

class CSerial
{
public:
	CSerial();
	virtual ~CSerial();

// Operations
public:
	// Retrieve a list of COM ports by name
	static std::list<LPTSTR> *GetCOMPortList();

	// Open the COM port given by pPort.  (e.g., _T("COM1:"))
	HRESULT Open(LPCTSTR pDeviceName, int baudRate, int parity, int stopBits, bool handshake=false);

	// Close the port
	HRESULT Close (void);

	// Write data to the serial port
	BOOL Write(LPCVOID lpBuffer, DWORD bufLen, DWORD* numWritten = 0);
	// Write an ANSII string to the serial port (not UNICODE)
	BOOL Write(LPCSTR pString, DWORD* numWritten = 0);

	// Read data from the serial port.
	BOOL Read(LPVOID lpBuffer, DWORD bufLen, DWORD* numRead = 0);

	// Make sure all characters have been written
	BOOL Flush();

	// Make sure all characters have been written
	BOOL Purge();

	// Status
	bool IsOpen();

	// Obtain CTS/DSR/RING/RLSD settings
	bool GetCTS (void);
	bool GetDSR (void);
	bool GetRing (void);
	bool GetRLSD (void);

	// Set BRK/RTS/DTR
	void SetBRK(bool onState);
	void SetRTS(bool onState);
	void SetDTR(bool onState);

protected:
	HANDLE	m_hFile; // File handle associated with the COM port
	std::list<int> charBuffer;
	unsigned char tmpBuf[128];
};

#endif	// __SERIAL0_H
